/**********************************************************************************
*
* $Header: /cvs/sakai/chef-tool/src/java/org/sakaiproject/tool/email/MailboxAction.java,v 1.9 2004/06/22 03:05:00 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

// package
package org.sakaiproject.tool.email;

// imports
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.sakaiproject.cheftool.Context;
import org.sakaiproject.cheftool.JetspeedRunData;
import org.sakaiproject.cheftool.PagedResourceAction;
import org.sakaiproject.cheftool.PortletConfig;
import org.sakaiproject.cheftool.RunData;
import org.sakaiproject.cheftool.VelocityPortlet;
import org.sakaiproject.cheftool.menu.Menu;
import org.sakaiproject.cheftool.menu.MenuDivider;
import org.sakaiproject.cheftool.menu.MenuEntry;
import org.sakaiproject.exception.IdUnusedException;
import org.sakaiproject.exception.InUseException;
import org.sakaiproject.exception.PermissionException;
import org.sakaiproject.service.legacy.alias.cover.AliasService;
import org.sakaiproject.service.framework.config.cover.ServerConfigurationService;
import org.sakaiproject.service.legacy.content.cover.ContentTypeImageService;
import org.sakaiproject.service.legacy.email.MailArchiveChannel;
import org.sakaiproject.service.legacy.email.MailArchiveChannelEdit;
import org.sakaiproject.service.legacy.email.MailArchiveMessage;
import org.sakaiproject.service.legacy.email.cover.MailArchiveService;
import org.sakaiproject.service.legacy.message.MessageEdit;
import org.sakaiproject.service.framework.portal.cover.PortalService;
import org.sakaiproject.service.legacy.resource.Reference;
import org.sakaiproject.service.framework.session.cover.UsageSessionService;
import org.sakaiproject.service.legacy.site.ToolConfigurationEdit;
import org.sakaiproject.service.legacy.site.cover.SiteService;
import org.sakaiproject.service.framework.state.SessionState;
import org.sakaiproject.service.legacy.user.User;
import org.sakaiproject.util.Validator;
import org.sakaiproject.tool.helper.PermissionsAction;
import org.sakaiproject.util.StringUtil;
import org.sakaiproject.util.observer.EventObservingCourier;

/**
* <p>MailboxAction is a the CHEF mailbox tool.</p>
*
* @author University of Michigan, CHEF Software Development Team
* @version $Revision: 1.9 $
*/
public class MailboxAction
	extends PagedResourceAction
{
	/** portlet configuration parameter names. */
	private static final String PARAM_CHANNEL = "channel";
	private static final String PARAM_SITE = "site";

	/** Configure form field names. */
	private static final String FORM_CHANNEL = "channel";
	private static final String FORM_PAGESIZE = "pagesize";
	private static final String FORM_OPEN = "open";
	private static final String FORM_ALIAS = "alias";

	/** List request parameters. */
	private static final String VIEW_ID = "view-id";

	/** state attribute names. */
	private static final String STATE_CHANNEL_REF = "channelId";
	private static final String STATE_ASCENDING = "ascending";
	private static final String STATE_SORT = "sort";
	private static final String STATE_VIEW_HEADERS = "view-headers";
	private static final String STATE_OPTION_PAGESIZE = "optSize";
	private static final String STATE_OPTION_OPEN = "optOpen";
	private static final String STATE_OPTION_ALIAS = "optAlias";

	/** channel / message problem messages */
	private static final String CHANNEL_MISSING = "This mailbox is not available.";
	private static final String CHANNEL_PERMISSION = "You do not have permission to view this mailbox.";
	private static final String MESSAGE_MISSING = "This email message is not available.";
	private static final String MESSAGE_PERMISSION = "You do not have permission to view this email message.";

	/** Sort codes. */
	private static final int SORT_FROM = 0;
	private static final int SORT_DATE = 1;
	private static final int SORT_SUBJECT = 2;

	/**
	* Sort support for email messages.
	*/
	private class MyComparator
		implements Comparator
	{
		/** the criteria - a sort code. */
		private int m_criteria = SORT_DATE;
		
		/** True for ascending sort, false for descending. */
		private boolean m_asc = true;
		
		/**
		* constructor
		* @param criteria The sort criteria string
		* @param asc The sort order string. "true" if ascending; "false" otherwise.
		*/
		public MyComparator (int criteria, boolean asc)
		{
			m_criteria = criteria;
			m_asc = asc;

		}	// MyComparator
		
		/**
		* Compares its two arguments for order.
		* @param o1 The first object.
		* @param o2 The second object.
		* @return a a negative integer,  zero, or a positive integer
		* as the first argument is less than, equal  to, or greater than the second.
		*/
		public int compare (Object o1, Object o2)
		{
			// make these MailArchiveMessages - let the class cast exception be thrown if not
			MailArchiveMessage m1 = (MailArchiveMessage) o1;
			MailArchiveMessage m2 = (MailArchiveMessage) o2;

			// if these are the same object (or equal - it may be cheaper to just let the comparison find the equality)
			if ((m1 == m2) /*|| (m1.equals(m2)) */) return 0;

			// if it's ascending, use the compareTo value - if descending, reverse it
			int orderFactor = (m_asc ? 1 : -1);

			switch (m_criteria)
			{
				case SORT_FROM:
				{
					String addr1 = m1.getMailArchiveHeader().getFromAddress();
					String addr2 = m2.getMailArchiveHeader().getFromAddress();
					String rex = "^[^<>]*<[^<>@]*@[^<>@]*>[^<>@]*";
					if(addr1.matches(rex))
					{
						addr1 = addr1.substring(addr1.indexOf('<') + 1, addr1.indexOf('>'));
					}
					if(addr2.matches(rex))
					{
						addr2 = addr2.substring(addr2.indexOf('<') + 1, addr2.indexOf('>'));
					}
					return orderFactor * addr1.compareTo(addr2);
				}

				case SORT_DATE:
				{
					return orderFactor *
						m1.getMailArchiveHeader().getDateSent().compareTo(
							m2.getMailArchiveHeader().getDateSent());
				}

				case SORT_SUBJECT:
				{
					String subj1 = m1.getMailArchiveHeader().getSubject().toUpperCase();
					String subj2 = m2.getMailArchiveHeader().getSubject().toUpperCase();
					while(subj1.startsWith("RE:"))
					{
						subj1 = subj1.substring(subj1.indexOf(':') + 1).trim();
					}
					while(subj2.startsWith("RE:"))
					{
						subj2 = subj2.substring(subj2.indexOf(':') + 1).trim();
					}
					return orderFactor * subj1.compareTo(subj2);
				}
			}

			// trouble!
			Log.warn("chef", "MailboxAction.MyComparator - invalid sort: " + m_criteria);
			return 0;

		}	// compare
		
	}	// class MyComparator	

	/**
	* Implement this to return alist of all the resources that there are to page.
	* Sort them as appropriate.
	*/
	protected List readAllResources(SessionState state)
	{
		// read all channel messages
		List allMessages = null;
		try
		{
			MailArchiveChannel channel = MailArchiveService.getMailArchiveChannel(
					(String) state.getAttribute(STATE_CHANNEL_REF));
				
			allMessages = channel.getMessages(null, true);
		}
		catch (PermissionException e) {}
		catch (IdUnusedException e) {}

		// deal with no messages
		if (allMessages == null) return new Vector();

		String search = (String) state.getAttribute(STATE_SEARCH);
		
		// filter if searching
		if (search != null)
		{
			Vector filtered = new Vector();
			for (Iterator iMsgs = allMessages.iterator(); iMsgs.hasNext();)
			{
				MailArchiveMessage msg = (MailArchiveMessage) iMsgs.next();

				if (	StringUtil.containsIgnoreCase(msg.getMailArchiveHeader().getSubject(),search)
					||	StringUtil.containsIgnoreCase(msg.getMailArchiveHeader().getFromAddress(),search)
					||	StringUtil.containsIgnoreCase(msg.getBody(),search))
				{
					filtered.add(msg);
				}
			}
			
			allMessages = filtered;
		}

		// if other than ascending date, sort them all
		boolean ascending = ((Boolean) state.getAttribute(STATE_ASCENDING)).booleanValue();
		int sort = ((Integer) state.getAttribute(STATE_SORT)).intValue();
		if ((allMessages.size() > 1) && ((!ascending) || (sort != SORT_DATE)))
		{
			Collections.sort(allMessages, new MyComparator(sort, ascending));
		}

		return allMessages;

	}	// readAllResources

	/**
	* Populate the state object, if needed.
	* @param state The state object.
	* @param portlet The portlet.
	* @param rundata The current request's rundata.
	*/
	protected void initState(SessionState state, VelocityPortlet portlet, JetspeedRunData rundata)
	{
		super.initState(state, portlet, rundata);

		if (state.getAttribute(STATE_CHANNEL_REF) == null)
		{
			PortletConfig config = portlet.getPortletConfig();

			// start in list mode
			state.setAttribute(STATE_MODE, "list");
			
			// read the channel from configuration, or, if not specified, use the default for the request
			String channel = StringUtil.trimToNull(config.getInitParameter(PARAM_CHANNEL));
			if (channel == null)
			{
				channel = MailArchiveService.channelReference(PortalService.getCurrentSiteId(), SiteService.MAIN_CONTAINER);
			}
			state.setAttribute(STATE_CHANNEL_REF, channel);

			if (state.getAttribute(STATE_ASCENDING) == null)
			{
				state.setAttribute(STATE_ASCENDING, new Boolean(false));
			}
			
			if (state.getAttribute(STATE_SORT) == null)
			{
				state.setAttribute(STATE_SORT, new Integer(SORT_DATE));
			}

			if (state.getAttribute(STATE_VIEW_HEADERS) == null)
			{
				state.setAttribute(STATE_VIEW_HEADERS, new Boolean(false));
			}

			// setup the observer to notify our main panel
			if (state.getAttribute(STATE_OBSERVER) == null)
			{
				// the delivery location for this tool
				String deliveryId = clientWindowId(state, portlet.getID());
				
				// the html element to update on delivery
				String elementId = mainPanelUpdateId(portlet.getID());
				
				// the event resource reference pattern to watch for
				Reference r = new Reference(channel);
				String pattern = MailArchiveService.messageReference(r.getContext(), r.getId(), "");

				state.setAttribute(STATE_OBSERVER, new EventObservingCourier(deliveryId, elementId, pattern));
			}
		}

		// make sure the observer is in sync with state
		updateObservationOfChannel(state, portlet.getID());

	}   // initState

	/**
	* Setup our observer to be watching for change events for our channel.
	* @param state The state object.
	* @param peid The portlet id.
	*/
	private void updateObservationOfChannel(SessionState state, String peid)
	{
		EventObservingCourier observer = (EventObservingCourier) state.getAttribute(STATE_OBSERVER);

		// make sure the pattern matches the channel we are looking at
		String channel = (String) state.getAttribute(STATE_CHANNEL_REF);
		Reference r = new Reference(channel);
		String pattern = MailArchiveService.messageReference(r.getContext(), r.getId(), "");
		observer.setResourcePattern(pattern);

		// the delivery location for this tool
		String deliveryId = clientWindowId(state, peid);
		observer.setDeliveryId(deliveryId);
		
	}   // updateObservationOfChannel

	/** 
	* build the context for the main panel
	* @return (optional) template name for this panel
	*/
	public String buildMainPanelContext(VelocityPortlet portlet, 
										Context context,
										RunData rundata,
										SessionState state)
	{
		// if we are in edit permissions...
		String helperMode = (String) state.getAttribute(PermissionsAction.STATE_MODE);
		if (helperMode != null)
		{
			String template = PermissionsAction.buildHelperContext(portlet, context, rundata, state);
			if (template == null)
			{
				addAlert(state, "There is a problem setting Permissions at this time.\n");
			}
			else
			{
				return template;
			}
		}
		
		String mode = (String) state.getAttribute(STATE_MODE);

		context.put(Menu.CONTEXT_ACTION, state.getAttribute(STATE_ACTION));

		if ("list".equals(mode))
		{
			return buildListModeContext(portlet, context, rundata, state);
		}
		
		else if ("confirm-remove".equals(mode))
		{
			return buildConfirmModeContext(portlet, context, rundata, state);
		}
		
		else if ("view".equals(mode))
		{
			return buildViewModeContext(portlet, context, rundata, state);
		}

		else if (MODE_OPTIONS.equals(mode))
		{
			return buildOptionsPanelContext(portlet, context, rundata, state);
		}

		else
		{
			Log.warn("chef", this + ".buildMainPanelContext: invalid mode: " + mode);
			return null;
		}

	}   // buildMainPanelContext

	/** 
	* build the context for the View mode (in the Main panel)
	* @return (optional) template name for this panel
	*/
	private String buildViewModeContext(VelocityPortlet portlet, 
										Context context,
										RunData rundata,
										SessionState state)
	{
		boolean allowDelete = false;

		// prepare the sort of messages
		prepPage(state);

		// get the message
		MailArchiveMessage message = null;
		try
		{
			MailArchiveChannel channel = MailArchiveService.getMailArchiveChannel(
					(String) state.getAttribute(STATE_CHANNEL_REF));
			message = channel.getMailArchiveMessage((String) state.getAttribute(STATE_VIEW_ID));
			context.put("email", message);

			allowDelete = channel.allowRemoveMessage(message);
		}
		catch (IdUnusedException e) {}
		catch (PermissionException e) {}
		if (message == null)
		{
			context.put("message", MESSAGE_MISSING);
		}

		context.put("viewheaders", state.getAttribute(STATE_VIEW_HEADERS));

		context.put("contentTypeImageService", ContentTypeImageService.getInstance());

		// build the menu
		Menu bar = new Menu(portlet, rundata, (String) state.getAttribute(STATE_ACTION));
		bar.add( new MenuEntry("List All", "doList"));
		addViewPagingMenus(bar, state);
		if (((Boolean)state.getAttribute(STATE_VIEW_HEADERS)).booleanValue())
		{
			bar.add( new MenuEntry("Hide Headers", "doHide_headers"));
		}
		else
		{
			bar.add( new MenuEntry("View Headers", "doView_headers"));
		}
		if (allowDelete) bar.add( new MenuEntry("Remove", "doRemove"));

		// make sure there's not leading or trailing dividers		
		bar.adjustDividers();

		context.put(Menu.CONTEXT_MENU, bar);

		return (String)getContext(rundata).get("template") + "-view";

	}   // buildViewModeContext

	/**
	* Build the context for the confirm remove mode (in the Main panel).
	*/
	private String buildConfirmModeContext(VelocityPortlet portlet, 
										Context context,
										RunData rundata,
										SessionState state)
	{
		// get the message
		MailArchiveMessage message = null;
		try
		{
			MailArchiveChannel channel = MailArchiveService.getMailArchiveChannel(
					(String) state.getAttribute(STATE_CHANNEL_REF));
			message =
				channel.getMailArchiveMessage((String) state.getAttribute(STATE_VIEW_ID));
			context.put("email", message);
		}
		catch (IdUnusedException e) {}
		catch (PermissionException e) {}

		if (message == null)
		{
			context.put("message", MESSAGE_MISSING);
		}

		context.put("viewheaders", state.getAttribute(STATE_VIEW_HEADERS));

		return (String)getContext(rundata).get("template") + "-confirm_remove";

	}	// buildConfirmModeContext

	/** 
	* build the context for the list mode (in the Main panel).
	*/
	private String buildListModeContext(VelocityPortlet portlet, 
										Context context,
										RunData rundata,
										SessionState state)
	{
		// prepare the page of messages
		List messages = prepPage(state);
		context.put("messages", messages);

		// build the menu
		Menu bar = new Menu(portlet, rundata, (String) state.getAttribute(STATE_ACTION));

		// add paging commands
		addListPagingMenus(bar, state);

		// add the search commands
		addSearchMenus(bar, state);

		// add the refresh commands
		addRefreshMenus(bar, state);

		if (SiteService.allowUpdateSite(PortalService.getCurrentSiteId()))
		{
			bar.add( new MenuDivider());

			// add options if allowed
			addOptionsMenu(bar, (JetspeedRunData) rundata);

			bar.add( new MenuEntry("Permissions...", "doPermissions") );
		}

		// make sure there's not leading or trailing dividers		
		bar.adjustDividers();

		context.put(Menu.CONTEXT_MENU, bar);

		if (state.getAttribute(STATE_NUM_MESSAGES) != null)
		{
			context.put("num-messages", state.getAttribute(STATE_NUM_MESSAGES));
		}

		// eventSubmit value and id field for drill down
		context.put("view-id", VIEW_ID);

		context.put(Menu.CONTEXT_ACTION, state.getAttribute(STATE_ACTION));

		context.put("sort-by", state.getAttribute(STATE_SORT));
		context.put("sort-order", state.getAttribute(STATE_ASCENDING));

		// the aliases for the channel
		List all = AliasService.getAliases((String) state.getAttribute(STATE_CHANNEL_REF));

		// and the aliases for the site (context)
		Reference channelRef = new Reference((String) state.getAttribute(STATE_CHANNEL_REF));
		String siteRef = SiteService.siteReference(channelRef.getContext());
		all.addAll(AliasService.getAliases(siteRef));

		// and the site id
		all.add(channelRef.getContext());

		context.put("aliases", all);
		context.put("serverName", ServerConfigurationService.getServerName());
		
		// if the user has permission to send mail, drop in the email address
		try
		{
			MailArchiveChannel channel = MailArchiveService.getMailArchiveChannel(
					(String) state.getAttribute(STATE_CHANNEL_REF));
			if (channel.getEnabled())
			{
				if (channel.getOpen())
				{
					// if open, mail from anywhere
					context.put("validFrom","*");
				}
				else if (channel.allowAddMessage())
				{
					User user = UsageSessionService.getSessionUser();
					String email = user.getEmail();
					context.put("validFrom", email);
				}
			}
		}
		catch (Exception e) {}

		// inform the observing courier that we just updated the page...
		// if there are pending requests to do so they can be cleared
		((EventObservingCourier) state.getAttribute(STATE_OBSERVER)).justDelivered();

		return (String)getContext(rundata).get("template") + "-List";

	}   // buildListModeContext

	/** 
	* Handle a user drill down request.
	**/	
	public void doView(RunData runData, Context context)
	{
		// access the portlet element id to find our state
		String peid = ((JetspeedRunData)runData).getJs_peid();
		SessionState state = ((JetspeedRunData)runData).getPortletSessionState(peid);

		// switch to view mode
		state.setAttribute(STATE_MODE, "view");

		// disable auto-updates while in view mode
		((EventObservingCourier) state.getAttribute(STATE_OBSERVER)).disable();

		// set the message id
		state.setAttribute(STATE_VIEW_ID, runData.getParameters().getString(VIEW_ID));

	}	// doView

	/** 
	* Handle a return-to-list-view request.
	**/	
	public void doList(RunData runData, Context context)
	{
		// access the portlet element id to find our state
		String peid = ((JetspeedRunData)runData).getJs_peid();
		SessionState state = ((JetspeedRunData)runData).getPortletSessionState(peid);

		// switch to view mode
		state.setAttribute(STATE_MODE, "list");

		// make sure auto-updates are enabled
		enableObserver(state);

		// cleanup
		state.removeAttribute(STATE_VIEW_ID);

	}	// doList

	/** 
	* Handle a view headers request.
	**/	
	public void doView_headers(RunData runData, Context context)
	{
		// access the portlet element id to find our state
		String peid = ((JetspeedRunData)runData).getJs_peid();
		SessionState state = ((JetspeedRunData)runData).getPortletSessionState(peid);

		// switch to view mode
		state.setAttribute(STATE_VIEW_HEADERS, new Boolean(true));

	}	// doView_headers

	/** 
	* Handle a hide headers request.
	**/	
	public void doHide_headers(RunData runData, Context context)
	{
		// access the portlet element id to find our state
		String peid = ((JetspeedRunData)runData).getJs_peid();
		SessionState state = ((JetspeedRunData)runData).getPortletSessionState(peid);

		// switch to view mode
		state.setAttribute(STATE_VIEW_HEADERS, new Boolean(false));

	}	// doHide_headers

	/** 
	* Handle a user request to change the sort to "from"
	**/	
	public void doSort_from(RunData runData, Context context)
	{
		// access the portlet element id to find our state
		String peid = ((JetspeedRunData)runData).getJs_peid();
		SessionState state = ((JetspeedRunData)runData).getPortletSessionState(peid);

		// we are changing the sort, so start from the first page again
		resetPaging(state);

		// if already from, swap the order
		if (((Integer) state.getAttribute(STATE_SORT)).intValue() == SORT_FROM)
		{
			boolean order = !((Boolean) state.getAttribute(STATE_ASCENDING)).booleanValue();
			state.setAttribute(STATE_ASCENDING, new Boolean(order));
		}

		// set state
		else
		{
			state.setAttribute(STATE_SORT, new Integer(SORT_FROM));
		}

	}	// doSort_from

	/** 
	* Handle a user request to change the sort to "date"
	**/	
	public void doSort_date(RunData runData, Context context)
	{
		// access the portlet element id to find our state
		String peid = ((JetspeedRunData)runData).getJs_peid();
		SessionState state = ((JetspeedRunData)runData).getPortletSessionState(peid);

		// we are changing the sort, so start from the first page again
		resetPaging(state);

		// if already date, swap the order
		if (((Integer) state.getAttribute(STATE_SORT)).intValue() == SORT_DATE)
		{
			boolean order = !((Boolean) state.getAttribute(STATE_ASCENDING)).booleanValue();
			state.setAttribute(STATE_ASCENDING, new Boolean(order));
		}

		// set state
		else
		{
			state.setAttribute(STATE_SORT, new Integer(SORT_DATE));
		}

	}	// doSort_date

	/** 
	* Handle a user request to change the sort to "subject"
	**/	
	public void doSort_subject(RunData runData, Context context)
	{
		// access the portlet element id to find our state
		String peid = ((JetspeedRunData)runData).getJs_peid();
		SessionState state = ((JetspeedRunData)runData).getPortletSessionState(peid);

		// we are changing the sort, so start from the first page again
		resetPaging(state);

		// if already subject, swap the order
		if (((Integer) state.getAttribute(STATE_SORT)).intValue() == SORT_SUBJECT)
		{
			boolean order = !((Boolean) state.getAttribute(STATE_ASCENDING)).booleanValue();
			state.setAttribute(STATE_ASCENDING, new Boolean(order));
		}

		// set state
		else
		{
			state.setAttribute(STATE_SORT, new Integer(SORT_SUBJECT));
		}

	}	// doSort_subject

	/**
	* doRemove called when "eventSubmit_doRemove" is in the request parameters to confirm removal of the group
	*/
	public void doRemove(RunData data, Context context)
	{
		// access the portlet element id to find our state
		String peid = ((JetspeedRunData)data).getJs_peid();
		SessionState state = ((JetspeedRunData)data).getPortletSessionState(peid);

		// go to remove confirm mode
		state.setAttribute(STATE_MODE, "confirm-remove");

		// disable auto-updates while in confirm mode
		((EventObservingCourier) state.getAttribute(STATE_OBSERVER)).disable();

	}	// doRemove

	/**
	* doRemove_confirmed called when "eventSubmit_doRemove_confirmed" is in the request parameters to remove the group
	*/
	public void doRemove_confirmed(RunData data, Context context)
	{
		// access the portlet element id to find our state
		String peid = ((JetspeedRunData)data).getJs_peid();
		SessionState state = ((JetspeedRunData)data).getPortletSessionState(peid);

		// remove
		try
		{
			MailArchiveChannel channel = MailArchiveService.getMailArchiveChannel(
					(String) state.getAttribute(STATE_CHANNEL_REF));

			MessageEdit edit = channel.editMessage((String) state.getAttribute(STATE_VIEW_ID));
			channel.removeMessage(edit);
		}
		catch (PermissionException e)
		{
			addAlert(state, "You do not have permission to remove this message.\n");
		}
		catch (InUseException e)
		{
			addAlert(state, "Someone else is editing this message.\n");
		}
		catch (IdUnusedException e)
		{
			addAlert(state, "This message has already been removed.\n");
		}

		// go to list mode
		doList(data, context);

	}	// doRemove_confirmed

	/**
	* doCancel_remove called when "eventSubmit_doCancel_remove" is in the request parameters to cancel group removal
	*/
	public void doRemove_cancel(RunData data, Context context)
	{
		// access the portlet element id to find our state
		String peid = ((JetspeedRunData)data).getJs_peid();
		SessionState state = ((JetspeedRunData)data).getPortletSessionState(peid);

		// return to view mode
		state.setAttribute(STATE_MODE, "view");

		// disable auto-updates while in view mode
		((EventObservingCourier) state.getAttribute(STATE_OBSERVER)).disable();

	}	// doRemove_cancel

	/** 
	* Handle a request to set options.
	*/	
	public void doOptions(RunData runData, Context context)
	{
		super.doOptions(runData, context);

		// access the portlet element id to find our state
		String peid = ((JetspeedRunData)runData).getJs_peid();
		SessionState state = ((JetspeedRunData)runData).getPortletSessionState(peid);

		// if we ended up in options mode, do whatever else ...
		if (!MODE_OPTIONS.equals(state.getAttribute(STATE_MODE))) return;

	}	// doOptions

	/** 
	* Setup for options.
	*/
	public String buildOptionsPanelContext( VelocityPortlet portlet, 
											Context context,
											RunData rundata,
											SessionState state)
	{
		// provide "pagesize" with the current page size setting
		context.put("pagesize", ((Integer) state.getAttribute(STATE_PAGESIZE)).toString());

		// provide form names
		context.put("form-pagesize", FORM_PAGESIZE);
		context.put("form-open", FORM_OPEN);
		context.put("form-alias", FORM_ALIAS);
		context.put("form-submit", BUTTON + "doUpdate");
		context.put("form-cancel", BUTTON + "doCancel");

		// in progress values
		if (state.getAttribute(STATE_OPTION_PAGESIZE) != null)
			context.put(STATE_OPTION_PAGESIZE, state.getAttribute(STATE_OPTION_PAGESIZE));
		if (state.getAttribute(STATE_OPTION_OPEN) != null)
			context.put(STATE_OPTION_OPEN, state.getAttribute(STATE_OPTION_OPEN));
		if (state.getAttribute(STATE_OPTION_ALIAS) != null)
			context.put(STATE_OPTION_ALIAS, state.getAttribute(STATE_OPTION_ALIAS));

		// provide the channel
		try
		{
			MailArchiveChannel channel = MailArchiveService.getMailArchiveChannel(
					(String) state.getAttribute(STATE_CHANNEL_REF));
			context.put("channel", channel);
		}
		catch (Exception ignore) {}

		// place the current alias, if any, in to context
		List all = AliasService.getAliases((String) state.getAttribute(STATE_CHANNEL_REF));
		if (!all.isEmpty()) context.put("alias", all.get(0));

		// pick the "-customize" template based on the standard template name
		String template = (String)getContext(rundata).get("template");
		return template + "-customize";

	}   // buildOptionsPanelContext

	/**
	* doUpdate called for form input tags type="submit" named="eventSubmit_doUpdate"
	* update/save from the options process
	*/
	public void doUpdate(RunData data, Context context)
	{
		// access the portlet element id to find our state
		String peid = ((JetspeedRunData)data).getJs_peid();
		SessionState state = ((JetspeedRunData)data).getPortletSessionState(peid);

		// collect & save in state (for possible form re-draw)
		String pagesize = StringUtil.trimToZero(data.getParameters().getString(FORM_PAGESIZE));
		state.setAttribute(STATE_OPTION_PAGESIZE, pagesize);
		String open = data.getParameters().getString(FORM_OPEN);
		state.setAttribute(STATE_OPTION_OPEN, open);
		String alias = StringUtil.trimToNull(data.getParameters().getString(FORM_ALIAS));
		state.setAttribute(STATE_OPTION_ALIAS, alias);

		MailArchiveChannel channel = null;
		try
		{
			channel = MailArchiveService.getMailArchiveChannel((String) state.getAttribute(STATE_CHANNEL_REF));
		}
		catch (Exception e)
		{
			addAlert(state, "Cannot find the Email Archive Channel!\n");
		}

		// first validate - page size
		int size = 0;
		try
		{
			size = Integer.parseInt(pagesize);
			if (size <= 0)
			{
				addAlert(state,"Page size must be a positive number.\n");
			}
		}
		catch (Exception any)
		{
			addAlert(state, "Page size must be a positive number.\n");
		}

		// validate the email alias
		if (alias != null)
		{
			if (!Validator.checkEmailLocal(alias))
			{
				addAlert(state, "The Email Alias must be made up of alpha numeric characters or any of !#$&'*+-=?^_`{|}~. (no spaces).\n");
			}
		}

		// make sure we can get to the channel
		MailArchiveChannelEdit edit = null;
		try
		{
			edit = (MailArchiveChannelEdit) MailArchiveService.editChannel(channel.getReference());
		}
		catch (Exception any)
		{
			addAlert(state, "The Email Archive Channel may not be modified at this time.\n");
		}

		// if all is well, save
		if (state.getAttribute(STATE_MESSAGE) == null)
		{
			// get any current alias for this channel
			List all = AliasService.getAliases((String) state.getAttribute(STATE_CHANNEL_REF));
			String curAlias = null;
			if (!all.isEmpty()) curAlias = (String) all.get(0);
	
			// alias from the form
			if (StringUtil.different(curAlias, alias))
			{
				boolean ok = false;

				// see if this alias exists
				if (alias != null)
				{
					try
					{
						String target = AliasService.getTarget(alias);
						
						// if so, is it this channel?
						ok = target.equals(channel.getReference());
					}
					catch (IdUnusedException e)
					{
						// not in use
						ok = true;
					}
				}
				else
				{
					// no alias is desired
					ok = true;
				}
				
				if (ok)
				{
					try
					{
						// first, clear any alias set to this channel				
						AliasService.removeTargetAliases(channel.getReference());
		
						// then add the desired alias
						if (alias != null)
						{
							AliasService.setAlias(alias, channel.getReference());
						}
					}
					catch (Exception any)
					{
						addAlert(state, "The Email Alias cannot be changed at this time.\n");
					}
				}
				else
				{
					addAlert(state, "The Email Alias is otherwise in use.\n");
				}
			}

			// if the alias saving went well, go on to the rest
			if (state.getAttribute(STATE_MESSAGE) == null)
			{
				// update the channel for open (if changed)
				boolean ss = new Boolean(open).booleanValue();
				if (channel.getOpen() != ss)
				{
					edit.setOpen(ss);
					MailArchiveService.commitChannel(edit);
				}
				else
				{
					MailArchiveService.cancelChannel(edit);
				}
				edit = null;
	
				// update the tool config & state for page size (if different)
				if (size != ((Integer) state.getAttribute(STATE_PAGESIZE)).intValue())
				{
					state.setAttribute(STATE_PAGESIZE, new Integer(size));
					ToolConfigurationEdit tool = (ToolConfigurationEdit) state.getAttribute(STATE_TOOL_EDIT);
					tool.getPropertiesEdit().addProperty(PARAM_PAGESIZE, Integer.toString(size));
		
					// commit the options change
					saveOptions();
				}
				else
				{
					cancelOptions();
				}

				// we are done with customization... back to the main (list) mode
				state.setAttribute(STATE_MODE, "list");

				// clear state temps.
				state.removeAttribute(STATE_OPTION_PAGESIZE);
				state.removeAttribute(STATE_OPTION_OPEN);
				state.removeAttribute(STATE_OPTION_ALIAS);

				// re-enable auto-updates when going back to list mode
				enableObserver(state);
			}
		}

		// before leaving, make sure the edit was cleared
		if (edit != null)
		{
			MailArchiveService.cancelChannel(edit);
			edit = null;
		}

	}   // doUpdate

	/**
	* doCancel called for form input tags type="submit" named="eventSubmit_doCancel"
	* cancel the options process
	*/
	public void doCancel(RunData data, Context context)
	{
		// access the portlet element id to find our state
		String peid = ((JetspeedRunData)data).getJs_peid();
		SessionState state = ((JetspeedRunData)data).getPortletSessionState(peid);

		// cancel the options
		cancelOptions();

		// we are done with customization... back to the main (list) mode
		state.setAttribute(STATE_MODE, "list");

		// clear state temps.
		state.removeAttribute(STATE_OPTION_PAGESIZE);
		state.removeAttribute(STATE_OPTION_OPEN);
		state.removeAttribute(STATE_OPTION_ALIAS);

		// re-enable auto-updates when going back to list mode
		enableObserver(state);

	}   // doCancel

	/**
	* Fire up the permissions editor
	*/
	public void doPermissions(RunData data, Context context)
	{
		SessionState state = ((JetspeedRunData)data).getPortletSessionState(((JetspeedRunData)data).getJs_peid());

		String channelRefStr = (String) state.getAttribute(STATE_CHANNEL_REF);
		Reference channelRef = new Reference(channelRefStr);
		String siteRef = SiteService.siteReference(channelRef.getContext());

		// setup for editing the permissions of the site for this tool, using the roles of this site, too
		state.setAttribute(PermissionsAction.STATE_REALM_ID, siteRef);
		state.setAttribute(PermissionsAction.STATE_REALM_ROLES_ID, siteRef);

		// ... with this description
		state.setAttribute(PermissionsAction.STATE_DESCRIPTION, "Set permissions for Email Archive in worksite "
				+ SiteService.getSiteDisplay(channelRef.getContext()));

		// ... showing only locks that are prpefixed with this
		state.setAttribute(PermissionsAction.STATE_PREFIX, "mail.");

		// start the helper
		state.setAttribute(PermissionsAction.STATE_MODE, PermissionsAction.MODE_MAIN);

	}	// doPermissions

}   // MailboxAction

/**********************************************************************************
*
* $Header: /cvs/sakai/chef-tool/src/java/org/sakaiproject/tool/email/MailboxAction.java,v 1.9 2004/06/22 03:05:00 ggolden Exp $
*
**********************************************************************************/
