/**********************************************************************************
*
* $Header: /cvs/sakai/chef-tool/src/java/org/sakaiproject/util/MergedList.java,v 1.6 2004/06/22 03:04:59 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/
package org.sakaiproject.util;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.sakaiproject.cheftool.PortletConfig;
import org.sakaiproject.cheftool.RunData;
import org.sakaiproject.cheftool.VelocityPortlet;
import org.sakaiproject.exception.IdUnusedException;
import org.sakaiproject.service.framework.log.cover.Log;
import org.sakaiproject.service.framework.portal.cover.PortalService;
import org.sakaiproject.service.legacy.resource.ResourceProperties;
import org.sakaiproject.service.legacy.security.cover.SecurityService;
import org.sakaiproject.service.framework.session.cover.UsageSessionService;
import org.sakaiproject.service.legacy.site.Site;
import org.sakaiproject.service.legacy.site.cover.SiteService;

/**
 * Contains the list of merged/non-merged channels
 */
public class MergedList extends ArrayList
{
	/**
	 * channel entry used to communicate with the Velocity templates when dealing with merged channels.
	 */
	public interface MergedEntry extends Comparable
	{
		/**
		 * Returns the display string for the channel.
		 * @return
		 */
		public String getDisplayName();

		/**
		 * Returns the ID of the group.  (The ID is used as a key.)
		 * @return
		 */
		public String getReference();

		/**
		 * Returns true if this channel is currently being merged.
		 * @return
		 */
		public boolean isMerged();

		/**
		 * Marks this channel as being merged or not.
		 * @param b
		 */
		public void setMerged(boolean b);

		/**
		 * This returns true if this list item should be visible to the user.
		 * @return
		 */
		public boolean isVisible();

		/**
		 * Implemented so that we can order by the group full name.
		 */
		public int compareTo(Object arg0);
	}

	/**
	 * This interface is used to describe a generic list entry provider so that
	 * a variety of list entries can be used.  This currently serves merged sites
	 * for the schedule and merged channels for announcements.
	 */
	public interface EntryProvider
	{
		/**
		 * Gets an iterator for the channels, calendars, etc.
		 * @return
		 */
		public Iterator getIterator();
		
		/**
		 * See if we can do a "get" on the calendar, channel, etc.
		 * @param obj
		 * @return
		 */
		public boolean allowGet(String ref);
		
		/**
		 * Generically access the context of the resource provided
		 * by the getIterator() call.
		 * @return The context.
		 */
		public String getContext(Object obj);
		
		/**
		 * Generically access the reference of the resource provided
		 * by the getIterator() call.
		 * @param obj
		 * @return
		 */
		public String getReference(Object obj);
		
		/**
		 * Generically access the resource's properties.
		 * @return The resource's properties.
		 */
		public ResourceProperties getProperties(Object obj);
	}

	/** This is used to separate group names in the config parameter. */
	static private final String ID_DELIMITER = "_,_";

	/**
	 * Implementation of channel entry used for rendering the list of merged channels
	 */
	private class MergedChannelEntryImpl implements MergedEntry
	{
		final private String channelReference;
		final private String channelFullName;
		private boolean merged;
		private boolean visible;

		/**
		 * @param channelReference
		 * @param channelFullName
		 * @param merged
		 * @param visible
		 */
		public MergedChannelEntryImpl(
			String channelReference,
			String channelFullName,
			boolean merged,
			boolean visible)
		{
			this.channelReference = channelReference;
			this.channelFullName = channelFullName;
			this.merged = merged;
			this.visible = visible;
		}

		/* (non-Javadoc)
		 * @see org.chefproject.actions.channelAction.MergedCalenderEntry#getchannelDisplayName()
		 */
		public String getDisplayName()
		{
			return channelFullName;
		}

		/* (non-Javadoc)
		 * @see org.chefproject.actions.channelAction.MergedCalenderEntry#getchannelReference()
		 */
		public String getReference()
		{
			return channelReference;
		}

		/* (non-Javadoc)
		 * @see org.chefproject.actions.channelAction.MergedCalenderEntry#isMerged()
		 */
		public boolean isMerged()
		{
			return merged;
		}

		/* (non-Javadoc)
		 * @see org.chefproject.actions.channelAction.MergedCalenderEntry#setMerged(boolean)
		 */
		public void setMerged(boolean b)
		{
			merged = b;
		}

		/* (non-Javadoc)
		 * @see java.lang.Comparable#compareTo(java.lang.Object)
		 */
		public int compareTo(Object arg0)
		{
			MergedChannelEntryImpl compObj = (MergedChannelEntryImpl) arg0;

			return this.getDisplayName().compareTo(compObj.getDisplayName());
		}

		/* (non-Javadoc)
		 * @see org.chefproject.actions.channelAction.MergedCalenderEntry#isVisible()
		 */
		public boolean isVisible()
		{
			return visible;
		}
	}
	
	/**
	 * Prepares the context for the merge channels feature of the option button.
	 * @param portlet
	 * @param context
	 */
	public void loadFromPortletConfig(
		VelocityPortlet portlet,
		String primarychannelReference,
		boolean isOnWorkspaceTab,
		String portlentConfigParameterName,
		EntryProvider entryProvider)
	{
		// Remove any initial list contents.
		this.clear();

		Map currentlyMergedchannels =
			getChannelsFromConfig(
				portlet,
				primarychannelReference,
				portlentConfigParameterName);

		// Loop through all the channels, not just the ones the
		// user can access.
		Iterator it = entryProvider.getIterator();

		while (it.hasNext())
		{
			Object channel = it.next();

			// If true, this channel will be added to the list of
			// channels that may be merged.
			boolean addThisChannel = false;

			// If true, this channel will be marked as "merged".
			boolean merged = false;

			// If true, then this channel will be in the list, but will not
			// be shown to the user.
			boolean hidden = false;

			// If true, this is the channel associated with the current
			// user.
			boolean thisIsTheUsersMyWorkspaceChannel = false;

			// If true, this is a user channel.
			boolean thisIsUserChannel =
				SiteService.isUserSite(entryProvider.getContext(channel));

			// If true, this is a "special" site.
			boolean isSpecialSite =
				SiteService.isSpecialSite(entryProvider.getContext(channel));

			if (thisIsUserChannel)
			{
				String channelUser =
					SiteService.getSiteUserId(
						entryProvider.getContext(channel));
				if (UsageSessionService
					.getSessionUser()
					.getId()
					.equals(channelUser))
				{
					thisIsTheUsersMyWorkspaceChannel = true;
				}
			}

			//
			// Don't put the channels of other users in the merge list.
			// Go to the next item in the loop.
			//
			if (thisIsUserChannel && !thisIsTheUsersMyWorkspaceChannel)
			{
				continue;
			}

			// Only add to the list if the user can access this channel.
			if (entryProvider.allowGet(entryProvider.getReference(channel)))
			{
				// Merge *almost* everything the user can access.
				if (thisIsTheUsersMyWorkspaceChannel)
				{
					// Don't merge the user's channel in with a
					// group channel.  If we're on the "MyWorkspace"
					// tab, then it's okay to merge.
					if (isOnWorkspaceTab)
					{
						merged = true;
					}
					else
					{
						merged = false;
					}
				}
				else
				{
					//
					// If we're the admin, and we're on our "MyWorkspace" tab, then only
					// use our channel (handled above).  We'd be overloaded if we could
					// see everyone's events.
					//
					if (SecurityService.isSuperUser() && isOnWorkspaceTab)
					{
						merged = false;
					}
					else
					{
						// Set it to merged if the channel was specified in the merged
						// channel list that we got from the portlet configuration.
						if (isOnWorkspaceTab)
						{
							merged = true;
						}
						else
						{
							merged =
								currentlyMergedchannels.containsKey(
									entryProvider.getReference(channel));
						}
					}
				}

				addThisChannel = true;

				// Hide user or "special" sites from the user interface merge list.
				if (thisIsUserChannel || isSpecialSite)
				{
					// Hide the user's own channel from them.
					hidden = true;
				}
			}

			if (addThisChannel)
			{
				String siteDisplayName = "";
				
				// There is no point in getting the display name for hidden
				// items.
				if ( !hidden )
				{
					String displayNameProperty =
					entryProvider.getProperties(channel).getProperty(
						entryProvider
							.getProperties(channel)
							.getNamePropDisplayName());

					// If the channel has a displayName property and use that instead.
					if (displayNameProperty != null
						&& displayNameProperty.length() != 0)
					{
						siteDisplayName = displayNameProperty;
					}
					else
					{
						String channelName = "";
	
						try
						{
							Site site =
								SiteService.getSite(
									entryProvider.getContext(channel));
	
							if (site != null)
							{
								boolean isCurrentSite =
									PortalService.getCurrentSiteId().equals(
										site.getId());
	
								//
								// Hide and force the current site to be merged.
								//
								if (isCurrentSite)
								{
									hidden = true;
									merged = true;
								}
								else
								{
									// Else just get the name.
									channelName = site.getTitle();
									siteDisplayName =
										channelName
											+ " ("
											+ site.getId()
                                                                                        + ") ";
											//+ ") site's channel";
                                                                                        // site display name format changed by Joanne
								}
							}
						}
						catch (IdUnusedException e1)
						{
							// No site - chances are that this was deleted.
							// Continue the next iteration of the loop.
							continue;
						}
					}
				}

				this.add(
					new MergedChannelEntryImpl(
						entryProvider.getReference(channel),
						siteDisplayName,
						merged,
						!hidden));
			}
		}

		// MergedchannelEntry implements Comparable, so the sort will work correctly.
		Collections.sort(this);
	} // loadFromPortletConfig
	
	/**
	 * This gets a list of channels from the portlet configuration information.
	 * Channels here can really be a channel or a schedule from a site.
	 * @param portlet
	 * @param primarychannelReference
	 * @param portlentConfigParameterName
	 * @return
	 */
	private Map getChannelsFromConfig(
		VelocityPortlet portlet,
		String primarychannelReference,
		String portlentConfigParameterName)
	{
		PortletConfig portletConfig = portlet.getPortletConfig();

		if (portletConfig == null)
		{
			if (Log.getLogger("chef").isDebugEnabled())
			{
				Log.debug(
					"chef",
					this +".initContextForMergeOptions portletConfig == null");
			}
		}

		String mergedChannels = null;

		// Get a list of the currently merged channels.  This is a comma-delimited list.
		if ( portletConfig != null )
		{
			mergedChannels =
				StringUtil.trimToNull(
					portletConfig.getInitParameter(
						portlentConfigParameterName));
		}

		String[] mergedChannelArray = null;

		// Split the configuration string into an array of channel references.
		if (mergedChannels != null)
		{
			mergedChannelArray = mergedChannels.split(ID_DELIMITER);
		}
		else
		{
			// If there are no merged channels, default to the primary channel.
			mergedChannelArray = new String[1];
			mergedChannelArray[0] = primarychannelReference;
		}

		// Make a map of those channels that are currently merged.
		Map currentlyMergedchannels = new HashMap();

		if (mergedChannelArray != null)
		{
			for (int i = 0; i < mergedChannelArray.length; i++)
			{
				currentlyMergedchannels.put(
					mergedChannelArray[i],
					Boolean.valueOf(true));
			}
		}
		return currentlyMergedchannels;
	} // getChannelsFromConfig
	
	/**
	 * Loads data input by the user into this list and then saves the list to
	 * the portlet config information.  The initContextForMergeOptions() function
	 * must have previously been called.
	 * @param runData
	 */
	public void loadFromRunData(RunData runData)
	{
		Iterator it = this.iterator();

		while (it.hasNext())
		{
			MergedEntry entry = (MergedEntry) it.next();

			// If the group is even mentioned in the parameters, then
			// it means that the checkbox was selected.  Deselected checkboxes
			// will not be present in the parameter list.
			if (runData.getParameters().getString(entry.getReference())
				!= null)
			{
				entry.setMerged(true);
			}
			else
			{
				//
				// If the entry isn't visible, then we can't "unmerge" it due to
				// the lack of a checkbox in the user interface.
				//
				if (entry.isVisible())
				{
					entry.setMerged(false);
				}
			}
		}
	}

	/**
	 * Loads data input by the user into this list and then saves the list to
	 * the portlet config information.  The initContextForMergeOptions() function
	 * must have previously been called.
	 * @param runData
	 */
	public String getPortletConfigString()
	{
		StringBuffer mergedReferences = new StringBuffer("");
			
		Iterator it = this.iterator();
		boolean firstEntry = true;
			
		while (it.hasNext())
		{
			MergedEntry entry = (MergedEntry) it.next();
				
			if (entry.isMerged())
			{
				// Add a delimiter, if appropriate.
				if ( !firstEntry )
				{
					mergedReferences.append(ID_DELIMITER);
				}
				else
				{
					firstEntry = false;
				}
					
				// If we get here, then the checkbox was NOT checked
					
				// Add to our list
				mergedReferences.append(entry.getReference());
			}
				
		}
			
		// Return the comma-delimited list of un-merged references
		return mergedReferences.toString();
	}
		
	/**
	 * Returns an array of merged references.
	 * @return
	 */
	public List getReferenceList()
	{
		List references = new ArrayList();
			
		Iterator it = this.iterator();
			
		while (it.hasNext())
		{
			MergedEntry mergedEntry = (MergedEntry) it.next();
				
			// Only add it to the list if it has been merged.
			if (mergedEntry.isMerged())
			{
				references.add(mergedEntry.getReference());
			}
		}
			
		return references;
	}
}
