/**********************************************************************************
*
* $Header: /cvs/sakai/framework-component/src/java/org/sakaiproject/component/framework/email/BasicEmailService.java,v 1.10 2004/06/22 03:13:32 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

// package
package org.sakaiproject.component.framework.email;

// imports
import java.util.Properties;
import java.util.StringTokenizer;

import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.AddressException;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;

import org.sakaiproject.service.framework.config.ServerConfigurationService;
import org.sakaiproject.service.framework.email.EmailService;
import org.sakaiproject.service.framework.log.Logger;

/**
 * <p>BasicEmailService implements the EmailService.</p>
 * <p>See the {@link org.sakaiproject.service.framework.email.EmailService} interface for details.</p>
 * 
 * @author University of Michigan, CHEF Software Development Team
 * @version $Revision: 1.10 $
 */
public class BasicEmailService implements EmailService
{
	/*******************************************************************************
	* Dependencies and their setter methods
	*******************************************************************************/

	/** Dependency: logging service. */
	protected Logger m_logger = null;

	/**
	 * Dependency: logging service.
	 * @param service The logging service.
	 */
	public void setLogger(Logger service)
	{
		m_logger = service;
	}

	/** Dependency: ServerConfigurationService. */
	protected ServerConfigurationService m_serverConfigurationService = null;

	/**
	 * Dependency: ServerConfigurationService.
	 * @param service The ServerConfigurationService.
	 */
	public void setServerConfigurationService(ServerConfigurationService service)
	{
		m_serverConfigurationService = service;
	}

	/** Configuration: smtp server to use. */
	protected String m_smtp = null;

	/**
	 * Configuration: smtp server to use.
	 * @param value The smtp server string.
	 */
	public void setSmtp(String value)
	{
		m_smtp = value;
	}

	/*******************************************************************************
	* Init and Destroy
	*******************************************************************************/

	/**
	 * Final initialization, once all dependencies are set.
	 */
	public void init()
	{
		// look for a configuration override of the smtp
		m_smtp = m_serverConfigurationService.getString("smtp", m_smtp);

		m_logger.info(this +".init(): smtp: " + m_smtp);
	}

	/**
	 * Final cleanup.
	 */
	public void destroy()
	{
		m_logger.info(this +".destroy()");
	}

	/*******************************************************************************
	* Work interface methods: org.sakai.service.email.EmailService
	*******************************************************************************/

	protected static final String SERVER_PROP = "mail.smtp.host";

	protected static final String CONTENT_TYPE = "text/plain";

	/**
	 * {@inheritDoc}
	 */
	public void sendMail(
		InternetAddress from,
		InternetAddress[] to,
		String subject,
		String content,
		InternetAddress[] headerTo,
		InternetAddress[] replyTo)
	{
		Properties props = new Properties();
		props.put(SERVER_PROP, m_smtp);

		Session session = Session.getDefaultInstance(props, null);

		if (m_logger.isDebugEnabled())
		{
			m_logger.debug("Email.sendMail: from: " + from);
			if (to != null)
			{
				for (int i = 0; i < to.length; i++)
					m_logger.debug("  ** to: " + to[i]);
			}
			if (headerTo != null)
			{
				for (int i = 0; i < headerTo.length; i++)
					m_logger.debug("  ** headerTo: " + headerTo[i]);
			}
			if (replyTo != null)
			{
				for (int i = 0; i < replyTo.length; i++)
					m_logger.debug("  ** replyTo: " + replyTo[i]);
			}
		}

		try
		{
			MimeMessage msg = new MimeMessage(session);

			msg.setFrom(from);
			if (headerTo != null)
			{
				msg.setRecipients(Message.RecipientType.TO, headerTo);
			}
			else
			{
				msg.setRecipients(Message.RecipientType.TO, to);
			}

			msg.setSubject(subject);
			
			// msg.setText(content, "UTF-8");
			msg.setContent(content, CONTENT_TYPE);

			if (replyTo != null)
			{
				msg.setReplyTo(replyTo);
			}

			if (headerTo != null)
			{
				Transport.send(msg, to);
			}
			else
			{
				Transport.send(msg);
			}
		}
		catch (MessagingException e)
		{
			//			System.out.println(e);
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public void send(String fromStr, String toStr, String subject, String content, String headerToStr, String replyToStr)
	{
		try
		{
			InternetAddress from = new InternetAddress(fromStr);

			StringTokenizer tokens = new StringTokenizer(toStr, ", ");
			InternetAddress[] to = new InternetAddress[tokens.countTokens()];

			int i = 0;
			while (tokens.hasMoreTokens())
			{
				String next = (String) tokens.nextToken();
				to[i] = new InternetAddress(next);

				i++;
			} // cycle through and collect all of the Internet addresses from the list.

			InternetAddress[] headerTo = null;
			if (headerToStr != null)
			{
				headerTo = new InternetAddress[1];
				headerTo[0] = new InternetAddress(headerToStr);
			}

			InternetAddress[] replyTo = null;
			if (replyToStr != null)
			{
				replyTo = new InternetAddress[1];
				replyTo[0] = new InternetAddress(replyToStr);
			}

			sendMail(from, to, subject, content, headerTo, replyTo);

		}
		catch (AddressException e)
		{
			m_logger.warn(this + ".send: " + e);
		}
	}
}

/**********************************************************************************
*
* $Header: /cvs/sakai/framework-component/src/java/org/sakaiproject/component/framework/email/BasicEmailService.java,v 1.10 2004/06/22 03:13:32 ggolden Exp $
*
**********************************************************************************/
