/**********************************************************************************
*
* $Header: /cvs/sakai/framework-component/src/java/org/sakaiproject/component/framework/memory/BasicMemoryService.java,v 1.12 2004/06/22 03:13:32 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

// package
package org.sakaiproject.component.framework.memory;

// imports
import java.util.HashSet;
import java.util.Iterator;
import java.util.Observable;
import java.util.Observer;
import java.util.Set;

import org.sakaiproject.exception.PermissionException;
import org.sakaiproject.service.legacy.event.Event;
import org.sakaiproject.service.legacy.event.EventTrackingService;
import org.sakaiproject.service.framework.log.Logger;
import org.sakaiproject.service.framework.memory.Cache;
import org.sakaiproject.service.framework.memory.CacheRefresher;
import org.sakaiproject.service.framework.memory.Cacher;
import org.sakaiproject.service.framework.memory.MemoryService;
import org.sakaiproject.service.legacy.security.cover.SecurityService;
import org.sakaiproject.service.framework.state.cover.StateService;
import org.sakaiproject.util.SessionUtil;

/**
* <p>MemoryService is an implementation for the CHEF MemoryService which reports memory usage and
* runs a periodic garbage collection to keep memory available.</p>
* 
* @author University of Michigan, CHEF Software Development Team
* @version $Revision: 1.12 $
*/
public class BasicMemoryService implements MemoryService, Runnable, Observer
{
	/** How long to wait between clean & report passes (ms). */
	protected static final long PERIOD = 1000 * 60;

	/** Event for the memory reset. */
	protected static final String EVENT_RESET = "memory.reset";

	/** Byte value below which we are in low memory condition (anything below 100 meg). */
	protected long m_lowMemoryThresholdBytes = 100 * 1024 * 1024;

	/** The thread I run my periodic clean and report on. */
	protected Thread m_thread = null;

	/** My thread's quit flag. */
	protected boolean m_threadStop = false;

	/** Last memory check. */
	protected long m_lastMemoryValue = 0;

	/** Set of registered cachers. */
	protected Set m_cachers = new HashSet();

	/**
	* Start the clean and report thread.
	*/
	protected void start()
	{
		m_threadStop = false;

		m_thread = new Thread(this, getClass().getName());
		m_thread.start();

	} // start

	/**
	* Stop the clean and report thread.
	*/
	protected void stop()
	{
		if (m_thread == null)
			return;

		// signal the thread to stop
		m_threadStop = true;

		// wake up the thread
		m_thread.interrupt();

		m_thread = null;

	} // stop

	/**
	* Run the clean and report thread.
	*/
	public void run()
	{
		// loop till told to stop
		while ((!m_threadStop) && (!Thread.currentThread().isInterrupted()))
		{
			try
			{
				// run the garbage collector now
				System.gc();

				// check memory
				m_lastMemoryValue = Runtime.getRuntime().freeMemory();

				// check for low memory threshold
				if (m_lastMemoryValue < m_lowMemoryThresholdBytes)
				{
					m_logger.info(
						this
							+ " ** Low Memory ("
							+ m_lastMemoryValue
							+ " < "
							+ m_lowMemoryThresholdBytes
							+ "): reseting caches");

					// reset cachers
					doReset();
				}

				// log bytes of free memory, in M.K
				if (m_logger.isDebugEnabled())
				{
					long k = m_lastMemoryValue / 1024;
					short m = (short) (k / 1024);
					k = (m_lastMemoryValue - (m * 1024 * 1024)) / 1024;

					long totalBytes = Runtime.getRuntime().totalMemory();
					long tk = totalBytes / 1024;
					short tm = (short) (tk / 1024);
					tk = (totalBytes - (tm * 1024 * 1024)) / 1024;

					m_logger.debug("Memory: " + m + "." + k + "(" + m_lastMemoryValue + ") "
					// + "max: " + Runtime.getRuntime().maxMemory()
					+"total: " + tm + "." + tk + "(" + totalBytes + ") "
					//+ "processors: " + Runtime.getRuntime().availableProcessors()
					);
				}
			}
			catch (Throwable e)
			{
				m_logger.warn(this +": exception: ", e);
			}

			// take a small nap
			try
			{
				Thread.sleep(PERIOD);
			}
			catch (Exception ignore)
			{
			}

		} // while

	} // run

	/*******************************************************************************
	* Dependencies and their setter methods
	*******************************************************************************/

	/**
	 * Configuration: low memory threshold
	 */
	public void setLowMemoryThreshold(String value)
	{
		try
		{
			m_lowMemoryThresholdBytes = Long.parseLong(value);
		}
		catch (Exception ignore)
		{
		}
	}

	/** Dependency: EventTrackingService. */
	protected EventTrackingService m_eventTrackingService = null;

	/**
	 * Dependency: EventTrackingService.
	 */
	public void setEventTrackingService(EventTrackingService service)
	{
		m_eventTrackingService = service;
	}

	/** Dependency: logging service. */
	protected Logger m_logger = null;

	/**
	 * Dependency: logging service
	 */
	public void setLogger(Logger service)
	{
		m_logger = service;
	}

	/*******************************************************************************
	* Init and Destroy
	*******************************************************************************/

	/**
	 * Final initialization, once all dependencies are set.
	 */
	public void init()
	{
		try
		{
			// get notified of events to watch for a reset
			m_eventTrackingService.addObserver(this);

			// here's where we start our thread
			start();

			m_logger.info(this +".init()");
		}
		catch (Throwable t)
		{
			m_logger.warn(this +".init(): ", t);
		}

	} // init

	/**
	* Returns to uninitialized state.
	*/
	public void destroy()
	{
		m_eventTrackingService.deleteObserver(this);

		stop();
		m_cachers.clear();

		m_logger.info(this +".destroy()");
	}

	/*******************************************************************************
	* MemoryService implementation
	*******************************************************************************/

	/**
	* Return the amount of available memory.
	* @return the amount of available memory.
	*/
	public long getAvailableMemory()
	{
		return m_lastMemoryValue;

	} // getAvailableMemory

	/**
	* Cause less memory to be used by clearing any optional caches.
	*/
	public void resetCachers() throws PermissionException
	{
		// check that this is a "super" user with the security service
		if (SecurityService.isSuperUser())
		{
			throw new PermissionException(SessionUtil.getSessionId(StateService.getInstance()), EVENT_RESET, "");
		}

		// post the event so this and any other app servers in the cluster will reset
		m_eventTrackingService.post(m_eventTrackingService.newEvent(EVENT_RESET, "", true));

	} // resetMemory

	/**
	* Do a reset of all cachers
	*/
	protected void doReset()
	{
		if (!m_cachers.isEmpty())
		{
			// tell all our memory users to reset their memory use
			Iterator it = m_cachers.iterator();
			while (it.hasNext())
			{
				Cacher cacher = (Cacher) it.next();
				cacher.resetCache();
			}

			// run the garbage collector now
			System.gc();

			// update the memory reading
			m_lastMemoryValue = Runtime.getRuntime().freeMemory();
		}

		m_logger.info(this +".doReset():  Low Memory Recovery to: " + m_lastMemoryValue);

	} // doReset

	/**
	* Register as a cache user
	*/
	public void registerCacher(Cacher cacher)
	{
		m_cachers.add(cacher);

	} // registerCacher

	/**
	* Unregister as a cache user
	*/
	public void unregisterCacher(Cacher cacher)
	{
		m_cachers.remove(cacher);

	} // unregisterCacher

	/**
	 * {@inheritDoc}
	 */
	public Cache newCache(CacheRefresher refresher, String pattern)
	{
		return new MemCache(this, m_eventTrackingService, refresher, pattern);
	}

	/**
	 * {@inheritDoc}
	 */
	public Cache newCache(CacheRefresher refresher, long sleep)
	{
		return new MemCache(this, m_eventTrackingService, refresher, sleep);
	}

	/**
	 * {@inheritDoc}
	 */
	public Cache newCache()
	{
		return new MemCache(this, m_eventTrackingService);
	}

	/*******************************************************************************
	* Observer implementation
	*******************************************************************************/

	/**
	* This method is called whenever the observed object is changed. An
	* application calls an <tt>Observable</tt> object's
	* <code>notifyObservers</code> method to have all the object's
	* observers notified of the change.
	*
	* default implementation is to cause the courier service to deliver to the
	* interface controlled by my controller.  Extensions can override.
	*
	* @param   o     the observable object.
	* @param   arg   an argument passed to the <code>notifyObservers</code>
	*                 method.
	*/
	public void update(Observable o, Object arg)
	{
		// arg is Event
		if (!(arg instanceof Event))
			return;
		Event event = (Event) arg;

		// look for the memory reset event
		String function = event.getEvent();
		if (!function.equals(EVENT_RESET))
			return;

		// do the reset here, too!
		doReset();

	}

} // BasicMemoryService

/**********************************************************************************
*
* $Header: /cvs/sakai/framework-component/src/java/org/sakaiproject/component/framework/memory/BasicMemoryService.java,v 1.12 2004/06/22 03:13:32 ggolden Exp $
*
**********************************************************************************/
