/**********************************************************************************
*
* $Header: /cvs/sakai/framework-component/src/java/org/sakaiproject/component/framework/memory/MemCache.java,v 1.7 2004/06/22 03:13:32 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

// package
package org.sakaiproject.component.framework.memory;

// import
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;
import java.util.Set;
import java.util.Vector;

import org.sakaiproject.service.legacy.event.Event;
import org.sakaiproject.service.legacy.event.EventTrackingService;
import org.sakaiproject.service.framework.log.cover.Logger;
import org.sakaiproject.service.framework.memory.Cache;
import org.sakaiproject.service.framework.memory.CacheRefresher;
import org.sakaiproject.service.framework.memory.MemoryService;

/**
* <p>A Cache of objects with keys with a limited lifespan.</p>
* <p>When the object expires, the cache calls upon a CacheRefresher to update the key's value.
* The update is done in a separate thread.</p>
*
* @author University of Michigan, CHEF Software Development Team
* @version $Revision: 1.7 $
*/
public class MemCache implements Cache, Runnable, Observer
{
	/** Map holding cached entries. */
	protected Map m_map = null;
	
	/** The object that will deal with expired entries. */
	protected CacheRefresher m_refresher = null;

	/** The string that all resources in this cache will start with. */
	protected String m_resourcePattern = null;

	/** The number of seconds to sleep between expiration checks. */
	protected long m_refresherSleep = 60;

	/** If true, we are disabled. */
	protected boolean m_disabled = false;

	/** If true, we have all the entries that there are in the cache. */
	protected boolean m_complete = false;

	/** Alternate isComplete, based on patterns. */
	protected Set m_partiallyComplete = new HashSet();

	/** If true, we are going to hold any events we see in the m_heldEvents list for later processing. */
	protected boolean m_holdEventProcessing = false;

	/** The events we are holding for later processing. */
	protected List m_heldEvents = new Vector();

	/** Constructor injected memory service. */
	protected MemoryService m_memoryService = null;

	/** Constructor injected event tracking service. */
	protected EventTrackingService m_eventTrackingService = null;

	/**
	* The cache entry.  Holds a time stamped payload.
	*/
	protected class CacheEntry
	{
		/** currentTimeMillis when this expires. */
		protected long m_expires = 0;

		/** The thing cached. */
		protected Object m_payload = null;

		/** The time (seconds) to keep this cached (0 means don't exipre). */
		protected int m_duration = 0;

		/**
		* Construct to cache the payload for the duration.
		* @param payload The thing to cache.
		* @param duration The time (seconds) to keep this cached.
		*/
		public CacheEntry(Object payload, int duration)
		{
			m_payload = payload;
			m_duration = duration;
			if (m_duration > 0) m_expires = System.currentTimeMillis() + (duration * 1000);

		}	// CacheEntry

		/**
		* Get the cached object.
		* @return The cached object.
		*/
		public Object getPayload() { return m_payload; }

		/**
		* Check for expiration.
		* @return true if expired, false if still good.
		*/
		public boolean hasExpired() { return ((m_duration > 0) ? (System.currentTimeMillis() > m_expires) : false); }

		/**
		* Access the duration.
		* @return The time (seconds) before the entry expires.
		*/
		public int getDuration() { return m_duration; }

	}	// CacheEntry

	/**
	* Construct the Cache.  No automatic refresh handling.
	*/
	public MemCache(MemoryService memoryService, EventTrackingService eventTrackingService)
	{
		// inject our dependencies
		m_memoryService = memoryService;
		m_eventTrackingService = eventTrackingService;

		m_map = new HashMap();

		// register as a cacher
		m_memoryService.registerCacher(this);
	}
			
	/**
	* Construct the Cache.  Attempts to keep complete on Event notification by
	* calling the refresher.
	* @param refresher The object that will handle refreshing of event notified modified or added entries.
	* @param pattern The "startsWith()" string for all resources that may be in this cache -
	* if null, don't watch events for updates.
	*/
	public MemCache(MemoryService memoryService, EventTrackingService eventTrackingService, CacheRefresher refresher, String pattern)
	{
		this(memoryService, eventTrackingService);
		m_refresher = refresher;
		m_resourcePattern = pattern;

		// register to get events - first, before others
		if (pattern != null)
		{
			m_eventTrackingService.addPriorityObserver(this);
		}
	}

	/**
	* Construct the Cache.  Automatic refresh handling if refresher is not null.
	* @param refresher The object that will handle refreshing of expired entries.
	* @param sleep The number of seconds to sleep between expiration checks.
	*/
	public MemCache(MemoryService memoryService, EventTrackingService eventTrackingService, CacheRefresher refresher, long sleep)
	{
		this(memoryService, eventTrackingService);
		m_refresherSleep = sleep;

		if (refresher != null)
		{
			m_refresher = refresher;
			
			// start the expiration thread
			start();
		}
	}

	/**
	* Clean up.
	*/
	public void destroy()
	{
		clear();

		// unregister as a cacher
		m_memoryService.unregisterCacher(this);

		// ungister to get events
		m_eventTrackingService.deleteObserver(this);

		// stop our expiration thread (if any)
		stop();
	}

	/**
	* Cache an object
	* @param key The key with which to find the object.
	* @param payload The object to cache.
	* @param duration The time to cache the object (seconds).
	*/
	public synchronized void put(Object key, Object payload, int duration)
	{
		if (disabled()) return;

		m_map.put(key, new CacheEntry(payload, duration));

	}	// cache

	/**
	* Cache an object - don't automatically exipire it.
	* @param key The key with which to find the object.
	* @param payload The object to cache.
	* @param duration The time to cache the object (seconds).
	*/
	public synchronized void put(Object key, Object payload)
	{
		put(key, payload, 0);

	}	// cache

	/**
	* Test for an entry in the cache - expired or not.
	* @param key The cache key.
	* @return true if the key maps to a cache entry, false if not.
	*/
	public synchronized boolean containsKeyExpiredOrNot(Object key)
	{
		if (disabled()) return false;

		// is it there?
		return m_map.containsKey(key);

	}	// containsKeyExpiredOrNot

	/**
	* Test for a non expired entry in the cache.
	* @param key The cache key.
	* @return true if the key maps to a non-expired cache entry, false if not.
	*/
	public synchronized boolean containsKey(Object key)
	{
		if (disabled()) return false;

		// is it there?
		if (m_map.containsKey(key))
		{
			// has it expired?
			CacheEntry entry = (CacheEntry) m_map.get(key);
			if (entry.hasExpired())
			{
				// if so, remove it
				m_map.remove(key);
				return false;
			}
			return true;
		}

		return false;

	}	// containsKey

	/**
	* Expire this object.
	* @param key The cache key.
	*/
	public synchronized void expire(Object key)
	{
		if (disabled()) return;

		// is it there?
		if (m_map.containsKey(key))
		{
			// remove it
			m_map.remove(key);
		}

	}	// expire

	/**
	* Get the entry, or null if not there (expired entries are returned, too).
	* @param key The cache key.
	* @return The payload, or null if the payload is null, the key is not found.
	* (Note: use containsKey() to remove this ambiguity).
	*/
	public synchronized Object getExpiredOrNot(Object key)
	{
		if (disabled()) return null;

		// is it there?
		if (m_map.containsKey(key))
		{
			CacheEntry entry = (CacheEntry) m_map.get(key);
			return entry.getPayload();
		}

		return null;

	}	// getExpiredOrNot

	/**
	* Get the non expired entry, or null if not there (or expired)
	* @param key The cache key.
	* @return The payload, or null if the payload is null, the key is not found,
	* or the entry has expired (Note: use containsKey() to remove this ambiguity).
	*/
	public synchronized Object get(Object key)
	{
		if (disabled()) return null;

		// is it there?
		if (m_map.containsKey(key))
		{
			// has it expired?
			CacheEntry entry = (CacheEntry) m_map.get(key);
			if (entry.hasExpired())
			{
				// if so, remove it
				m_map.remove(key);
				return null;
			}
			return entry.getPayload();
		}

		return null;

	}	// get

	/**
	* Get all the non-expired non-null entries.
	* @return all the non-expired non-null entries, or an empty list if none.
	*/
	public synchronized List getAll()
	{
		List rv = new Vector();

		if (disabled()) return rv;
		if (m_map.isEmpty()) return rv;

		// get the keys as of now
		Object [] keys = m_map.keySet().toArray();
		
		// for each entry in the cache
		for (int i = 0; i < keys.length; i++)
		{
			CacheEntry entry = (CacheEntry) m_map.get(keys[i]);

			// skip expired
			if (entry.hasExpired()) continue;

			Object payload = entry.getPayload();

			// skip nulls
			if (payload == null) continue;

			// skip inappropriate types
			// if ((type != null) && (!(type.isInstance(payload)))) continue;

			// filter out those not matching the filter
			// if ((filter != null) && (((String) keys[i]).startsWith(filter))) continue;

			// we'll take it 
			rv.add(payload);
		}

		return rv;

	}	// getAll

	/**
	* Get all the non-expired non-null entries that are in the specified reference path.
	* Note: only works with String keys.
	* @param path The reference path.
	* @return all the non-expired non-null entries, or an empty list if none.
	*/
	public synchronized List getAll(String path)
	{
		List rv = new Vector();

		if (disabled()) return rv;
		if (m_map.isEmpty()) return rv;

		// get the keys as of now
		Object [] keys = m_map.keySet().toArray();
		
		// for each entry in the cache
		for (int i = 0; i < keys.length; i++)
		{
			CacheEntry entry = (CacheEntry) m_map.get(keys[i]);

			// skip expired
			if (entry.hasExpired()) continue;

			Object payload = entry.getPayload();

			// skip nulls
			if (payload == null) continue;

			// take only if keys start with path, and have no SEPARATOR following other than at the end %%%
			String keyPath = referencePath((String) keys[i]);
			if (!keyPath.equals(path)) continue;

			// we'll take it 
			rv.add(payload);
		}

		return rv;

	}	// getAll

	/**
	* Get all the keys
	* @return The List of key values (Object).
	*/
	public List getKeys()
	{
		List rv = new Vector();
		rv.addAll(m_map.keySet());
		return rv;

	}	// getKeys

	/**
	* Get all the keys, eache modified to remove the resourcePattern prefix.
	* Note: only works with String keys.
	* @return The List of keys converted from references to ids (String).
	*/
	public List getIds()
	{
		List rv = new Vector();

		List keys = new Vector();
		keys.addAll(m_map.keySet());

		Iterator it = keys.iterator();
		while (it.hasNext())
		{
			String key = (String) it.next();
			int i = key.indexOf(m_resourcePattern);
			if (i != -1) key = key.substring(i+m_resourcePattern.length());
			rv.add(key);
		}

		return rv;

	}	// getKeys

	/**
	* Clear all entries.
	*/
	public synchronized void clear()
	{
		m_map.clear();
		m_complete = false;
		m_partiallyComplete.clear();

	}	// clear

	/**
	* Remove this entry from the cache.
	* @param key The cache key.
	*/
	public synchronized void remove(Object key)
	{
		if (disabled()) return;

		m_map.remove(key);

	}	// remove

	/**
	* Disable the cache.
	*/
	public void disable()
	{
		m_disabled = true;
		m_eventTrackingService.deleteObserver(this);
		clear();

	}	// disable

	/**
	* Enable the cache.
	*/
	public void enable()
	{
		m_disabled = false;

		if (m_resourcePattern != null)
		{
			m_eventTrackingService.addPriorityObserver(this);
		}

	}	// enable

	/**
	* Is the cache disabled?
	* @return true if the cache is disabled, false if it is enabled.
	*/
	public boolean disabled()
	{
		return m_disabled;

	}	// disabled

	/**
	* Are we complete?
	* @return true if we have all the possible entries cached, false if not.
	*/
	public boolean isComplete()
	{
		if (disabled()) return false;

		return m_complete;

	}	// isComplete

	/**
	* Set the cache to be complete, containing all possible entries.
	*/
	public void setComplete()
	{
		if (disabled()) return;

		m_complete = true;

	}	// isComplete

	/**
	* Are we complete for one level of the reference hierarchy?
	* @param path The reference to the completion level.
	* @return true if we have all the possible entries cached, false if not.
	*/
	public boolean isComplete(String path)
	{
		return m_partiallyComplete.contains(path);

	}	// isComplete

	/**
	* Set the cache to be complete for one level of the reference hierarchy.
	* @param path The reference to the completion level.
	*/
	public void setComplete(String path)
	{
		m_partiallyComplete.add(path);

	}	// setComplete

	/**
	* Set the cache to hold events for later processing to assure an atomic "complete" load.
	*/
	public synchronized void holdEvents()
	{
		m_holdEventProcessing = true;

	}	// holdEvents

	/**
	* Restore normal event processing in the cache, and process any held events now.
	*/
	public synchronized void processEvents()
	{
		m_holdEventProcessing = false;

		for (int i = 0; i < m_heldEvents.size(); i++)
		{
			Event event = (Event) m_heldEvents.get(i);
			continueUpdate(event);
		}
		
		m_heldEvents.clear();

	}	// holdEvents

	/*******************************************************************************
	* Cacher implementation
	*******************************************************************************/

	/**
	* Clear out as much as possible anything cached; re-sync any cache that is needed to be kept.
	*/
	public void resetCache()
	{
		clear();

	}	// resetCache

	/*******************************************************************************
	* Runnable implementation
	*******************************************************************************/

	/** The thread which runs the expiration check. */
	protected Thread m_thread = null;

	/** My thread's quit flag. */
	protected boolean m_threadStop = false;

	/**
	* Start the expiration thread.
	*/
	protected void start()
	{
		m_threadStop = false;

		m_thread = new Thread(this, getClass().getName());
		m_thread.setDaemon(true);
		m_thread.setPriority(Thread.MIN_PRIORITY+2);
		m_thread.start();
		
	}	// start

	/**
	* Stop the expiration thread.
	*/
	protected void stop()
	{
		if (m_thread == null) return;

		// signal the thread to stop
		m_threadStop = true;

		// wake up the thread
		m_thread.interrupt();

		m_thread = null;

	}	// stop

	/**
	* Run the expiration thread.
	*/
	public void run()
	{
		// loop till told to stop
		while (		(!m_threadStop)
				&&	(!Thread.currentThread().isInterrupted()))
		{
			long startTime = 0;
			try
			{
				if (Logger.isDebugEnabled())
				{
					startTime = System.currentTimeMillis();
					Logger.debug(this + ".checking ...");
				}

				if (!m_map.isEmpty())
				{
					// get the keys as of now
					Object [] keys = m_map.keySet().toArray();
					
					// for each entry in the cache
					for (int i = 0; i < keys.length; i++)
					{
						CacheEntry entry = (CacheEntry) m_map.get(keys[i]);

						// if it has expired
						if (entry.hasExpired())
						{
							// ask the refresher to deal with it
							Object newValue = m_refresher.refresh(keys[i], entry.getPayload(), null);
							
							// if the response is not null, replace and rejuvinate
							if (newValue != null)
							{
								put(keys[i], newValue, entry.getDuration());
							}

							// otherwise remove
							else
							{
								remove(keys[i]);
							}
						}
					}
				}
			}
			catch (Throwable e) {Logger.warn(this + ": exception: ", e);}

			if (Logger.isDebugEnabled())
			{
				Logger.debug(this + ".done. Time: " + (System.currentTimeMillis() - startTime));
			}

			// take a small nap
			try
			{
				Thread.sleep(m_refresherSleep * 1000);
			}
			catch (Exception ignore) {}

		}	// while

	}	// run

	/*******************************************************************************
	* Observer implementation
	*******************************************************************************/

	/**
	* This method is called whenever the observed object is changed. An
	* application calls an <tt>Observable</tt> object's
	* <code>notifyObservers</code> method to have all the object's
	* observers notified of the change.
	*
	* default implementation is to cause the courier service to deliver to the
	* interface controlled by my controller.  Extensions can override.
	*
	* @param   o     the observable object.
	* @param   arg   an argument passed to the <code>notifyObservers</code>
	*                 method.
	*/
	public void update(Observable o, Object arg)
	{
		if (disabled()) return;

		// arg is Event
		if (!(arg instanceof Event)) return;
		Event event = (Event) arg;

		// if this is just a read, not a modify event, we can ignore it
		if (!event.getModify()) return;

		String key = event.getResource();

		// if this resource is not in my pattern of resources, we can ignore it
		if (!key.startsWith(m_resourcePattern)) return;

		// if we are holding event processing
		if (m_holdEventProcessing)
		{
			m_heldEvents.add(event);
			return;
		}

		continueUpdate(event);

	}	// update

	/**
	* Complete the update, given an event that we know we need to act upon.
	* @param event The event to process.
	*/
	protected void continueUpdate(Event event)
	{
		String key = event.getResource();

		if (Logger.isDebugEnabled())
			Logger.debug(this + ".update() [" + m_resourcePattern + "] resource: " + key + " event: " + event.getEvent());

		// do we have this in our cache?
		Object oldValue = get(key);
		if (m_map.containsKey(key))
		{
			// invalidate our copy
			remove(key);
		}

		// if we are being complete, we need to get this cached.
		if (m_complete)
		{
			// we can only get it cached if we have a refresher
			if (m_refresher != null)
			{
				// ask the refresher for the value
				Object value = m_refresher.refresh(key, oldValue, event);
				if (value != null)
				{
					put(key, value);
				}
			}
			else
			{
				// we can no longer claim to be complete
				m_complete = false;
			}
		}
		
		// if we are partially complete
		else if (!m_partiallyComplete.isEmpty())
		{
			// what is the reference path that this key lives within?
			String path = referencePath(key);
			
			// if we are partially complete for this path
			if (m_partiallyComplete.contains(path))
			{
				// we can only get it cached if we have a refresher
				if (m_refresher != null)
				{
					// ask the refresher for the value
					Object value = m_refresher.refresh(key, oldValue, event);
					if (value != null)
					{
						put(key, value);
					}
				}
				else
				{
					// we can no longer claim to be complete for this path
					m_partiallyComplete.remove(path);
				}
			}
		}

	}	// continueUpdate

	/**
	* Compute the reference path (i.e. the container) for a given reference.
	* @param ref The reference string.
	* @return The reference root for the given reference.
	*/
	protected String referencePath(String ref)
	{
		String path = null;

		// Note: there may be a trailing separator
		int pos = ref.lastIndexOf("/", ref.length()-2);

		// if no separators are found, place it even before the root!
		if (pos == -1)
		{
			path = "";
		}

		// use the string up to and including that last separator
		else
		{
			path = ref.substring(0, pos+1);
		}
		
		return path;

	}	// referencePath
}

/**********************************************************************************
*
* $Header: /cvs/sakai/framework-component/src/java/org/sakaiproject/component/framework/memory/MemCache.java,v 1.7 2004/06/22 03:13:32 ggolden Exp $
*
**********************************************************************************/
