/**********************************************************************************
*
* $Header: /cvs/sakai/framework-component/src/java/org/sakaiproject/component/framework/session/BaseUsageSessionService.java,v 1.18 2004/06/22 03:13:32 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

// package
package org.sakaiproject.component.framework.session;

// imports
import org.sakaiproject.exception.IdUnusedException;
import org.sakaiproject.service.framework.config.ServerConfigurationService;
import org.sakaiproject.service.framework.current.CurrentService;
import org.sakaiproject.service.legacy.event.Event;
import org.sakaiproject.service.legacy.event.EventTrackingService;
import org.sakaiproject.service.legacy.id.IdService;
import org.sakaiproject.service.framework.log.Logger;
import org.sakaiproject.service.framework.memory.Cache;
import org.sakaiproject.service.framework.memory.MemoryService;
import org.sakaiproject.service.legacy.presence.cover.PresenceService;
import org.sakaiproject.service.framework.session.UsageSession;
import org.sakaiproject.service.framework.session.UsageSessionService;
import org.sakaiproject.service.framework.state.SessionStateBindingListener;
import org.sakaiproject.service.framework.state.StateService;
import org.sakaiproject.service.legacy.time.Time;
import org.sakaiproject.service.legacy.time.TimeService;
import org.sakaiproject.service.legacy.user.User;
import org.sakaiproject.service.legacy.user.cover.UserDirectoryService;
import org.sakaiproject.util.SessionUtil;

/**
* <p>BaseUsageSessionService is a CHEF usage session service implemented as a Turbine Service.</p>
* 
* @author University of Michigan, CHEF Software Development Team
* @version $Revision: 1.18 $
*/
public abstract class BaseUsageSessionService implements UsageSessionService
{
	/** Storage manager for this service. */
	protected Storage m_storage = null;

	/** A Cache for this service - UsageSessions keyed by reference (only for non-local). */
	protected Cache m_cache = null;

	/*******************************************************************************
	* Abstractions, etc.
	*******************************************************************************/

	/**
	* Construct storage for this service.
	*/
	protected abstract Storage newStorage();

	/*******************************************************************************
	* Dependencies and their setter methods
	*******************************************************************************/

	/** Dependency: logging service */
	protected Logger m_logger = null;

	/**
	 * Dependency: logging service.
	 * @param service The logging service.
	 */
	public void setLogger(Logger service)
	{
		m_logger = service;
	}

	/** Dependency: memory service */
	protected MemoryService m_memoryService = null;

	/**
	 * Dependency: memory service.
	 * @param service The memory service.
	 */
	public void setMemoryService(MemoryService service)
	{
		m_memoryService = service;
	}

	/** Dependency: ServerConfigurationService. */
	protected ServerConfigurationService m_serverConfigurationService = null;

	/**
	 * Dependency: ServerConfigurationService.
	 * @param service The ServerConfigurationService.
	 */
	public void setServerConfigurationService(ServerConfigurationService service)
	{
		m_serverConfigurationService = service;
	}

	/** Dependency: CurrentService. */
	protected CurrentService m_currentService = null;

	/**
	 * Dependency: CurrentService.
	 * @param service The CurrentService.
	 */
	public void setCurrentService(CurrentService service)
	{
		m_currentService = service;
	}

	/** Dependency: EventTrackingService. */
	protected EventTrackingService m_eventTrackingService = null;

	/**
	 * Dependency: EventTrackingService.
	 * @param service The EventTrackingService.
	 */
	public void setEventTrackingService(EventTrackingService service)
	{
		m_eventTrackingService = service;
	}

	/** Dependency: IdService. */
	protected IdService m_idService = null;

	/**
	 * Dependency: IdService.
	 * @param service The IdService.
	 */
	public void setIdService(IdService service)
	{
		m_idService = service;
	}

	/** Dependency: TimeService. */
	protected TimeService m_timeService = null;

	/**
	 * Dependency: TimeService.
	 * @param service The TimeService.
	 */
	public void setTimeService(TimeService service)
	{
		m_timeService = service;
	}

	/** Dependency: StateService. */
	protected StateService m_stateService = null;

	/**
	 * Dependency: StateService.
	 * @param service The StateService.
	 */
	public void setStateService(StateService service)
	{
		m_stateService = service;
	}

	/*******************************************************************************
	* Init and Destroy
	*******************************************************************************/

	public BaseUsageSessionService()
	{
		// construct storage and read
		m_storage = newStorage();
	}

	/**
	 * Final initialization, once all dependencies are set.
	 */
	public void init()
	{
		try
		{
			// open storage
			m_storage.open();

			// make the cache - don't watch any events for updates, no auto-refresh
			m_cache = m_memoryService.newCache();

			m_logger.info(this +".init()");
		}
		catch (Throwable t)
		{
			m_logger.warn(this +".init(): ", t);
		}

	} // init

	/**
	* Returns to uninitialized state.
	*/
	public void destroy()
	{
		if (m_cache != null)
		{
			m_cache.destroy();
			m_cache = null;
		}

		m_storage.close();

		m_logger.info(this +".destroy()");

	}

	/*******************************************************************************
	* UsageSessionService implementation
	*******************************************************************************/

	/**
	* Authenticate, and if successful, set as the session's authenticated user.
	* @param id The user id
	* @param pw The password
	* @param remoteAddress The IP address from the user is making a request.
	* @param userAgent The string describing the user's browser.
	* @return true if successfully authenticated, false if not.
	*/
	public boolean loginSessionUser(String id, String pw, String remoteAddress, String userAgent)
	{
		User user = UserDirectoryService.authenticate(id, pw);

		// if not authenticated
		if (user == null)
			return false;

		UsageSession session = new BaseUsageSession(m_serverConfigurationService.getServerIdInstance(), user, remoteAddress, userAgent);

		// associate with current session
		SessionUtil.setSession(m_stateService, session);

		// store
		m_storage.addSession(session);

		// track the event
		Event event = m_eventTrackingService.newEvent(EVENT_LOGIN, session.toString(), true); // %%% really, what resource?
		m_eventTrackingService.post(event);

		return true;

	} // loginSessionUser

	/**
	* Logout the session user
	*/
	public void logoutSessionUser()
	{
		// get the session
		BaseUsageSession session = (BaseUsageSession) SessionUtil.getSession(m_stateService);
		if (session == null)
			return;

		// mark as closed
		session.close();

		// retire the presence for this session
		PresenceService.removePresence(session);
		PresenceService.doInContextMaintenance(false);	

		// removal of the session from state triggers the sessions's valueUnbound(), which generates the EVENT.LOGOUT event
		SessionUtil.setSession(m_stateService, null);

		// just to be consistent with loginSessionUser
		UserDirectoryService.destroyAuthentication();

	} // logoutSessionUser

	/**
	* Accept a user id as the "user" for the current (non-session) thread.
	* @param id The user id.
	*/
	public void loginThreadUser(String id)
	{
		// make sure that this thread is NOT a session thread
		if (SessionUtil.getSession(m_stateService) != null)
			return;

		SessionUtil.setThreadSessionUserId(m_currentService, id);

	} // loginThreadUser

	/**
	* Remove the user for this thread.
	*/
	public void logoutThreadUser()
	{
		SessionUtil.setThreadSessionUserId(m_currentService, null);

	} // logoutThreadUser

	/**
	* Access the authenticated session user for the current request's session.
	* @return The User object for the current authenticated user, or the anon. user if none.
	*/
	public User getSessionUser()
	{
		UsageSession us = SessionUtil.getSession(m_stateService);
		if (us != null)
		{
			User user = us.getUser();
			if (user != null)
			{
				return user;
			}
		}

		// if no session, try for a thread user
		else
		{
			String id = SessionUtil.getThreadSessionUserId(m_currentService);
			if (id != null)
			{
				try
				{
					return UserDirectoryService.getUser(id);
				}
				catch (IdUnusedException e){}
			}
		}

		// no user, so return the anon. users
		return UserDirectoryService.getAnonymousUser();

	} // getSessionUser

	/**
	* Access all the information about this session.
	* @return The UsageSession object holding the information about this session.
	*/
	public UsageSession getSession()
	{
		return SessionUtil.getSession(m_stateService);

	} // getSession

	/**
	* Access all the information about a session.
	* @param id the Session id.
	* @return The UsageSession object holding the information about a session.
	*/
	public UsageSession getSession(String id)
	{
		// try it as a local session
		UsageSession rv = m_storage.getLocalSession(id);

		if (rv == null)
		{
			if (m_cache != null)
			{
				// try the cache
				rv = (UsageSession) m_cache.get(id);

				if (rv == null)
				{
					// try getting it and caching it
					synchronized (m_cache)
					{
						rv = (UsageSession) m_cache.get(id);
						if (rv == null)
						{
							rv = m_storage.getSession(id);
							m_cache.put(id, rv);
						}
					}
				}
			}
			
			// we may not have a cache or not be inited yet or have been shutdown...
			else
			{
				rv = m_storage.getSession(id);
			}
		}

		return rv;

	} // getSession

	/**
	* Is this session a local one?
	* @param id The session id.
	* @return true if the session with this id exists and is locally maintained, false if not.
	*/
	public boolean isLocalSession(String id)
	{
		return m_storage.getLocalSession(id) != null;

	} // isLocalSession

	/*******************************************************************************
	* Storage
	*******************************************************************************/

	protected interface Storage
	{
		/**
		* Open.
		*/
		public void open();

		/**
		* Close.
		*/
		public void close();

		/**
		* Take this session into storage.
		* @param session The usage session.
		*/
		public void addSession(UsageSession session);

		/**
		* Access a session by id
		* @param id The session id.
		* @return The session object.
		*/
		public UsageSession getSession(String id);

		/**
		* Access a local (i.e. maintained in this app server) session by id
		* @param id The session id.
		* @return The session object.
		*/
		public UsageSession getLocalSession(String id);

		/**
		* This session is now closed.
		* @param session The session which is closed.
		*/
		public void closeSession(UsageSession session);

	} // Storage

	/*******************************************************************************
	* UsageSession
	*******************************************************************************/

	protected class BaseUsageSession implements UsageSession, SessionStateBindingListener
	{
		/** The user object for this session. */
		protected User m_user = null;

		/** The unique id for this session. */
		protected String m_id = null;

		/** The server which is hosting the session. */
		protected String m_server = null;

		/** The IP Address from which this session originated. */
		protected String m_ip = null;

		/** The User Agent string describing the browser used in this session. */
		protected String m_userAgent = null;

		/** The BrowserID string describing the browser used in this session. */
		protected String m_browserId = null;

		/** The time the session was started */
		protected Time m_start = null;

		/** The time the session was closed. */
		protected Time m_end = null;

		/**
		* Construct.
		* @param server The server id which is hosting the session.
		* @param user The user object for this session.
		* @param address The IP Address from which this session originated.
		* @param agent The User Agent string describing the browser used in this session.
		*/
		public BaseUsageSession(String server, User user, String address, String agent)
		{
			m_id = m_idService.getUniqueId();
			m_server = server;
			m_user = user;
			m_ip = address;
			m_userAgent = agent;
			m_start = m_timeService.newTime();
			m_end = m_start;
			setBrowserId(agent);

		} // BaseUsageSession

		/**
		* Construct fully.
		* @param id The session id.
		* @param server The server id which is hosting the session.
		* @param user The user object for this session.
		* @param address The IP Address from which this session originated.
		* @param agent The User Agent string describing the browser used in this session.
		*/
		public BaseUsageSession(String id, String server, User user, String address, String agent, Time start, Time end)
		{
			m_id = server + id;
			m_server = server;
			m_user = user;
			m_ip = address;
			m_userAgent = agent;
			m_start = start;
			m_end = end;
			setBrowserId(agent);

		} // BaseUsageSession

		/**
		* Set the browser id for this session, decoded from the user agent string.
		* @param agent The user agent string.
		*/
		protected void setBrowserId(String agent)
		{
			// test whether agent is UserAgent value for a known browser.
			// should we also check version number?
			if (agent.indexOf("Netscape") >= 0 && agent.indexOf("Mac") >= 0)
			{
				m_browserId = MAC_NN;
			}
			else if (agent.indexOf("Netscape") >= 0 && agent.indexOf("Windows") >= 0)
			{
				m_browserId = WIN_NN;
			}
			else if (agent.indexOf("MSIE") >= 0 && agent.indexOf("Mac") >= 0)
			{
				m_browserId = MAC_IE;
			}
			else if (agent.indexOf("MSIE") >= 0 && agent.indexOf("Windows") >= 0)
			{
				m_browserId = WIN_IE;
			}
			else if (agent.indexOf("Camino") >= 0 && agent.indexOf("Macintosh") >= 0)
			{
				m_browserId = MAC_CM;
			}
			else if (agent.startsWith("Mozilla") && agent.indexOf("Windows") >= 0)
			{
				m_browserId = WIN_MZ;
			}
			else if (agent.startsWith("Mozilla") && agent.indexOf("Macintosh") >= 0)
			{
				m_browserId = MAC_MZ;
			}
			else
			{
				m_browserId = UNKNOWN;
			}

		} // setBrowserId

		/**
		* Close the session.
		*/
		protected void close()
		{
			if (!isClosed())
			{
				m_end = m_timeService.newTime();
				m_storage.closeSession(this);
			}

		} // close

		/**
		* Force the session to be closed.
		*/
		public void forceClosed()
		{
			close();

		} // forceClosed

		/**
		* Is this session closed?
		* @return true if the session is closed, false if open.
		*/
		public boolean isClosed()
		{
			return (!(m_end.equals(m_start)));

		} // isClosed

		/**
		* Access the user object for this session.
		* @return the user object for this session.
		*/
		public User getUser()
		{
			return m_user;
		}

		/**
		* Access the unique id for this session.
		* @return the unique id for this session.
		*/
		public String getId()
		{
			return m_id;
		}

		/**
		* Access the server id which is hosting this session.
		* @return the server id which is hosting this session.
		*/
		public String getServer()
		{
			return m_server;
		}

		/**
		* Access the IP Address from which this session originated.
		* @return the IP Address from which this session originated.
		*/
		public String getIpAddress()
		{
			return m_ip;
		}

		/**
		* Access the User Agent string describing the browser used in this session.
		* @return the User Agent string describing the browser used in this session.
		*/
		public String getUserAgent()
		{
			return m_userAgent;
		}

		/**
		* Access a short string describing the class of browser used in this session.
		* @return the short ID describing the browser used in this session.
		*/
		public String getBrowserId()
		{
			return m_browserId;
		}

		/**
		* Access the start time of the session
		* @return The time the session started.
		*/
		public Time getStart()
		{
			return m_timeService.newTime(m_start.getTime());

		} // getStart

		/**
		* Access the end time of the session.
		* @return The time the session ended.  If still going, this will .equals() the getStart() value.
		*/
		public Time getEnd()
		{
			return m_timeService.newTime(m_end.getTime());

		} // getEnd

		/**
		* Accept notification that this object has been bound as a SessionState attribute.
		* @param sessionStateKey The id of the session state which holds the attribute.
		* @param attributeName The id of the attribute to which this object is now the value.
		*/
		public void valueBound(String sessionStateKey, String attributeName)
		{
		}

		/**
		* Accept notification that this object has been removed from a SessionState attribute.
		* @param sessionStateKey The id of the session state which held the attribute.
		* @param attributeName The id of the attribute to which this object was the value.
		*/
		public void valueUnbound(String sessionStateKey, String attributeName)
		{
			// track the event
			Event event = m_eventTrackingService.newEvent(EVENT_LOGOUT, null, true);
			m_eventTrackingService.post(event, this);

			// retire the presence for this session
			PresenceService.removePresence(this);

			close();
		}

		/**
		* Compare this object with the specified object for order.
		* @return A negative integer, zero, or a positive integer as this object is
		* less than, equal to, or greater than the specified object.
		*/
		public int compareTo(Object obj)
		{
			if (!(obj instanceof UsageSession))
				throw new ClassCastException();

			// if the object are the same, say so
			if (obj == this)
				return 0;

			// start the compare by comparing their users
			int compare = getUser().compareTo(((UsageSession) obj).getUser());

			// if these are the same
			if (compare == 0)
			{
				// sort based on (unique) id
				compare = getId().compareTo(((UsageSession) obj).getId());
			}

			return compare;

		} // compareTo

		public String toString()
		{
			return "["
				+ ((m_id == null) ? "" : m_id)
				+ " | "
				+ ((m_server == null) ? "" : m_server)
				+ " | "
				+ ((m_user == null) ? "" : m_user.getId())
				+ " | "
				+ ((m_ip == null) ? "" : m_ip)
				+ " | "
				+ ((m_userAgent == null) ? "" : m_userAgent)
				+ " | "
				+ m_start.toStringGmtFull()
				+ " ]";
		}

	} // class BaseUsageSession

} // BaseUsageSessionService

/**********************************************************************************
*
* $Header: /cvs/sakai/framework-component/src/java/org/sakaiproject/component/framework/session/BaseUsageSessionService.java,v 1.18 2004/06/22 03:13:32 ggolden Exp $
*
**********************************************************************************/
