/**********************************************************************************
*
* $Header: /cvs/sakai/framework-component/src/java/org/sakaiproject/component/framework/session/ClusterUsageSessionService.java,v 1.10 2004/06/22 03:13:32 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

// package
package org.sakaiproject.component.framework.session;

// imports
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import org.sakaiproject.exception.IdUnusedException;
import org.sakaiproject.service.legacy.event.Event;
import org.sakaiproject.service.legacy.presence.cover.PresenceService;
import org.sakaiproject.service.framework.session.UsageSession;
import org.sakaiproject.service.framework.sql.SqlReader;
import org.sakaiproject.service.framework.sql.SqlService;
import org.sakaiproject.service.legacy.time.Time;
import org.sakaiproject.service.legacy.user.User;
import org.sakaiproject.service.legacy.user.cover.UserDirectoryService;

/**
* <p>ClusterUsageSessionService is a CHEF usage session service that is backed by an external database, so can be
* used in a Cluster.</p>
* <p>A Maintenance thread watches cluster records in the database looking for ones that have stopped being updated
* indicating a failed app server, and cleans up these, their sessions and presence.</p>
* 
* @author University of Michigan, CHEF Software Development Team
* @version $Revision: 1.10 $
*/
public class ClusterUsageSessionService
	extends BaseUsageSessionService
{
	/** The maintenance. */
	protected Maintenance m_maintenance = null;

	/** How often to register that we are alive with the cluster table (seconds). */
	protected static final long REFRESH = 60;
	protected long m_refresh = REFRESH;

	/** App servers who have not checked in in this many seconds are considered dead. */
	protected static final long EXPIRED = 600;
	protected long m_expired = EXPIRED;

	/*******************************************************************************
	* Constructors, Dependencies and their setter methods
	*******************************************************************************/

	/**
	 * Configuration: set the refresh value
	 * @param value The refresh value.
	 */
	public void setRefresh(String value)
	{
		try
		{
			m_refresh = Long.parseLong(value);
		}
		catch (Exception ignore) {}
	}

	/**
	 * Configuration: set the expired value
	 * @param value The expired value.
	 */
	public void setExpired(String value)
	{
		try
		{
			m_expired = Long.parseLong(value);
		}
		catch (Exception ignore) {}
	}

	/** Dependency: SqlService. */
	protected SqlService m_sqlService = null;

	/**
	 * Dependency: SqlService.
	 * @param service The SqlService.
	 */
	public void setSqlService(SqlService service)
	{
		m_sqlService = service;
	}

	/*******************************************************************************
	* Init and Destroy
	*******************************************************************************/

	/**
	 * Final initialization, once all dependencies are set.
	 */
	public void init()
	{
		try
		{
			super.init();

			// start the maintenance thread
			m_maintenance = new Maintenance();
			m_maintenance.start();

			m_logger.info(this + "init: refresh: " + m_refresh + " expired: " + m_expired);
		}
		catch (Throwable t)
		{
			m_logger.warn(this +".init(): ", t);
		}
	}

	/**
	 * Final cleanup.
	 */
	public void destroy()
	{
		m_maintenance.stop();
		m_maintenance = null;

		super.destroy();
	}

	/*******************************************************************************
	* Abstractions, etc.
	*******************************************************************************/

	/**
	* Construct storage for this service.
	*/
	protected Storage newStorage() { return new ClusterStorage(); }

	/*******************************************************************************
	* Storage
	*******************************************************************************/

	protected class ClusterStorage
		implements Storage
	{
		/** The locally kept sessions, keyed by id. */
		protected Map m_sessions = new Hashtable();

		/**
		* Open and be ready to read / write.
		*/
		public void open()
		{
		}	// open

		/**
		* Close.
		*/
		public void close()
		{
			// close any sessions still open
			List sessions = new Vector();
			sessions.addAll(m_sessions.values());
			Iterator it = sessions.iterator();
			while (it.hasNext())
			{
				BaseUsageSession session = (BaseUsageSession) it.next();
				session.close();
			}

			if (!m_sessions.isEmpty())
			{
				m_logger.warn(this + ".close(): sessions remain");
			}

			m_sessions.clear();

		}	// close

		/**
		* Take this session into storage.
		* @param session The usage session.
		*/
		public void addSession(UsageSession session)
		{
			// keep it here
			m_sessions.put(session.getId(), session);

			// and store it in the db
			String statement =
					"insert into CHEF_SESSION (SESSION_ID,SESSION_SERVER,SESSION_USER,SESSION_IP,SESSION_USER_AGENT,SESSION_START,SESSION_END) values (?, ?, ?, ?, ?, ?, ?)";

			// collect the fields
			Object fields[] = new Object[7];
			fields[0] = session.getId();
			fields[1] = session.getServer();
			fields[2] = session.getUser().getId();
			fields[3] = session.getIpAddress();
			fields[4] = session.getUserAgent();
			fields[5] = session.getStart();
			fields[6] = session.getEnd();

			// process the insert
			boolean ok = m_sqlService.dbWrite(statement, fields);
			if (!ok)
			{
				m_logger.warn(this + ".addSession(): dbWrite failed");
			}

		}	// addSession

		/**
		* Access a session by id
		* @param id The session id.
		* @return The session object.
		*/
		public UsageSession getSession(String id)
		{
			// look here
			UsageSession rv = (UsageSession) m_sessions.get(id);

			// if not local
			if (rv == null)
			{
				// check the db for new events
				String statement =
					"select SESSION_ID,SESSION_SERVER,SESSION_USER,SESSION_IP,SESSION_USER_AGENT,SESSION_START,SESSION_END from CHEF_SESSION where SESSION_ID = ?";

				// send in the last seq number parameter
				Object[] fields = new Object[1];
				fields[0] = id;

				List sessions = m_sqlService.dbRead(statement, fields,
					new SqlReader()
					{
						public Object readSqlResultRecord(ResultSet result)
						{
							try
							{
								// read the UsageSession
								String id =  result.getString(1);
								String server = result.getString(2);
								String userId = result.getString(3);
								User user = null;
								try
								{
									user = UserDirectoryService.getUser(userId);
								}
								catch (IdUnusedException e)
								{
									user = UserDirectoryService.getAnonymousUser();
								}
								String ip = result.getString(4);
								String agent = result.getString(5);
								Time start = m_timeService.newTime(result.getTimestamp(6, m_sqlService.getCal()).getTime());
								Time end = m_timeService.newTime(result.getTimestamp(7, m_sqlService.getCal()).getTime());
								
								UsageSession session = new BaseUsageSession(id, server, user, ip, agent, start, end);
								return session;
							}
							catch (SQLException ignore) { return null;}
						}
					} );

				if (!sessions.isEmpty()) rv = (UsageSession) sessions.get(0);
			}

			return rv;

		}	// getSession

		/**
		* Access a local (i.e. maintained in this app server) session by id
		* @param id The session id.
		* @return The session object.
		*/
		public UsageSession getLocalSession(String id)
		{
			// look here only
			UsageSession rv = (UsageSession) m_sessions.get(id);
			return rv;

		}	// getLocalSession

		/**
		* This session is now closed.
		* @param session The session which is closed.
		*/
		public void closeSession(UsageSession session)
		{
			// if not local (that would be strange...) skip
			if (m_sessions.get(session.getId()) == null) return;

			// forget it
			m_sessions.remove(session.getId());

			// to record the end date and closed status, update the db
			// and store it in the db
			String statement =
					"update CHEF_SESSION set SESSION_END = ? where SESSION_ID = ?";
	
			// collect the fields
			Object fields[] = new Object[2];
			fields[0] = session.getEnd();
			fields[1] = session.getId();

			// process the statement
			boolean ok = m_sqlService.dbWrite(statement, fields);
			if (!ok)
			{
				m_logger.warn(this + ".closeSession(): dbWrite failed");
			}

		}	// closeSession

	}   // Storage


	/*******************************************************************************
	* Maintenance
	*******************************************************************************/

	protected class Maintenance
		implements Runnable
	{
		/** My thread running my timeout checker. */
		protected Thread m_maintenanceChecker = null;
		
		/** Signal to the timeout checker to stop. */
		protected boolean m_maintenanceCheckerStop = false;

		/**
		* Construct.
		*/
		public Maintenance()
		{
		}	// Maintenance

		/**
		* Start the maintenance thread, registering this app server in the cluster table.
		*/
		public void start()
		{
			if (m_maintenanceChecker != null) return;

			// register in the cluster table
			String statement =
					"insert into CHEF_CLUSTER (SERVER_ID,START_TIME,END_TIME,UPDATE_TIME) values (?, ?, ?, ?)";
			Object fields[] = new Object[4];
			fields[0] = m_serverConfigurationService.getServerIdInstance();
			fields[1] = m_timeService.newTime();
			fields[2] = fields[1];
			fields[3] = fields[1];
			boolean ok = m_sqlService.dbWrite(statement, fields);
			if (!ok)
			{
				m_logger.warn(this + ".start(): dbWrite failed");
			}

			m_maintenanceChecker = new Thread(this, "BaseUsageSessionService.Maintenance");
			m_maintenanceCheckerStop = false;
			m_maintenanceChecker.start();

		}	// start

		/**
		* Stop the maintenance thread, removing this app server's registration from the cluster table.
		*/
		public void stop()
		{
			if (m_maintenanceChecker != null)
			{
				m_maintenanceCheckerStop = true;
				m_maintenanceChecker.interrupt();
				try
				{
					// wait for it to die
					m_maintenanceChecker.join();
				}
				catch (InterruptedException ignore) {}
				m_maintenanceChecker = null;
			}

			// close our entry from the database
			String statement =
					"update CHEF_CLUSTER set END_TIME = ? where SERVER_ID = ?";
			Object fields[] = new Object[2];
			fields[0] = m_timeService.newTime();
			fields[1] = m_serverConfigurationService.getServerIdInstance();
			boolean ok = m_sqlService.dbWrite(statement, fields);
			if (!ok)
			{
				m_logger.warn(this + ".stop(): dbWrite failed");
			}

		}	// stop

		/**
		* Run the maintenance thread.
		* Every REFRESH seconds, re-register this app server as alive in the cluster.
		* Then check for any cluster entries that are more than EXPIRED seconds old, indicating a failed
		* app server, and remove that record, that server's sessions, and presence, generating appropriate
		* session and presence events so the other app servers know what's going on.
		*/
		public void run()
		{
			if (m_logger.isDebugEnabled())
				m_logger.debug(this + ".run()");

			while (!m_maintenanceCheckerStop)
			{
				try
				{
					if (m_logger.isDebugEnabled())
						m_logger.debug(this + ".checking...");

					final long now = System.currentTimeMillis();

					// see if we have been closed!
					String statement =
							"select SERVER_ID from CHEF_CLUSTER where START_TIME != END_TIME and SERVER_ID = ?";
					Object[] fields = new Object[1];
					fields[0] = m_serverConfigurationService.getServerIdInstance();
					List results = m_sqlService.dbRead(statement, fields, null);
					if (!results.isEmpty())
					{
						m_logger.warn(this + ".run(): server has been closed in cluster table, reopened: " + m_serverConfigurationService.getServerIdInstance());
					}

					// register that this app server is alive and well
					statement =
							"update CHEF_CLUSTER set END_TIME = START_TIME, UPDATE_TIME = ? where SERVER_ID = ?";
					fields = new Object[2];
					fields[0] = m_timeService.newTime(now);
					fields[1] = m_serverConfigurationService.getServerIdInstance();
					boolean ok = m_sqlService.dbWrite(statement, fields);
					if (!ok)
					{
						m_logger.warn(this + ".run(): dbWrite failed");
					}

					// get all expired open app servers
					statement =
							"select SERVER_ID, UPDATE_TIME from CHEF_CLUSTER where START_TIME = END_TIME";
					List instances = m_sqlService.dbRead(statement, null,
						new SqlReader()
						{
							public Object readSqlResultRecord(ResultSet result)
							{
								try
								{
									// read
									String serverId = result.getString(1);
									Time time = m_timeService.newTime(result.getTimestamp(2, m_sqlService.getCal()).getTime());
									
									// if the time is more than so old, select it
									if (now - (time.getTime()) > (m_expired * 1000L))
									{
										return serverId;
									}

									return null;
								}
								catch (SQLException ignore) { return null;}
							}
						} );

					// close any severs found to be expired
					for (Iterator iInstances = instances.iterator(); iInstances.hasNext();)
					{
						String serverId = (String) iInstances.next();

						// mark the end time with a time = 0 to show it was closed in this way
						statement =
								"update CHEF_CLUSTER set END_TIME = ? where SERVER_ID = ?";
						fields = new Object[2];
						fields[0] = m_timeService.newTime(0);
						fields[1] = serverId;		
						ok = m_sqlService.dbWrite(statement, fields);
						if (!ok)
						{
							m_logger.warn(this + ".run(): dbWrite failed");
						}

						m_logger.warn(this + ".run(): ghost-busting server: " + serverId + " from : " + m_serverConfigurationService.getServerIdInstance());
					}

					// find all the session ids of sessions that are open but are from closed servers
					statement =
								"select SESSION_ID from CHEF_SESSION where SESSION_START = SESSION_END "
							+	"and SESSION_SERVER in "
							+	"(select SERVER_ID from CHEF_CLUSTER where START_TIME != END_TIME)";
					List sessions = m_sqlService.dbRead(statement);

					// process each session to close it and loose it's presence
					for (Iterator iSessions = sessions.iterator(); iSessions.hasNext();)
					{
						String sessionId = (String) iSessions.next();
						
						// get all the presence for this session
						statement =
								"select LOCATION_ID from CHEF_PRESENCE where SESSION_ID = ?";
						fields = new Object[1];
						fields[0] = sessionId;		
						List presence = m_sqlService.dbRead(statement, fields, null);
						
						// remove all the presence for this session
						statement =
								"delete from CHEF_PRESENCE where SESSION_ID = ?";			
						ok = m_sqlService.dbWrite(statement, fields);
						if (!ok)
						{
							m_logger.warn(this + ".run(): dbWrite failed");
						}

						// get the session
						UsageSession session = getSession(sessionId);

						// send presence end events for these
						for (Iterator iPresence = presence.iterator(); iPresence.hasNext();)
						{
							String locationId = (String) iPresence.next();

							Event event = m_eventTrackingService.newEvent(PresenceService.EVENT_ABSENCE,
									PresenceService.presenceReference(locationId), true);
							m_eventTrackingService.post(event, session);
						}

						// a session closed event (logout)
						Event event = m_eventTrackingService.newEvent(EVENT_LOGOUT, null, true);
						m_eventTrackingService.post(event, session);

						// close this session on the db
						// mark the end time with a time = 0 to show it was closed in this way
						statement =
								"update CHEF_SESSION set SESSION_END = ? where SESSION_ID = ?";
						fields = new Object[2];
						fields[0] = m_timeService.newTime(0);
						fields[1] = sessionId;		
						ok = m_sqlService.dbWrite(statement, fields);
						if (!ok)
						{
							m_logger.warn(this + ".run(): dbWrite failed");
						}
					}

					// %%% do we need to check for presence from closed sessions?
					//select <> from CHEF_PRESENCE where SESSION_ID in
					//	(select SESSION_ID from CHEF_SESSION where SESSION_START != SESSION_END)
					// -ggolden
					
				}
				catch (Throwable e) {m_logger.warn(this + ": exception: ", e);}

				// cycle every REFRESH seconds
				try
				{
					Thread.sleep(m_refresh * 1000L);
				}
				catch (Exception ignore) {}
			}

			if (m_logger.isDebugEnabled())
				m_logger.debug(this + ": done");
			
		}	// run

	}	// class Maintenance

}   // ClusterUsageSessionService

/**********************************************************************************
*
* $Header: /cvs/sakai/framework-component/src/java/org/sakaiproject/component/framework/session/ClusterUsageSessionService.java,v 1.10 2004/06/22 03:13:32 ggolden Exp $
*
**********************************************************************************/
