/**********************************************************************************
*
* $Header: /cvs/sakai/framework-component/src/java/org/sakaiproject/component/framework/sql/BasicSqlService.java,v 1.8 2004/06/22 03:13:32 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

// package
package org.sakaiproject.component.framework.sql;

// imports
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.StringReader;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.TimeZone;
import java.util.Vector;

import org.apache.commons.pool.PoolableObjectFactory;
import org.apache.commons.pool.impl.GenericObjectPool;
import org.sakaiproject.exception.IdUnusedException;
import org.sakaiproject.exception.IdUsedException;
import org.sakaiproject.service.framework.config.ServerConfigurationService;
import org.sakaiproject.service.framework.log.Logger;
import org.sakaiproject.service.framework.session.UsageSession;
import org.sakaiproject.service.framework.session.cover.UsageSessionService;
import org.sakaiproject.service.framework.sql.ConnectionPool;
import org.sakaiproject.service.framework.sql.SqlReader;
import org.sakaiproject.service.framework.sql.SqlService;
import org.sakaiproject.service.legacy.time.Time;
import org.sakaiproject.service.legacy.user.User;

/**
* <p>BasicSqlService ...</p>
*
* @author University of Michigan, CHEF Software Development Team
* @version $Revision: 1.8 $
*/
public class BasicSqlService implements SqlService
{
	/** Default pool's real id. */
	protected final static String DEFAULT_POOL = "sakai";

	/** Map of pools, keyed by pool id. */
	protected Map m_pools = null;

	/** The pool that is the default pool. */
	protected ConnectionPool m_defaultPool = null;

	/** The driver string for the default pool. */
	protected String m_driver = null;

	/** The connect string for the default pool. */
	protected String m_connect = null;

	/** The user id string for the default pool. */
	protected String m_user = null;

	/** The pw string for the default pool. */
	protected String m_pw = null;

	/*******************************************************************************
	* Dependencies and their setter methods
	*******************************************************************************/

	/** Dependency: logging service */
	protected Logger m_logger = null;

	/**
	 * Dependency: logging service.
	 * @param service The logging service.
	 */
	public void setLogger(Logger service)
	{
		m_logger = service;
	}

	/** Dependency: ServerConfigurationService. */
	protected ServerConfigurationService m_serverConfigurationService = null;

	/**
	 * Dependency: ServerConfigurationService.
	 * @param service The ServerConfigurationService.
	 */
	public void setServerConfigurationService(ServerConfigurationService service)
	{
		m_serverConfigurationService = service;
	}

	/**
	 * Configuration: Set the driver string for the default pool.
	 * @param value the driver string for the default pool.
	 */
	public void setDriver(String value)
	{
		m_driver = value;
	}

	/**
	 * Configuration: Set the connect string for the default pool.
	 * @param value the connect string for the default pool.
	 */
	public void setConnect(String value)
	{
		m_connect = value;
	}

	/**
	 * Configuration: Set the user id string for the default pool.
	 * @param value the user id string for the default pool.
	 */
	public void setUser(String value)
	{
		m_user = value;
	}

	/**
	 * Configuration: Set the pw string for the default pool.
	 * @param value the user pw for the default pool.
	 */
	public void setPw(String value)
	{
		m_pw = value;
	}

	/*******************************************************************************
	* Init and Destroy
	*******************************************************************************/

	/**
	 * Final initialization, once all dependencies are set.
	 */
	public void init()
	{
		// pick up settings from config, if present
		m_driver = m_serverConfigurationService.getString("sql.driver", m_driver);
		m_connect = m_serverConfigurationService.getString("sql.connect", m_connect);
		m_user = m_serverConfigurationService.getString("sql.user", m_user);
		m_pw = m_serverConfigurationService.getString("sql.pw", m_pw);

		try
		{
			// create a map to hold pools
			m_pools = new HashMap();

			// create a default pool
			ConnectionPool pool = newPool(DEFAULT_POOL);
			pool.setConnect(m_driver, m_connect, m_user, m_pw);
			setDefaultPool(DEFAULT_POOL);

			m_logger.info(this +".init(): " + m_driver + " " + m_connect + " " + m_user);
		}
		catch (Throwable t)
		{
			m_logger.warn(this +".init(): exception: " + t);
		}
	}

	/**
	 * Final cleanup.
	 */
	public void destroy()
	{
		m_logger.info(this +".destroy()");
	}

	/*******************************************************************************
	* Work interface methods: org.sakaiproject.sql.SqlService
	*******************************************************************************/

	/**
	 * {@inheritdoc}
	 */
	public Connection borrowConnection() throws SQLException
	{
		if (m_defaultPool != null)
		{
			return m_defaultPool.borrowConnection();
		}
		else
		{
			throw new SQLException("no default pool.");
		}
	}

	/**
	 * {@inheritdoc}
	 */
	public void returnConnection(Connection conn)
	{
		if (m_defaultPool != null)
		{
			m_defaultPool.returnConnection(conn);
		}
	}

	/**
	 * {@inheritdoc}
	 */
	public ConnectionPool newPool(String poolId) throws IdUsedException
	{
		try
		{
			getPool(poolId);
			throw new IdUsedException(poolId);
		}
		catch (IdUnusedException e)
		{
		}

		// create a new pool object
		MyConnectionPool pool = new MyConnectionPool(poolId);

		// save it by id
		synchronized (m_pools)
		{
			m_pools.put(pool.getId(), pool);
		}

		return pool;
	}

	/**
	 * {@inheritdoc}
	 */
	public ConnectionPool getPool(String poolId) throws IdUnusedException
	{
		// handle default
		if (DEFAULT.equals(poolId))
		{
			if (m_defaultPool != null)
			{
				return m_defaultPool;
			}
			else
			{
				throw new IdUnusedException(poolId);
			}
		}

		// handle non-default names
		ConnectionPool rv = null;
		synchronized (m_pools)
		{
			rv = (ConnectionPool) m_pools.get(poolId);
		}

		if (rv == null)
		{
			throw new IdUnusedException(poolId);
		}

		return rv;
	}

	/**
	 * {@inheritdoc}
	 */
	public List getPools()
	{
		List rv = new Vector();

		synchronized (m_pools)
		{
			for (Iterator iEntries = m_pools.entrySet().iterator(); iEntries.hasNext();)
			{
				Map.Entry entry = (Map.Entry) iEntries.next();
				rv.add(entry.getValue());
			}
		}

		return rv;
	}

	/**
	 * {@inheritdoc}
	 */
	public void deletePool(String poolId) throws IdUnusedException
	{
		// get to throw the exception if missing
		ConnectionPool pool = getPool(poolId);

		// clear the pool
		pool.clear();

		// forget about it
		synchronized (m_pools)
		{
			m_pools.remove(poolId);
		}

		// if this was the default pool, unset default		
		if (m_defaultPool != null)
		{
			if (m_defaultPool.getId().equals(poolId))
			{
				m_defaultPool = null;
			}
		}

	}

	/**
	 * {@inheritdoc}
	 */
	public void setDefaultPool(String poolId) throws IdUnusedException
	{
		// get to throw the exception if missing
		ConnectionPool pool = getPool(poolId);

		m_defaultPool = pool;
	}

	// control time debug statements
	protected final boolean DEBUG = false;

	/** Name of the database pool to use for requests. */
	protected final String POOL_NAME = "default";

	/** Used to work with dates in GMT in the db. */
	protected final GregorianCalendar m_cal = new GregorianCalendar(TimeZone.getTimeZone("GMT"));

	/**
	 * {@inheritDoc}
	 */
	public GregorianCalendar getCal()
	{
		return m_cal;
	}

	/**
	* Read a single field from the db, from multiple records, returned as string[], one per record.
	* @param sql The sql statement.
	* @return The List of Strings of single fields of the record found, or empty if none found.
	*/
	public List dbRead(String sql)
	{
		return dbRead(sql, null, null);

	}	// dbRead

	/**
	* Process a query, filling in with fields, and return the results as a List, one per record read.
	* If a reader is provided, it will be called for each record to prepare the Object placed into the List.
	* Otherwise, the first field of each record, as a String, will be placed in the list.
	* @param sql The sql statement.
	* @param fields The array of fields for parameters.
	* @param reader The reader object to read each record.
	* @return The List of things read, one per record.
	*/
	public List dbRead(String sql, Object[] fields, SqlReader reader)
	{
		// for DEBUG
		long start = 0;
		long connectionTime = 0;
		long statementTime = 0;
		int lenRead = 0;

		if (m_logger.isDebugEnabled())
		{
			String userId = safelyGetSessionUserId();
			StringBuffer buf = new StringBuffer();
			if (fields != null)
			{
				buf.append(fields[0]);
				for (int i = 1; i < fields.length; i++)
				{
					buf.append(", ");
					buf.append(fields[i]);
				}
			}
			m_logger.debug("Sql.dbRead: " + userId + "\n" + sql + "\n" + buf);
		}

		Connection conn = null;
		PreparedStatement pstmt = null;
		ResultSet result = null;
		ResultSetMetaData meta = null;
		List rv = new Vector();

		try
		{
			if (DEBUG) start = System.currentTimeMillis();
			conn = borrowConnection();
			if (DEBUG) connectionTime = System.currentTimeMillis() - start;
			if (DEBUG) start = System.currentTimeMillis();

			pstmt = conn.prepareStatement(sql);

			// put in all the fields
			prepareStatement(pstmt, fields);

			result = pstmt.executeQuery();

			if (DEBUG) statementTime = System.currentTimeMillis()-start;
			while (result.next())
			{
				// without a reader, we read the first String from each record
				if (reader == null)
				{
					String s = result.getString(1);
					rv.add(s);
				}
				else
				{
					Object obj = reader.readSqlResultRecord(result);
					if (obj != null)
					{
						rv.add(obj);
					}
				}
			}
		}
		catch (Exception e)
		{
			m_logger.warn("Sql.dbRead(): ", e);
		}
		finally
		{
			try
			{
				if (null != result) result.close();
				if (null != pstmt) pstmt.close();
				if (null != conn) returnConnection(conn);
			}
			catch (Exception e)
			{
				m_logger.warn("Sql.dbRead: ", e);
			}
		}

		if (DEBUG) m_logger.info("Sql.dbRead:  connection: "
						+ connectionTime + "  statement: " + statementTime);

		return rv;

	}	// dbRead

	/**
	* Process a query, filling in with fields, and return the results as a List, one per record read.
	* If a reader is provided, it will be called for each record to prepare the Object placed into the List.
	* Otherwise, the first field of each record, as a String, will be placed in the list.
	* @param conn The db connection object to use.
	* @param sql The sql statement.
	* @param fields The array of fields for parameters.
	* @param reader The reader object to read each record.
	* @return The List of things read, one per record.
	*/
	public List dbRead(Connection conn, String sql, Object[] fields, SqlReader reader)
	{
		// for DEBUG
		long start = 0;
		long statementTime = 0;
		int lenRead = 0;

		if (m_logger.isDebugEnabled())
		{
			String userId = safelyGetSessionUserId();
			StringBuffer buf = new StringBuffer();
			if (fields != null)
			{
				buf.append(fields[0]);
				for (int i = 1; i < fields.length; i++)
				{
					buf.append(", ");
					buf.append(fields[i]);
				}
			}
			m_logger.debug("Sql.dbRead: " + userId + "\n" + sql + "\n" + buf);
		}

		PreparedStatement pstmt = null;
		ResultSet result = null;
		ResultSetMetaData meta = null;
		List rv = new Vector();

		try
		{
			if (DEBUG) start = System.currentTimeMillis();

			pstmt = conn.prepareStatement(sql);

			// put in all the fields
			prepareStatement(pstmt, fields);

			result = pstmt.executeQuery();

			if (DEBUG) statementTime = System.currentTimeMillis()-start;
			while (result.next())
			{
				// without a reader, we read the first String from each record
				if (reader == null)
				{
					String s = result.getString(1);
					rv.add(s);
				}
				else
				{
					Object obj = reader.readSqlResultRecord(result);
					if (obj != null)
					{
						rv.add(obj);
					}
				}
			}
		}
		catch (Exception e)
		{
			m_logger.warn("Sql.dbRead(): ", e);
		}
		finally
		{
			try
			{
				if (null != result) result.close();
				if (null != pstmt) pstmt.close();
				//if (null != conn) conn.close();
			}
			catch (Exception e)
			{
				m_logger.warn("Sql.dbRead: ", e);
			}
		}

		if (DEBUG) m_logger.info("Sql.dbRead:  statement: " + statementTime);

		return rv;

	}	// dbRead

	/**
	* Read a single field from the db, from multiple record - concatenating the binary values into value.
	* @param sql The sql statement.
	* @param value The array of bytes to fill with the value read from the db.
	*/
	public void dbReadBinary(String sql, byte[] value)
	{
		// for DEBUG
		long start = 0;
		long connectionTime = 0;
		long statementTime = 0;
		int lenRead = 0;

		if (m_logger.isDebugEnabled())
		{
			String userId = safelyGetSessionUserId();
			m_logger.debug("Sql.dbReadBinary(): " + userId + "\n" + sql);
		}

		Connection conn = null;
		Statement stmt = null;
		ResultSet result = null;
		ResultSetMetaData meta = null;

		try
		{
			if (DEBUG) start = System.currentTimeMillis();
			conn = borrowConnection();
			if (DEBUG) connectionTime = System.currentTimeMillis() - start;
			if (DEBUG) start = System.currentTimeMillis();
			stmt = conn.createStatement();
			result = stmt.executeQuery(sql);
			if (DEBUG) statementTime = System.currentTimeMillis()-start;
			int index = 0;
			while (result.next() && (index < value.length))
			{
				InputStream stream = result.getBinaryStream(1);
				int len = stream.read(value, index, value.length-index);
				stream.close();
				index += len;
				if (DEBUG) lenRead += len;
			}
		}
		catch (Exception e)
		{
			m_logger.warn("Sql.dbReadBinary(): " + e);
		}
		finally
		{
			try
			{
				if (null != result) result.close();
				if (null != stmt) stmt.close();
				if (null != conn) returnConnection(conn);
			}
			catch (Exception e)
			{
				m_logger.warn("Sql.dbReadBinary(): " + e);
			}
		}

		if (DEBUG) m_logger.info("sql read binary: len: "  + lenRead + "  connection: " + connectionTime + "  statement: " + statementTime);

	}	// dbReadBinary 

	/**
	* Execute the "write" sql - no response.
	* @param sql The sql statement.
	* @return true if successful, false if not.
	*/
	public boolean dbWrite(String sql)
	{
		return dbWrite(sql, null, null);

	}	// dbWrite

	/**
	* Execute the "write" sql - no response.
	* a long field is set to "?" - fill it in with var
	* @param sql The sql statement.
	* @param var The value to bind to the first parameter in the sql statement.
	* @return true if successful, false if not.
	*/
	public boolean dbWrite(String sql, String var)
	{
		return dbWrite(sql, null, var);

	}	// dbWrite

	/**
	* Execute the "write" sql - no response.
	* a long binary field is set to "?" - fill it in with var
	* @param sql The sql statement.
	* @param var The value to bind to the first parameter in the sql statement.
	* @param offset The start within the var to write
	* @param len The number of bytes of var, starting with index, to write
	* @return true if successful, false if not.
	*/
	public boolean dbWriteBinary(String sql, byte[] var, int offset, int len)
	{
		// for DEBUG
		long start = 0;
		long connectionTime = 0;
		long statementTime = 0;

		if (m_logger.isDebugEnabled())
		{
			String userId = safelyGetSessionUserId();
			m_logger.debug("Sql.dbWriteBinary(): " + userId + "\n" + sql + "  size:" + var.length);
		}

		Connection conn = null;
		PreparedStatement pstmt = null;
		boolean autoCommit = true;
		boolean resetAutoCommit = false;

		// stream from the var
		InputStream varStream = new ByteArrayInputStream(var, offset, len);

		try
		{
			if (DEBUG) start = System.currentTimeMillis();
			conn = borrowConnection();
			if (DEBUG) connectionTime = System.currentTimeMillis()-start;
			
			// make sure we have auto commit - will change and reset if needed
			autoCommit = conn.getAutoCommit();
			if (!autoCommit)
			{
				conn.setAutoCommit(true);
				resetAutoCommit = true;
			}

			if (DEBUG) start = System.currentTimeMillis();
			pstmt = conn.prepareStatement(sql);
			pstmt.setBinaryStream(1, varStream, len);
			int result = pstmt.executeUpdate();
			if (DEBUG) statementTime = System.currentTimeMillis()-start;
		}
		catch (SQLException e)
		{
			// this is likely due to a key constraint problem...
			return false;
		}
		catch (Exception e)
		{
			m_logger.warn("Sql.dbWriteBinary(): " + e);
			return false;
		}
		finally
		{
			try
			{
				if (null != pstmt) pstmt.close();
				varStream.close();
				if (null != conn)
				{
					// if we changed the auto commit, reset here
					if (resetAutoCommit)
					{
						conn.setAutoCommit(autoCommit);
					}
					returnConnection(conn);
				}
			}
			catch (Exception e)
			{
				m_logger.warn("Sql.dbWriteBinary(): " + e);
			}
		}
		
		if (DEBUG) m_logger.info("sql write binary: len: "  + len + "  connection: " + connectionTime + "  statement: " + statementTime);

		return true;

	}	// dbWriteBinary

	/**
	* Execute the "write" sql - no response, using a set of fields from an array plus one more as params.
	* @param sql The sql statement.
	* @param fields The array of fields for parameters.
	* @return true if successful, false if not.
	*/
	public boolean dbWrite(String sql, Object[] fields)
	{
		return dbWrite(sql, fields, null);

	}	// dbWrite

	/**
	* Execute the "write" sql - no response, using a set of fields from an array plus one more as params.
	* @param sql The sql statement.
	* @param fields The array of fields for parameters.
	* @param lastField The value to bind to the last parameter in the sql statement.
	* @return true if successful, false if not.
	*/
	public boolean dbWrite(String sql, Object[] fields, String lastField)
	{
		// for DEBUG
		long start = 0;
		long connectionTime = 0;
		long statementTime = 0;

		if (m_logger.isDebugEnabled())
		{
			String userId = safelyGetSessionUserId();
			StringBuffer buf = new StringBuffer();
			if (fields != null)
			{
				buf.append(fields[0]);
				for (int i = 1; i < fields.length; i++)
				{
					buf.append(", ");
					buf.append(fields[i]);
				}
				if (lastField != null)
				{
					buf.append(", ");
					buf.append(lastField);
				}
			}
			else if (lastField != null)
			{
				buf.append(lastField);
			}
			m_logger.debug("Sql.dbWrite(): " + userId + "\n" + sql + "\n" + buf);
		}

		Connection conn = null;
		PreparedStatement pstmt = null;
		boolean autoCommit = true;
		boolean resetAutoCommit = false;

		try
		{
			if (DEBUG) start = System.currentTimeMillis();
			conn = borrowConnection();
			if (DEBUG) connectionTime = System.currentTimeMillis() - start;

			// make sure we have auto commit - will change and reset if needed
			autoCommit = conn.getAutoCommit();
			if (!autoCommit)
			{
				conn.setAutoCommit(true);
				resetAutoCommit = true;
			}

			if (DEBUG) start = System.currentTimeMillis();
			pstmt = conn.prepareStatement(sql);

			// put in all the fields
			int pos = prepareStatement(pstmt, fields);

			// last, put in the string value
			if (lastField != null)
			{
				pstmt.setCharacterStream(pos, new StringReader(lastField), lastField.length());
				pos++;
			}

			int result = pstmt.executeUpdate();
			if (DEBUG) statementTime = System.currentTimeMillis()-start;
		}
		catch (SQLException e)
		{
			// this is likely due to a key constraint problem...
			m_logger.warn("Sql.dbWrite(): " + e);
			return false;
		}
		catch (Exception e)
		{
			m_logger.warn("Sql.dbWrite(): " + e);
			return false;
		}
		finally
		{
			try
			{
				if (null != pstmt) pstmt.close();
				if (null != conn)
				{
					// if we changed the auto commit, reset here
					if (resetAutoCommit)
					{
						conn.setAutoCommit(autoCommit);
					}
					returnConnection(conn);
				}
			}
			catch (Exception e)
			{
				m_logger.warn("Sql.dbWrite(): " + e);
			}
		}

		if (DEBUG) m_logger.info("Sql.dbWrite(): len: "  + lastField.length() + "  connection: " + connectionTime + "  statement: " + statementTime);

		return true;

	}	// dbWrite

	/**
	* Read a single field BLOB from the db from one record, and update it's bytes
	* with content.
	* @param sql The sql statement to select the BLOB.
	* @param content The new bytes for the BLOB.
	*/
	public void dbReadBlobAndUpdate(String sql, byte[] content)
	{
		// for DEBUG
		long start = 0;
		long connectionTime = 0;
		long statementTime = 0;
		int lenRead = 0;

		if (m_logger.isDebugEnabled())
		{
			String userId = safelyGetSessionUserId();
			m_logger.debug("Sql.dbReadBlobAndUpdate(): " + userId + "\n" + sql);
		}


		Connection conn = null;
		Statement stmt = null;
		ResultSet result = null;
		ResultSetMetaData meta = null;
		Object blob = null;
		OutputStream os = null;

		try
		{
			if (DEBUG) start = System.currentTimeMillis();
			conn = borrowConnection();
			if (DEBUG) connectionTime = System.currentTimeMillis() - start;
			if (DEBUG) start = System.currentTimeMillis();
			stmt = conn.createStatement();
			result = stmt.executeQuery(sql);
			if (DEBUG) statementTime = System.currentTimeMillis()-start;
			if (result.next())
			{
				blob = result.getBlob(1);
			}
			if (blob != null)
			{
				// %%% not supported? b.truncate(0);
				// int len = b.setBytes(0, content);
				try
				{
					// Use reflection to remove compile time dependency on oracle driver
					Class[] paramsClasses = new Class[0];
					Method getBinaryOutputStreamMethod = blob.getClass().getMethod
								("getBinaryOutputStream", paramsClasses);
					Object[] params = new Object[0];
					os = (OutputStream) getBinaryOutputStreamMethod.invoke(blob, params);
					os.write(content);
					os.close();
				}
				catch (NoSuchMethodException ex)
				{
					m_logger.warn("Oracle driver error: " + ex);
				}
				catch (IllegalAccessException ex)
				{
					m_logger.warn("Oracle driver error: " + ex);
				}
				catch (InvocationTargetException ex)
				{
					m_logger.warn("Oracle driver error: " + ex);
				}
			}
		}
		catch (Exception e)
		{
			m_logger.warn("Sql.dbReadBlobAndUpdate(): " + e);
		}
		finally
		{
			try
			{
				if (null != result) result.close();
				if (null != stmt) stmt.close();
				if (null != conn) returnConnection(conn);
			}
			catch (Exception e)
			{
				m_logger.warn("Sql.dbRead(): " + e);
			}
		}

		if (DEBUG) m_logger.info("sql dbReadBlobAndUpdate: len: "  + lenRead + "  connection: "
						+ connectionTime + "  statement: " + statementTime);

	}	// dbReadBlobAndUpdate

	/**
	* Read a single field from the db, from a single record, return the value found, and lock for update. 
	* @param sql The sql statement.
	* @param field A StringBuffer that will be filled with the field.
	* @return The Connection holding the lock.
	*/
	public Connection dbReadLock(String sql, StringBuffer field)
	{
		Connection conn = null;
		Statement stmt = null;
		ResultSet result = null;
		boolean autoCommit = true;
		boolean resetAutoCommit = false;
		boolean closeConn = false;

		try
		{
			// get a new conncetion
			conn = borrowConnection();

			// adjust to turn off auto commit - we need a transaction
			autoCommit = conn.getAutoCommit();
			if (autoCommit)
			{
				conn.setAutoCommit(false);
				resetAutoCommit = true;
			}

			if (m_logger.isDebugEnabled())
				m_logger.debug("Sql.dbReadLock():\n" + sql);

			// create a statement and execute
			stmt = conn.createStatement();
			result = stmt.executeQuery(sql);

			// if we have a result record
			if (result.next())
			{
				// get the result and pack into the return buffer
				String rv = result.getString(1);
				if (field != null) field.append(rv);
			}

			// otherwise we fail
			else
			{
				closeConn = true;
			}
		}

		// this is likely the error when the record is otherwise locked - we fail
		catch (SQLException e)
		{
			// Note: ORA-00054 gives an e.getErrorCode() of 54, if anyone cares...
			//m_logger.warn("Sql.dbUpdateLock(): " + e.getErrorCode() + " - " + e);
			closeConn = true;
		}

		catch (Exception e)
		{
			m_logger.warn("Sql.dbReadLock(): " + e);
			closeConn = true;
		}

		finally
		{
			try
			{
				// close the result and statement
				if (null != result) result.close();
				if (null != stmt) stmt.close();
				
				// if we are failing, restore and release the connectoin
				if ((closeConn) && (conn != null))
				{
					// just in case we got a lock
					conn.rollback();
					if (resetAutoCommit) conn.setAutoCommit(autoCommit);
					returnConnection(conn);
					conn = null;
				}
			}
			catch (Exception e)
			{
				m_logger.warn("Sql.dbReadLock(): " + e);
			}
		}

		return conn;

	}	// dbReadLock

	/**
	* Commit the update that was locked on this connection.
	* @param sql The sql statement.
	* @param fields The array of fields for parameters.
	* @param var The value to bind to the last parameter in the sql statement.
	* @param conn The database connection on which the lock was gained.
	*/
	public void dbUpdateCommit(String sql, Object[] fields, String var, Connection conn)
	{
		PreparedStatement pstmt = null;

		try
		{
			if (m_logger.isDebugEnabled())
				m_logger.debug("Sql.dbUpdateCommit():\n" + sql);

			pstmt = conn.prepareStatement(sql);

			// put in all the fields
			int pos = prepareStatement(pstmt, fields);

			// prepare the update statement and fill with the last variable (if any)
			if (var != null)
			{
				pstmt.setCharacterStream(pos, new StringReader(var), var.length());
				pos++;
			}

			// run the SQL statement
			int result = pstmt.executeUpdate();
			pstmt.close();
			pstmt = null;

			// commit
			conn.commit();
		}
		catch (Exception e)
		{
			m_logger.warn("Sql.dbUpdateCommit(): " + e);
		}
		finally
		{
			try
			{
				// close the statemenet and restore / release the connection
				if (null != pstmt) pstmt.close();
				if (null != conn)
				{
					// we don't really know what this should be, but we assume the
					// default is autoCommit
					conn.setAutoCommit(true);
					returnConnection(conn);
				}
			}
			catch (Exception e)
			{
				m_logger.warn("Sql.dbUpdateCommit(): " + e);
			}
		}

	}	// dbUpdateCommit

	/**
	* Cancel the update that was locked on this connection.
	* @param conn The database connection on which the lock was gained.
	*/
	public void dbCancel(Connection conn)
	{
		boolean autoCommit = true;
		boolean resetAutoCommit = true;//%%%

		try
		{
			// cancel any changes, release any locks
			conn.rollback();
			
			// we don't really know what this should be, but we assume the
			// default is autoCommit
			conn.setAutoCommit(autoCommit);
			returnConnection(conn);
		}
		catch (Exception e)
		{
			m_logger.warn("Sql.dbCancel(): " + e);
		}

	}	// dbCancel

	/**
	* Prepare a prepared statement with fields.
	* @param pstmt The prepared statement to fill in.
	* @param fields The Object array of values to fill in.
	* @return the next pos that was not filled in.
	*/
	protected int prepareStatement(PreparedStatement pstmt, Object[] fields)
		throws SQLException
	{
		// put in all the fields
		int pos = 1;
		if ((fields != null) && (fields.length > 0))
		{
			for (int i = 0; i < fields.length; i++)
			{
				if (fields[i] instanceof Time)
				{
					Time t = (Time) fields[i];
					pstmt.setTimestamp(pos, new Timestamp(t.getTime()), m_cal);
					pos++;
				}
				else if (fields[i] instanceof Long)
				{
					long l = ((Long) fields[i]).longValue();
					pstmt.setLong(pos, l);
					pos++;
				}
				// %%% support any other types specially?
				else
				{
					String value = fields[i].toString();
					pstmt.setCharacterStream(pos, new StringReader(value), value.length());
					pos++;
				}
			}
		}

		return pos;

	}	// prepareStatement

	/**
	* Access the authenticated session user id for the current request's session - if there is any.
	* @return The user id object for the current authenticated user (or thread user), or "" if none.
	*/
	protected String safelyGetSessionUserId()
	{
		UsageSession session = UsageSessionService.getSession();
		if (session == null) return "";
		User user = session.getUser();
		if (user == null) return "";
		return user.getId();
	}

	/*******************************************************************************
	* ConnectionPool
	*******************************************************************************/

	protected class MyConnectionPool implements ConnectionPool
	{
		/** The pool id. */
		protected String m_id = null;

		/** The database driver class name. */
		protected String m_driver = null;

		/** The database connect string. */
		protected String m_connect = null;

		/** The database user id. */
		protected String m_user = null;

		/** The database user password. */
		protected String m_pw = null;

		/** The default auto-commit setting for newly borrowed connections. */
		protected boolean m_autoCommit = true;

		/** The pool. */
		protected GenericObjectPool m_pool = null;

		/**
		 * Construct
		 * @param id The pool id.
		 */
		public MyConnectionPool(String id)
		{
			m_id = id;

			// create the generic object pool
			m_pool = new GenericObjectPool(new MyPoolableObjectFactory());
			m_pool.setWhenExhaustedAction(GenericObjectPool.WHEN_EXHAUSTED_BLOCK);
			m_pool.setTestOnBorrow(true);
			m_pool.setTestOnReturn(false);
			m_pool.setTestWhileIdle(false);
		}

		/**
		 * {@inheritdoc}
		 */
		public String getId()
		{
			return m_id;
		}

		/**
		 * {@inheritdoc}
		 */
		public String getDriver()
		{
			return m_driver;
		}

		/**
		 * {@inheritdoc}
		 */
		public String getConnect()
		{
			return m_connect;
		}

		/**
		 * {@inheritdoc}
		 */
		public String getUser()
		{
			return m_user;
		}

		/**
		 * {@inheritdoc}
		 */
		public void setConnect(String driver, String connect, String user, String pw) throws SQLException
		{
			// load the driver
			m_driver = driver;
			try
			{
				Class.forName(m_driver).newInstance();
			}
			catch (InstantiationException e)
			{
				throw new SQLException("InstantiationException for driver: " + driver);
			}
			catch (IllegalAccessException e)
			{
				throw new SQLException("IllegalAccessException for driver: " + driver);
			}
			catch (ClassNotFoundException e)
			{
				throw new SQLException("ClassNotFoundException for driver: " + driver);
			}

			m_connect = connect;
			m_user = user;
			m_pw = pw;

			// TODO: now that we are re-set, deal with pooled connections
			m_pool.clear();
		}

		/**
		 * {@inheritdoc}
		 */
		public int getMaxSize()
		{
			return m_pool.getMaxActive();
		}

		/**
		 * {@inheritdoc}
		 */
		public void setMaxSize(int size)
		{
			m_pool.setMaxActive(size);
		}

		/**
		 * {@inheritdoc}
		 */
		public int getMinSize()
		{
			return m_pool.getMaxIdle();
		}

		/**
		 * {@inheritdoc}
		 */
		public void setMinSize(int size)
		{
			m_pool.setMaxIdle(size);
		}

		/**
		 * {@inheritdoc}
		 */
		public int getIncrement()
		{
			return 0;
		}

		/**
		 * {@inheritdoc}
		 */
		public void setIncrement(int size)
		{
		}

		/**
		 * {@inheritdoc}
		 */
		public long getMaxLifetime()
		{
			return 0;
		}

		/**
		 * {@inheritdoc}
		 */
		public void setMaxLifetime(long ms)
		{
		}

		/**
		 * {@inheritdoc}
		 */
		public boolean getAutoCommit()
		{
			return m_autoCommit;
		}

		/**
		 * {@inheritdoc}
		 */
		public void setAutoCommit(boolean auto)
		{
			m_autoCommit = auto;
		}

		/**
		 * {@inheritdoc}
		 */
		public long getMaxWait()
		{
			return m_pool.getMaxWait();
		}

		/**
		 * {@inheritdoc}
		 */
		public void setMaxWait(long wait)
		{
			m_pool.setMaxWait(wait);
		}

		/**
		 * {@inheritdoc}
		 */
		public void clear()
		{
			m_pool.clear();
		}

		/**
		 * {@inheritdoc}
		 */
		public Connection borrowConnection() throws SQLException
		{
			try
			{
				return (Connection) m_pool.borrowObject();
			}
			catch (Exception e)
			{
				throw new SQLException("unable to borrow: " + e.toString());
			}
		}

		/**
		 * {@inheritdoc}
		 */
		public boolean returnConnection(Connection conn)
		{
			try
			{
				m_pool.returnObject(conn);
			}
			catch (Exception e)
			{
			}
			return true; // TODO
		}

		/**
		 * {@inheritdoc}
		 */
		public int getNumActive()
		{
			return m_pool.getNumActive();
		}

		/**
		 * {@inheritdoc}
		 */
		public int getNumIdle()
		{
			// TODO Auto-generated method stub
			return m_pool.getNumIdle();
		}

		/**
		 * {@inheritdoc}
		 */
		public long getAvgWait()
		{
			// TODO Auto-generated method stub
			return 0;
		}

		/**
		 * {@inheritdoc}
		 */
		public long getAvgActive()
		{
			// TODO Auto-generated method stub
			return 0;
		}

		/**
		 * {@inheritdoc}
		 */
		public void setMontiorPeriod(long ms)
		{
			// TODO Auto-generated method stub
		}

		/*******************************************************************************
		* PoolableObjectFactory
		*******************************************************************************/

		/**
		 * Factory for the actual connections.
		 */
		private class MyPoolableObjectFactory implements PoolableObjectFactory
		{
			/**
			 * {@inheritdoc}
			 */
			public Object makeObject() throws Exception
			{
				Connection conn = DriverManager.getConnection(m_connect, m_user, m_pw);

				return conn;
			}

			/**
			 * {@inheritdoc}
			 */
			public void destroyObject(Object obj) throws Exception
			{
			}

			/**
			 * {@inheritdoc}
			 */
			public boolean validateObject(Object obj)
			{
				// TODO: some sort of sql statement...
				return true;
			}

			/**
			 * {@inheritdoc}
			 */
			public void activateObject(Object obj) throws Exception
			{
				// set the default auto-commit mode
				 ((Connection) obj).setAutoCommit(m_autoCommit);

			} // activateObject

			/**
			 * {@inheritdoc}
			 */
			public void passivateObject(Object obj) throws Exception
			{
			}
		}
	}
}

/**********************************************************************************
*
* $Header: /cvs/sakai/framework-component/src/java/org/sakaiproject/component/framework/sql/BasicSqlService.java,v 1.8 2004/06/22 03:13:32 ggolden Exp $
*
**********************************************************************************/
