/**********************************************************************************
*
* $Header: /cvs/sakai/framework-component/src/java/org/sakaiproject/component/framework/state/BaseStateService.java,v 1.12 2004/06/22 03:13:31 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

// package
package org.sakaiproject.component.framework.state;

// imports
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

import javax.servlet.http.HttpSession;

import org.sakaiproject.service.framework.current.CurrentService;
import org.sakaiproject.service.framework.log.Logger;
import org.sakaiproject.service.framework.state.SessionState;
import org.sakaiproject.service.framework.state.SessionStateBindingListener;
import org.sakaiproject.service.framework.state.StateService;

/**
* <p>BaseStateManagerService is a Turbine Service implementation of the
* StateManagerService.</p>
* <p>Each SessionState is stored in a Map, storing the names and values
* of the state attributes.</p>
* <p>The set of states managed is stored in some specific way by extension classes.</p>
* <p>See the proposal: jakarta-jetspeed/proposals/StateManager.txt for more details.</p>
* @version $Revision: 1.12 $
* @author <a href="mailto:ggolden@apache.org">Glenn R. Golden</a>
*/
public abstract class BaseStateService implements StateService
{
	/** Dependency: a CurrentService */
	protected CurrentService currentService = null;

	/*******************************************************************************
	* Dependencies and their setter methods
	*******************************************************************************/

	/** Dependency: logging service */
	protected Logger m_logger = null;

	/**
	 * Dependency: logging service.
	 * @param service The logging service.
	 */
	public void setLogger(Logger service)
	{
		m_logger = service;
	}

	/**
	 * Access the CurrentService
	 * @return The CurrentService
	 */
	public CurrentService getCurrentService()
	{
		return currentService;
	}

	/**
	 * Dependency: Set my CurrentService
	 * @param service The CurrentService impl.
	 */
	public void setCurrentService(CurrentService service)
	{
		currentService = service;
	}

	/*******************************************************************************
	* Init and Destroy
	*******************************************************************************/

	/**
	 * Final initialization, once all dependencies are set.
	 */
	public void init()
	{
		try
		{
			// create our states storage
			initStates();

			m_logger.info(this +".init()");
		}
		catch (Throwable t)
		{
			m_logger.warn(this +".init(): ", t);
		}
	}

	/**
	* Returns to uninitialized state.
	*
	* You can use this method to release resources thet your Service
	* allocated when Turbine shuts down.
	*/
	public void destroy()
	{
		shutdownStates();

		m_logger.info(this +".destroy()");
	}

	/*******************************************************************************
	* Abstract methods
	*******************************************************************************/

	/**
	* Initialize the states storage.
	*/
	protected abstract void initStates();

	/**
	* Cleanup the states storage.
	*/
	protected abstract void shutdownStates();

	/**
	* Access the Map which is the set of attributes for a state.
	* @param key The state key.
	* @return The Map which is the set of attributes for a state.
	*/
	protected abstract Map getState(String key);

	/**
	* Add a new state to the states we are managing.
	* @param key The state key.
	* @param state The Map which is the set of attributes for the state.
	*/
	protected abstract void addState(String key, Map state);

	/**
	* Remove a state from the states we are managing.
	* @param key The state key.
	*/
	protected abstract void removeState(String key);

	/**
	* Access an array of the keys of all states managed, those that start with the parameter.
	* @param start The starting string used to select the keys.
	* @return an array of the keys of all states managed.
	*/
	protected abstract String[] getStateKeys(String start);

	/**
	* retire the attributes of the state.
	* @param key The state key.
	* @param state The Map of attributes to retire.
	*/
	protected void retireAttributes(String key, Map state)
	{
		if (state == null)
			return;

		Set attributes = state.entrySet();
		synchronized (state)
		{
			Iterator i = attributes.iterator();
			while (i.hasNext())
			{
				Map.Entry attribute = (Map.Entry) i.next();
				unBindAttributeValue(key, (String) attribute.getKey(), attribute.getValue());
			}
		}

		// remove all attributes
		state.clear();

	} // retireAttributes

	/**
	* If the object is a SessionStateBindingListener, unbind it
	* @param stateKey The state key.
	* @param attributeName The attribute name.
	* @param attribute The attribute object
	*/
	protected void unBindAttributeValue(String stateKey, String attributeName, Object attribute)
	{
		// if this object wants session binding notification
		if ((attribute != null) && (attribute instanceof SessionStateBindingListener))
		{
			try
			{
				((SessionStateBindingListener) attribute).valueUnbound(stateKey, attributeName);
				m_logger.debug("unbinding: " + attribute + " name: " + attributeName + " state: " + stateKey);
			}
			catch (Throwable e)
			{
				m_logger.warn("BaseStateService.unBindAttributeValue: unbinding exception: ", e);
			}
		}

	} // unBindAttributeValue

	/**
	* If the object is a SessionStateBindingListener, bind it
	* @param stateKey The state key.
	* @param attributeName The attribute name.
	* @param attribute The attribute object
	*/
	protected void bindAttributeValue(String stateKey, String attributeName, Object attribute)
	{
		// if this object wants session binding notification
		if ((attribute != null) && (attribute instanceof SessionStateBindingListener))
		{
			try
			{
				((SessionStateBindingListener) attribute).valueBound(stateKey, attributeName);
			}
			catch (Throwable e)
			{
				m_logger.warn("BaseStateService.bindAttributeValue: unbinding exception: ", e);
			}
		}

	} // bindAttributeValue

	/*******************************************************************************
	* StateManagerService implementation
	*******************************************************************************/

	/**
	* Access the named attribute of the keyed state.
	* @param key The state key.
	* @param name The attribute name.
	* @return The named attribute value of the keyed state.
	*/
	public Object getAttribute(String key, String name)
	{
		Object rv = null;

		Map state = getState(key);
		if (state != null)
		{
			rv = state.get(name);
		}

		if (m_logger.isDebugEnabled())
			m_logger.debug(this + "getAttribute: " + key + " " + name + " " + rv);

		return rv;

	} // getAttribute

	/**
	* Set the named state attribute of the keyed state with the provided object.
	* @param key The state key.
	* @param name The attribute name.
	* @param value The new value of the attribute (any object type).
	*/
	public void setAttribute(String key, String name, Object value)
	{
		Map state = getState(key);
		if (state == null)
		{
			// create a synchronized map to store the state attributes
			state = Collections.synchronizedMap(new HashMap());
			addState(key, state);
		}

		// get the old, if any
		Object old = getAttribute(key, name);

		// store the new
		state.put(name, value);

		// if there was an old value, unbind it
		if (old != null)
		{
			unBindAttributeValue(key, name, old);
		}

		// bind the new
		bindAttributeValue(key, name, value);

		if (m_logger.isDebugEnabled())
			m_logger.debug(this + "setAttribute: " + key + " " + name + " " + value);

	} // setAttribute

	/**
	* Remove the named state attribute of the keyed state, if it exists.
	* @param key The state key.
	* @param name The attribute name.
	*/
	public void removeAttribute(String key, String name)
	{
		Map state = getState(key);
		if (state == null)
			return;

		// get the old, if any
		Object old = getAttribute(key, name);

		// remove
		state.remove(name);

		// if the state is now empty, remove it
		if (state.isEmpty())
		{
			removeState(key);
		}

		// if there was an old value, unbind it
		if (old != null)
		{
			unBindAttributeValue(key, name, old);
		}

		if (m_logger.isDebugEnabled())
			m_logger.debug(this + "removeAttribute: " + key + " " + name + " " + old);

	} // removeAttribute

	/**
	* Remove all state attribute of the keyed state.
	* @param key The state key.
	*/
	public void clear(String key)
	{
		Map state = getState(key);
		if (state == null)
			return;

		// notify all attribute and clear the state
		retireAttributes(key, state);

		// and forget about it
		removeState(key);

		if (m_logger.isDebugEnabled())
			m_logger.debug(this + "clear: " + key);

	} // clear

	/**
	* Access an array of all names of attributes stored in the keyed state.
	* @param key The state key.
	* @return An array of all names of attributes stored in the keyed state.
	*/
	public List getAttributeNames(String key)
	{
		Map state = (Map) getState(key);
		if (state == null)
			return new Vector();
		if (state.size() == 0)
			return new Vector();

		// put the names into an array for return
		List rv = new Vector();
		rv.addAll(state.keySet());

		return rv;

	} // getAttributeNames

	/**
	* Access an SessionState object with the given key.
	* @param key The SessionState key.
	* @return an SessionState object with the given key.
	*/
	public SessionState getSessionState(String key)
	{
		return new MySessionState(key, this);

	} // getSessionState

	/**
	* Access the SessionState object associated with the current request's http session.
	* The session id is used as the key.
	* @return an SessionState object associated with the current request's http session.
	*/
	public SessionState getCurrentSessionState()
	{
		// get the "current" (for this thread) HttpSession
		HttpSession session = (HttpSession) getCurrentService().getInThread(HttpSession.class.getName());
		if (session == null)
			return null;

		return getSessionState(session.getId());

	} // getCurrentSessionState

	/**
	* Access the SessionState object associated with the current request's http session with the given key.
	* @param key The string to add to the session id to form the SessionState key.
	* @return an SessionState object associated with the current request's http session with the given key.
	*/
	public SessionState getCurrentSessionState(String key)
	{
		// get the "current" (for this thread) HttpSession
		HttpSession session = (HttpSession) getCurrentService().getInThread(HttpSession.class.getName());
		if (session == null)
			return null;

		return getSessionState(session.getId() + "." + key);

	} // getCurrentSessionState

	/**
	* Retire, forget about and clean up all states that start with the given key.
	* @param keyStart The beginning of the key of the states to clean up.
	*/
	public synchronized void retireState(String keyStart)
	{
		// get the current state keys into an array
		String keys[] = getStateKeys(keyStart);
		if (keys == null)
			return;

		// clear them
		for (int i = 0; i < keys.length; i++)
		{
			clear(keys[i]);
		}

	} // retireState

	/*******************************************************************************
	* SessionState implementation
	*******************************************************************************/

	/**
	* A SessionState implementation, as covers to this service, storing the key.
	*/
	private class MySessionState implements SessionState
	{
		/** The state key. */
		private String m_key = null;

		/** The StateManagerService object. */
		private BaseStateService m_service = null;

		/**
		* Construct.
		* @param key The state key.
		* @param service The JetspeedStateManagerService instance.
		*/
		public MySessionState(String key, BaseStateService service)
		{
			m_key = key;
			m_service = service;

		} // MySessionState

		/**
		* Access the named attribute.
		* @param name The attribute name.
		* @return The named attribute value.
		*/
		public Object getAttribute(String name)
		{
			return m_service.getAttribute(m_key, name);

		} // getAttribute

		/**
		* Set the named attribute value to the provided object.
		* @param name The attribute name.
		* @param value The value of the attribute (any object type).
		*/
		public void setAttribute(String name, Object value)
		{
			m_service.setAttribute(m_key, name, value);

		} // setAttribute

		/**
		* Remove the named attribute, if it exists.
		* @param name The attribute name.
		*/
		public void removeAttribute(String name)
		{
			m_service.removeAttribute(m_key, name);

		} // removeAttribute

		/**
		* Remove all attributes.
		*/
		public void clear()
		{
			m_service.clear(m_key);

		} // clear

		/**
		* Access an array of all names of attributes stored in the SessionState.
		* @return An array of all names of attribute stored in the SessionState.
		*/
		public List getAttributeNames()
		{
			return m_service.getAttributeNames(m_key);

		} // getAttributeNames

		/**
		* Access the full unique StateManager key for the SessionState.
		* @return the full unique StateManager key for the SessionState.
		*/
		public String getKey()
		{
			return m_key;

		} // getKey

		/**
		* Retire, forget about and clean up this state.
		*/
		public void retire()
		{
			m_service.retireState(m_key);

		} // retire

	} // class MySessionState

} // BaseStateManagerService

/**********************************************************************************
*
* $Header: /cvs/sakai/framework-component/src/java/org/sakaiproject/component/framework/state/BaseStateService.java,v 1.12 2004/06/22 03:13:31 ggolden Exp $
*
**********************************************************************************/
