/**********************************************************************************
*
* $Header: /cvs/sakai/framework-component/src/java/org/sakaiproject/component/framework/state/HttpStateService.java,v 1.5 2004/06/22 03:13:31 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

// package
package org.sakaiproject.component.framework.state;

// imports
import java.util.Enumeration;
import java.util.Map;
import java.util.Vector;

import javax.servlet.http.HttpSession;
import javax.servlet.http.HttpSessionBindingEvent;
import javax.servlet.http.HttpSessionBindingListener;

/**
* <p>HttpStateService is an implementation of the BaseStateService
* which manages the states stored in the "current" HttpSession.</p>
* <p>Note: This implementation of the StateService takes advantage of the
* Servlet container's management of the HttpSession.
* When the session is invalidated, the states we manage will be automatically cleaned up.
* When this happens, the objects placed into our states will have their
* SessionStateBindingListener mechanism invoked.</p>
* <p>Note: This implementation segments the states by session.  States created in one session will NOT BE AVAILABLE
* from other sessions.</p>
* @version $Revision: 1.5 $
* @author <a href="mailto:ggolden@apache.org">Glenn R. Golden</a>
*/
public class HttpStateService extends BaseStateService
{
	/**
	* Initialize the states storage.
	*/
	protected void initStates()
	{
	} // initStates

	/**
	* Cleanup the states storage.
	*/
	protected void shutdownStates()
	{
	} // shutdownStates

	/**
	* Access the current HttpSession.
	*/
	private HttpSession getSession()
	{
		// get the "current" (for this thread) HttpSession
		HttpSession session = (HttpSession) getCurrentService().getInThread(HttpSession.class.getName());
		if (session == null)
			return null;

		// call isNew just to see if the session has been invalidated already
		try
		{
			session.isNew();
		}
		catch (IllegalStateException e)
		{
			return null;
		}

		return session;

	} // getSession

	/**
	* Convert the key to a name safe to store directly in the session.
	* @param key The state key.
	* @return a name safe to store directly in the session based on key.
	*/
	private String getSessionKey(String key)
	{
		// we want our keys not to conflict with any other session usage...
		return HttpStateService.class.getName() + "." + key;

	} // getSessionKey

	/**
	* Access the Map which is the set of attributes for a state.
	* @param key The state key.
	* @return The Map which is the set of attributes for a state.
	*/
	protected Map getState(String key)
	{
		// get the session
		HttpSession session = getSession();
		if (session == null)
			return null;

		// get this state from our entry in the session
		StateEntry stateEntry = (StateEntry) session.getAttribute(getSessionKey(key));
		if (stateEntry == null)
			return null;

		return stateEntry.getMap();

	} // getState

	/**
	* Add a new state to the states we are managing.
	* @param key The state key.
	* @param state The Map which is the set of attributes for the state.
	*/
	protected void addState(String key, Map state)
	{
		// get the session
		HttpSession session = getSession();
		if (session == null)
			return;

		// create a stateEntry to hold our state Map
		StateEntry stateEntry = new StateEntry(key, state);

		// put it in the session
		session.setAttribute(getSessionKey(key), stateEntry);

	} // addState

	/**
	* Remove a state from the states we are managing.
	* @param key The state key.
	*/
	protected void removeState(String key)
	{
		// get the session
		HttpSession session = getSession();
		if (session == null)
			return;

		// remove the key from the session - the StateEntry will be notified
		session.removeAttribute(getSessionKey(key));

	} // removeState

	/**
	* Access an array of the keys of all states managed, those that start with the parameter.
	* @param start The starting string used to select the keys.
	* @return an array of the keys of all states managed.
	*/
	protected String[] getStateKeys(String start)
	{
		// get the session
		HttpSession session = getSession();
		if (session == null)
			return null;

		// use this as the test pattern
		String pattern = getSessionKey(start);

		// for those that match, this starts the session key
		int subStart = getSessionKey("").length();

		// collect for return
		Vector rv = new Vector();

		// get the session names
		Enumeration names = session.getAttributeNames();
		while (names.hasMoreElements())
		{
			String sessionName = (String) names.nextElement();

			// pick our states, and those whose key starts with the pattern
			if (sessionName.startsWith(pattern))
			{
				rv.add(sessionName.substring(subStart));
			}
		}

		if (rv.size() == 0)
			return null;

		return (String[]) rv.toArray(new String[rv.size()]);

	} // getStateKeys

	/**
	* Store the Map for the state, and listen for HttpSessionBinding events
	*/
	private class StateEntry implements HttpSessionBindingListener
	{
		/** Store the map. */
		private Map m_map = null;

		/** The state key. */
		private String m_key = null;

		/**
		* Construct.
		* @param key The state key.
		* @param map The map to hold.
		*/
		public StateEntry(String key, Map map)
		{
			m_key = key;
			m_map = map;

		} // StateEntry

		/**
		* Access the map we are holding.
		* @return the Map we are holding.
		*/
		public Map getMap()
		{
			return m_map;

		} // getMap

		/**
		* We don't care about when we are bound...
		*/
		public void valueBound(HttpSessionBindingEvent event)
		{
		}

		/**
		* When we are unbound, unbind our state's (map's) attributes
		*/
		public void valueUnbound(HttpSessionBindingEvent event)
		{
			// notify all attribute and clear the state
			retireAttributes(m_key, m_map);
			m_map = null;
			m_key = null;
		}

	} // class StateEntry
}

/**********************************************************************************
*
* $Header: /cvs/sakai/framework-component/src/java/org/sakaiproject/component/framework/state/HttpStateService.java,v 1.5 2004/06/22 03:13:31 ggolden Exp $
*
**********************************************************************************/
