/**********************************************************************************
*
* $Header: /cvs/sakai/framework-component/src/java/org/sakaiproject/component/framework/state/StandaloneStateService.java,v 1.3 2004/06/22 03:13:32 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

package org.sakaiproject.component.framework.state;

import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

/**
* <p>StandaloneStateManagerService is an implementation of the BaseStateService
* which manages the states stored in a local Map (synchronized HashMap).</p>
* <p>Note: This implementation of the StateManagerService is independent of all other
* services; but it has no automatic way to retire no longer used state.  If the
* application does not explicitly retire the states created, they will hang around
* forever. (see clear() and retireState() of the StateManagerService).</p>
* @version $Revision: 1.3 $
* @author <a href="mailto:ggolden@apache.org">Glenn R. Golden</a>
*/
public class StandaloneStateService extends BaseStateService
{
	/** Store each set of state parameters by state key
	    (each is a HashMap keyed by parameter name) */
	private Map m_states = null;

	/**
	* Initialize the states storage.
	*/
	protected void initStates()
	{
		// create our states map synchronized
		m_states = Collections.synchronizedMap(new HashMap());

	} // initStates

	/**
	* Cleanup the states storage.
	*/
	protected void shutdownStates()
	{
		m_states.clear();
		m_states = null;

	} // shutdownStates

	/**
	* Access the Map which is the set of attributes for a state.
	* @param key The state key.
	* @return The Map which is the set of attributes for a state.
	*/
	protected Map getState(String key)
	{
		return (Map) m_states.get(key);

	} // getState

	/**
	* Add a new state to the states we are managing.
	* @param key The state key.
	* @param state The Map which is the set of attributes for the state.
	*/
	protected void addState(String key, Map state)
	{
		m_states.put(key, state);

	} // addState

	/**
	* Remove a state from the states we are managing.
	* @param key The state key.
	*/
	protected void removeState(String key)
	{
		m_states.remove(key);

	} // removeState

	/**
	* Access an array of the keys of all states managed, those that start with the parameter.
	* @param start The starting string used to select the keys.
	* @return an array of the keys of all states managed.
	*/
	protected String[] getStateKeys(String start)
	{
		// collect for return
		Vector rv = new Vector();

		// get the entire set of keys to iterate over
		Set allStateKeys = m_states.keySet();
		synchronized (m_states)
		{
			Iterator i = allStateKeys.iterator();
			while (i.hasNext())
			{
				String key = (String) i.next();

				// if this matches our pattern
				if (key.startsWith(start))
				{
					rv.add(key);
				}
			}
		}

		if (rv.size() == 0)
			return null;

		return (String[]) rv.toArray(new String[rv.size()]);

	} // getStateKeys

} // JetspeedStateManagerService

/**********************************************************************************
*
* $Header: /cvs/sakai/framework-component/src/java/org/sakaiproject/component/framework/state/StandaloneStateService.java,v 1.3 2004/06/22 03:13:32 ggolden Exp $
*
**********************************************************************************/
