/**********************************************************************************
*
* $Header: /cvs/sakai/legacy-component/src/java/org/sakaiproject/component/legacy/alias/BaseAliasService.java,v 1.11 2004/06/22 03:13:56 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

// package
package org.sakaiproject.component.legacy.alias;

// imports
import java.util.Iterator;
import java.util.List;
import java.util.Stack;
import java.util.Vector;

import org.sakaiproject.exception.IdInvalidException;
import org.sakaiproject.exception.IdUnusedException;
import org.sakaiproject.exception.IdUsedException;
import org.sakaiproject.exception.InUseException;
import org.sakaiproject.exception.PermissionException;
import org.sakaiproject.service.legacy.alias.AliasEdit;
import org.sakaiproject.service.legacy.alias.AliasService;
import org.sakaiproject.service.framework.config.cover.ServerConfigurationService;
import org.sakaiproject.service.legacy.email.cover.MailArchiveService;
import org.sakaiproject.service.legacy.event.Event;
import org.sakaiproject.service.legacy.event.cover.EventTrackingService;
import org.sakaiproject.service.framework.log.Logger;
import org.sakaiproject.service.framework.memory.Cache;
import org.sakaiproject.service.framework.memory.CacheRefresher;
import org.sakaiproject.service.framework.memory.MemoryService;
import org.sakaiproject.service.legacy.resource.Edit;
import org.sakaiproject.service.legacy.resource.Reference;
import org.sakaiproject.service.legacy.resource.Resource;
import org.sakaiproject.service.legacy.resource.ResourceProperties;
import org.sakaiproject.service.legacy.resource.ResourcePropertiesEdit;
import org.sakaiproject.service.legacy.security.cover.SecurityService;
import org.sakaiproject.service.framework.session.cover.UsageSessionService;
import org.sakaiproject.service.legacy.site.cover.SiteService;
import org.sakaiproject.service.framework.state.SessionStateBindingListener;
import org.sakaiproject.service.legacy.time.cover.TimeService;
import org.sakaiproject.util.Validator;
import org.sakaiproject.util.resource.BaseResourcePropertiesEdit;
import org.sakaiproject.util.storage.StorageUser;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
* <p>BaseAliasService is a CHEF Alias services implemented as a Turbine Service.</p>
* 
* @author University of Michigan, CHEF Software Development Team
* @version $Revision: 1.11 $
*/
public abstract class BaseAliasService
	implements AliasService, StorageUser, CacheRefresher
{
	/** Storage manager for this service. */
	protected Storage m_storage = null;

	/** A Cache for this service - Aliass keyed by reference. */
	protected Cache m_cache = null;

	/** The initial portion of a relative access point URL. */
	protected String m_relativeAccessPoint = null;

	/** A alias directory provider class name, as configured. */
	protected String m_providerClassName = null;

	/*******************************************************************************
	* Abstractions, etc.
	*******************************************************************************/

	/**
	* Construct storage for this service.
	*/
	protected abstract Storage newStorage();

	/**
	* Access the partial URL that forms the root of resource URLs.
	* @param relative if true, form within the access path only (i.e. starting with /content)
	* @return the partial URL that forms the root of resource URLs.
	*/
	protected String getAccessPoint(boolean relative)
	{
		return (relative ? "" : ServerConfigurationService.getAccessUrl()) + m_relativeAccessPoint;

	}   // getAccessPoint

	/**
	* Access the internal reference which can be used to access the resource from within the system.
	* @param id The alias id string.
	* @return The the internal reference which can be used to access the resource from within the system.
	*/
	public String aliasReference(String id)
	{
		return getAccessPoint(true) + Resource.SEPARATOR + id;

	}   // aliasReference

	/**
	* Access the alias id extracted from a alias reference.
	* @param ref The alias reference string.
	* @return The the alias id extracted from a alias reference.
	*/
	protected String aliasId(String ref)
	{
		String start = getAccessPoint(true) + Resource.SEPARATOR;
		int i = ref.indexOf(start);
		if (i == -1) return ref;
		String id = ref.substring(i+start.length());
		return id;

	}   // aliasId

	/**
	* Check security permission.
	* @param lock The lock id string.
	* @param resource The resource reference string, or null if no resource is involved.
	* @return true if allowed, false if not
	*/
	protected boolean unlockCheck(String lock, String resource)
	{
		if (!SecurityService.unlock(lock, resource))
		{
			return false;
		}

		return true;

	}	// unlockCheck

	/**
	* Check security permission.
	* @param lock The lock id string.
	* @param resource The resource reference string, or null if no resource is involved.
	* @exception PermissionException Thrown if the user does not have access
	*/
	protected void unlock(String lock, String resource)
		throws PermissionException
	{
		if (!unlockCheck(lock, resource))
		{
			throw new PermissionException(UsageSessionService.getSessionUser().getId(), lock, resource);
		}

	}	// unlock

	/**
	* Check security permission, target modify based.
	* @param target The target resource reference string.
	* @return true if allowed, false if not
	*/
	protected boolean unlockTargetCheck(String target)
	{
		// check the target for modify access.
		// %%% Note: this is setup only for sites and mail archive channels,
		// we need a Reference based generic "allowModify()" -ggolden.
		Reference ref = new Reference(target);
		if (ref.getType().equals(SiteService.SERVICE_NAME))
		{
			 return SiteService.allowUpdateSite(ref.getId());
		}
		
		else if (ref.getType().equals(MailArchiveService.SERVICE_NAME))
		{
			return MailArchiveService.allowEditChannel(target);
		}

		return false;

	}	// unlockTargetCheck

	/**
	* Create the live properties for the user.
	*/
	protected void addLiveProperties(ResourcePropertiesEdit props)
	{
		String current = UsageSessionService.getSessionUser().getId();

		props.addProperty(ResourceProperties.PROP_CREATOR, current);
		props.addProperty(ResourceProperties.PROP_MODIFIED_BY, current);
		
		String now = TimeService.newTime().toString();
		props.addProperty(ResourceProperties.PROP_CREATION_DATE, now);
		props.addProperty(ResourceProperties.PROP_MODIFIED_DATE, now);

	}   //  addLiveProperties

	/**
	* Update the live properties for a user for when modified.
	*/
	protected void addLiveUpdateProperties(ResourcePropertiesEdit props)
	{
		String current = UsageSessionService.getSessionUser().getId();

		props.addProperty(ResourceProperties.PROP_MODIFIED_BY, current);
		props.addProperty(ResourceProperties.PROP_MODIFIED_DATE, TimeService.newTime().toString());

	}   //  addLiveUpdateProperties

	/*******************************************************************************
	* Constructors, Dependencies and their setter methods
	*******************************************************************************/

	/** Dependency: logging service */
	protected Logger m_logger = null;

	/**
	 * Dependency: logging service.
	 * @param service The logging service.
	 */
	public void setLogger(Logger service)
	{
		m_logger = service;
	}

	/** Dependency: MemoryService. */
	protected MemoryService m_memoryService = null;

	/**
	 * Dependency: MemoryService.
	 * @param service The MemoryService.
	 */
	public void setMemoryService(MemoryService service)
	{
		m_memoryService = service;
	}

	/*******************************************************************************
	* Init and Destroy
	*******************************************************************************/

	/**
	 * Final initialization, once all dependencies are set.
	 */
	public void init()
	{
		try
		{
			m_relativeAccessPoint = REFERENCE_ROOT;

			// construct storage and read
			m_storage = newStorage();
			m_storage.open();

			// make the cache
			m_cache = m_memoryService.newCache(this, aliasReference(""));

			m_logger.info(this +".init()");
		}
		catch (Throwable t)
		{
			m_logger.warn(this +".init(): ", t);
		}

	} // init

	/**
	* Returns to uninitialized state.
	*/
	public void destroy()
	{
		m_cache.destroy();
		m_cache = null;

		m_storage.close();
		m_storage = null;

		m_logger.info(this + ".destroy()");
	}

	/*******************************************************************************
	* AliasService implementation
	*******************************************************************************/

	/**
	* Check if the current user has permission to set this alias.
	* @param alias The alias.
	* @param target The resource reference string alias target.
	* @return true if the current user has permission to set this alias, false if not.
	*/
	public boolean allowSetAlias(String alias, String target)
	{
		return unlockTargetCheck(target);

	}	// allowSetAlias

	/**
	* Allocate an alias for a resource
	* @param alias The alias.
	* @param target The resource reference string alias target.
	* @throws IdUsedException if the alias is already used.
	* @throws IdInvalidException if the alias id is invalid.
	* @throws PermissionException if the current user does not have permission to set this alias.
	*/
	public void setAlias(String alias, String target)
		throws IdUsedException, IdInvalidException, PermissionException
	{
		// check for a valid alias name
		Validator.checkResourceId(alias);

		if (!unlockTargetCheck(target))
		{
			throw new PermissionException(UsageSessionService.getSessionUser().getId(), SECURE_ADD_ALIAS, target);
		}

		// attempt to register this alias with storage - if it's in use, this will return null
		AliasEdit a = m_storage.put(alias);
		if (a == null)
		{
			throw new IdUsedException(alias);
		}
		a.setTarget(target);

		// update the properties
		addLiveUpdateProperties(a.getPropertiesEdit());

		// complete the edit
		m_storage.commit(a);

		// track it
		EventTrackingService.post(
			EventTrackingService.newEvent(SECURE_ADD_ALIAS, aliasReference(alias), true));

	}	// setAlias

	/**
	* Check to see if the current user can remove this alias.
	* @param alias The alias.
	* @return true if the current user can remove this alias, false if not.
	*/
	public boolean allowRemoveAlias(String alias)
	{
		return unlockCheck(SECURE_REMOVE_ALIAS, aliasReference(alias));

	}	// allowRemoveAlias

	/**
	* Remove an alias.
	* @param alias The alias.
	* @exception IdUnusedException if not found.
	* @exception PermissionException if the current user does not have permission to remove this alias.
	* @exception InUseException if the Alias object is locked by someone else.
	*/
	public void removeAlias(String alias)
		throws IdUnusedException, PermissionException, InUseException
	{
		AliasEdit a = edit(alias);
		remove(a);

	}	// removeAlias

	/**
	* Check to see if the current user can remove these aliasese for this target resource reference.
	* @param target The target resource reference string.
	* @return true if the current user can remove these aliasese for this target resource reference, false if not.
	*/
	public boolean allowRemoveTargetAliases(String target)
	{
		return unlockTargetCheck(target);

	}	// allowRemoveTargetAliases

	/**
	* Remove all aliases for this target resource reference, if any.
	* @param target The target resource reference string.
	* @throws PermissionException if the current user does not have permission to remove these aliases.
	*/
	public void removeTargetAliases(String target)
		throws PermissionException
	{
		if (!unlockTargetCheck(target))
		{
			throw new PermissionException(UsageSessionService.getSessionUser().getId(), SECURE_REMOVE_ALIAS, target);
		}

		List all = getAliases(target);
		for (Iterator iAll = all.iterator(); iAll.hasNext();)
		{
			String alias = (String) iAll.next();
			try
			{
				AliasEdit a = m_storage.edit(alias);
				if (a != null)
				{
					// complete the edit
					m_storage.remove(a);
			
					// track it
					EventTrackingService.post(
						EventTrackingService.newEvent(SECURE_REMOVE_ALIAS, a.getReference(), true));
				}
			}
			catch (Exception ignore) {}
		}

	}	// removeTargetAliases

	/**
	* Find the target resource reference string associated with this alias.
	* @param alias The alias.
	* @return The target resource reference string associated with this alias.
	* @throws IdUnusedException if the alias is not defined.
	*/
	public String getTarget(String alias)
		throws IdUnusedException
	{
		String target = null;

		// check the cache for the alias
		String key = aliasReference(alias);
		if (m_cache.containsKey(key))
		{
			BaseAliasEdit a = (BaseAliasEdit) m_cache.get(key);
			target = a.getTarget();
		}

		// get the alias from storage
		else
		{
			BaseAliasEdit a = (BaseAliasEdit) m_storage.get(alias);
			if (a == null) throw new IdUnusedException(alias);	// %%% cache misses? -ggolden

			target = a.getTarget();
			
			// cache the result, hit or miss
			m_cache.put(key, a);
		}

		return target;

	}	// getTarget

	/**
	* Find all the aliases defined for this target.
	* @param target The target resource reference string.
	* @return A list (String) of all the alias defined for this target resource reference string.
	*/
	public List getAliases(String target)
	{
		List all = getAliases();

		// pick out from all those that are for this ref
		List found = new Vector();
		for (Iterator iAll = all.iterator(); iAll.hasNext();)
		{
			BaseAliasEdit a = (BaseAliasEdit) iAll.next();
			if (a.getTarget().equals(target)) found.add(a.getId());
		}

		return found;

	}	// getAliases

	/**
	* Find all the aliases.
	* @return A list (String) of all the aliases.
	*/
	public List getAliases()
	{
		List found = new Vector();
		List all = null;

		// if we have disabled the cache, don't use if
		if (m_cache.disabled())
		{
			all = m_storage.getAll();
		}

		else
		{
			// if the cache is complete, use it
			if (m_cache.isComplete())
			{
				all = m_cache.getAll();
			}
	
			// otherwise get all from storage
			else
			{
				// Note: while we are getting from storage, storage might change.  These can be processed
				// after we get the storage entries, and put them in the cache, and mark the cache complete.
				// -ggolden
				synchronized (m_cache)
				{
					// if we were waiting and it's now complete...
					if (m_cache.isComplete())
					{
						all = m_cache.getAll();
					}
					
					else
					{
						// save up any events to the cache until we get past this load
						m_cache.holdEvents();
		
						all = m_storage.getAll();
	
						// update the cache, and mark it complete
						for (Iterator iAll = all.iterator(); iAll.hasNext();)
						{
							BaseAliasEdit a = (BaseAliasEdit) iAll.next();
							m_cache.put(a.getReference(), a);
						}
		
						m_cache.setComplete();
		
						// now we are complete, process any cached events
						m_cache.processEvents();
					}
				}
			}
		}

		return all;

	}	// getAliases

	/**
	* Check to see if the current user can add an alias.
	* @return true if the current user can add an alias, false if not.
	*/
	public boolean allowAdd()
	{
		return unlockCheck(SECURE_ADD_ALIAS, aliasReference(""));

	}	// allowAdd

	/**
	* Add a new alias.  Must commit() to make official, or cancel() when done!
	* @param id The alias id.
	* @return A locked AliasEdit object (reserving the id).
	* @exception IdInvalidException if the alias id is invalid.
	* @exception IdUsedException if the alias id is already used.
	* @exception PermissionException if the current user does not have permission to add an alias.
	*/
	public AliasEdit add(String id)
		throws IdInvalidException, IdUsedException, PermissionException
	{
		// check for a valid user name
		Validator.checkResourceId(id);

		// check security (throws if not permitted)
		unlock(SECURE_ADD_ALIAS, aliasReference(id));

		// reserve an alias with this id from the info store - if it's in use, this will return null
		AliasEdit a = m_storage.put(id);
		if (a == null)
		{
			throw new IdUsedException(id);
		}

		((BaseAliasEdit) a).setEvent(SECURE_ADD_ALIAS);

		return a;

	}	// add

	/**
	* Check to see if the current user can edit this alias.
	* @param id The alias id string.
	* @return true if the current user can edit this alias, false if not.
	*/
	public boolean allowEdit(String id)
	{
		return unlockCheck(SECURE_UPDATE_ALIAS, aliasReference(id));

	}	// allowEdit

	/**
	* Get a locked alias object for editing. Must commit() to make official, or cancel() (or remove()) when done!
	* @param id The alias id string.
	* @return An AliasEdit object for editing.
	* @exception IdUnusedException if not found.
	* @exception PermissionException if the current user does not have permission to mess with this alias.
	* @exception InUseException if the Alias object is locked by someone else.
	*/
	public AliasEdit edit(String id)
		throws IdUnusedException, PermissionException, InUseException
	{
		if (id == null) throw new IdUnusedException("null");

		// check security (throws if not permitted)
		unlock(SECURE_UPDATE_ALIAS, aliasReference(id));

		// check for existance
		if ((m_cache.get(aliasReference(id)) == null) && (!m_storage.check(id)))
		{
			throw new IdUnusedException(id);
		}

		// ignore the cache - get the user with a lock from the info store
		AliasEdit a = m_storage.edit(id);
		if (a == null) throw new InUseException(id);

		((BaseAliasEdit) a).setEvent(SECURE_UPDATE_ALIAS);

		return a;

	}	// edit

	/**
	* Commit the changes made to a AliasEdit object, and release the lock.
	* The AliasEdit is disabled, and not to be used after this call.
	* @param user The AliasEdit object to commit.
	*/
	public void commit(AliasEdit edit)
	{
		// check for closed edit
		if (!edit.isActiveEdit())
		{
			try { throw new Exception(); }
			catch (Exception e) { m_logger.warn(this + ".commit(): closed AliasEdit", e); }
			return;
		}

		// update the properties
		addLiveUpdateProperties(edit.getPropertiesEdit());

		// complete the edit
		m_storage.commit(edit);

		// track it
		EventTrackingService.post(
			EventTrackingService.newEvent(((BaseAliasEdit) edit).getEvent(), edit.getReference(), true));

		// close the edit object
		((BaseAliasEdit) edit).closeEdit();

	}	// commit

	/**
	* Cancel the changes made to a AliasEdit object, and release the lock.
	* The AliasEdit is disabled, and not to be used after this call.
	* @param user The AliasEdit object to commit.
	*/
	public void cancel(AliasEdit edit)
	{
		// check for closed edit
		if (!edit.isActiveEdit())
		{
			try { throw new Exception(); }
			catch (Exception e) { m_logger.warn(this + ".cancel(): closed AliasEdit", e); }
			return;
		}

		// release the edit lock
		m_storage.cancel(edit);

		// close the edit object
		((BaseAliasEdit) edit).closeEdit();

	}	// cancel

	/**
	* Remove this alias information - it must be a user with a lock from edit().
	* The AliasEdit is disabled, and not to be used after this call.
	* @param edit The locked AliasEdit object to remove.
	* @exception PermissionException if the current user does not have permission to remove this alias.
	*/
	public void remove(AliasEdit edit)
		throws PermissionException
	{
		// check for closed edit
		if (!edit.isActiveEdit())
		{
			try { throw new Exception(); }
			catch (Exception e) { m_logger.warn(this + ".remove(): closed AliasEdit", e); }
			return;
		}

		// check security (throws if not permitted)
		unlock(SECURE_REMOVE_ALIAS, edit.getReference());

		// complete the edit
		m_storage.remove(edit);

		// track it
		EventTrackingService.post(
			EventTrackingService.newEvent(SECURE_REMOVE_ALIAS, edit.getReference(), true));

		// close the edit object
		((BaseAliasEdit) edit).closeEdit();

	}	// remove

	/*******************************************************************************
	* Alias implementation
	*******************************************************************************/

	/**
	* <p>BaseAlias is an implementation of the CHEF Alias object.</p>
	* 
	* @author University of Michigan, CHEF Software Development Team
	*/
	public class BaseAliasEdit
		implements AliasEdit, SessionStateBindingListener
	{
		/** The alias id. */
		protected String  m_id = null;

		/** The alias target. */
		protected String m_target = null;

		/** The properties. */
		protected ResourcePropertiesEdit m_properties = null;

		/** The event code for this edit. */
		protected String m_event = null;

		/** Active flag. */
		protected boolean m_active = false;

		/**
		* Clean up.
		*/
		protected void finalize()
		{
			// catch the case where an edit was made but never resolved
			if (m_active)
			{
				cancel(this);
			}

		}	// finalize

		/**
		* Construct.
		* @param id The id.
		*/
		public BaseAliasEdit(String id)
		{
			m_id = id;

			// setup for properties
			ResourcePropertiesEdit props = new BaseResourcePropertiesEdit();
			m_properties = props;

			// if not a reconstruction, add properties
			if ((m_id != null) && (m_id.length() > 0)) addLiveProperties(props);

		}   // BaseAlias

		/**
		* Construct from another Alias object.
		* @param alias The alias object to use for values.
		*/
		public BaseAliasEdit(BaseAliasEdit alias)
		{
			setAll(alias);

		}	// BaseAlias

		/**
		* Construct from information in XML.
		* @param el The XML DOM Element definining the alias.
		*/
		public BaseAliasEdit(Element el)
		{
			// setup for properties
			m_properties = new BaseResourcePropertiesEdit();

			m_id = el.getAttribute("id");
			m_target = el.getAttribute("target");

			// the children (properties)
			NodeList children = el.getChildNodes();
			final int length = children.getLength();
			for (int i = 0; i < length; i++)
			{
				Node child = children.item(i);
				if (child.getNodeType() != Node.ELEMENT_NODE) continue;
				Element element = (Element)child;

				// look for properties
				if (element.getTagName().equals("properties"))
				{
					// re-create properties
					m_properties = new BaseResourcePropertiesEdit(element);
				}
			}

		}	// BaseAlias

		/**
		* Take all values from this object.
		* @param alias The alias object to take values from.
		*/
		protected void setAll(BaseAliasEdit alias)
		{
			m_id = alias.m_id;
			m_target = alias.m_target;

			m_properties = new BaseResourcePropertiesEdit();
			m_properties.addAll(alias.getProperties());

		}   // setAll

		/**
		* Serialize the resource into XML, adding an element to the doc under the top of the stack element.
		* @param doc The DOM doc to contain the XML (or null for a string return).
		* @param stack The DOM elements, the top of which is the containing element of the new "resource" element.
		* @return The newly added element.
		*/
		public Element toXml(Document doc, Stack stack)
		{
			Element alias = doc.createElement("alias");

			if (stack.isEmpty())
			{
				doc.appendChild(alias);
			}
			else
			{
				((Element)stack.peek()).appendChild(alias);
			}

			stack.push(alias);

			alias.setAttribute("id", m_id);
			alias.setAttribute("target", m_target);

			// properties
			m_properties.toXml(doc, stack);

			stack.pop();

			return alias;

		}	// toXml

		/**
		* Access the alias id.
		* @return The alias id string.
		*/
		public String getId()
		{
			return m_id;

		}   // getId

		/**
		* Access the alias target.
		* @return The alias target.
		*/
		public String getTarget()
		{
			return m_target;

		}   // getTarget

		/**
		* Set the alias target.
		* @param target The alias target.
		*/
		public void setTarget(String target)
		{
			m_target = target;

		}   // setTarget

		/**
		* Access the URL which can be used to access the resource.
		* @return The URL which can be used to access the resource.
		*/
		public String getUrl()
		{
			return getAccessPoint(false) + m_id;

		}   // getUrl

		/**
		* Access the internal reference which can be used to access the resource from within the system.
		* @return The the internal reference which can be used to access the resource from within the system.
		*/
		public String getReference()
		{
			return aliasReference(m_id);

		}   // getReference

		/**
		* Access the resources's properties.
		* @return The resources's properties.
		*/
		public ResourceProperties getProperties()
		{
			return m_properties;

		}   // getProperties

		/**
		* Are these objects equal?  If they are both Alias objects, and they have
		* matching id's, they are.
		* @return true if they are equal, false if not.
		*/
		public boolean equals(Object obj)
		{
			if (!(obj instanceof BaseAliasEdit)) return false;
			return ((BaseAliasEdit)obj).getId().equals(getId());

		}   // equals

		/**
		* Make a hash code that reflects the equals() logic as well.
		* We want two objects, even if different instances, if they have the same id to hash the same.
		*/
		public int hashCode()
		{
			return getId().hashCode();

		}	// hashCode

		/**
		* Compare this object with the specified object for order.
		* @return A negative integer, zero, or a positive integer as this object is
		* less than, equal to, or greater than the specified object.
		*/
		public int compareTo(Object obj)
		{
			if (!(obj instanceof BaseAliasEdit)) throw new ClassCastException();

			// if the object are the same, say so
			if (obj == this) return 0;

			// sort based on (unique) id
			int compare = getId().compareTo(((BaseAliasEdit)obj).getId());

			return compare;

		}	// compareTo

		/**
		* Access the event code for this edit.
		* @return The event code for this edit.
		*/
		protected String getEvent() { return m_event; }
	
		/**
		* Set the event code for this edit.
		* @param event The event code for this edit.
		*/
		protected void setEvent(String event) { m_event = event; }

		/**
		* Access the resource's properties for modification
		* @return The resource's properties.
		*/
		public ResourcePropertiesEdit getPropertiesEdit()
		{
			return m_properties;

		}	// getPropertiesEdit

		/**
		* Enable editing.
		*/
		protected void activate()
		{
			m_active = true;

		}	// activate

		/**
		* Check to see if the edit is still active, or has already been closed.
		* @return true if the edit is active, false if it's been closed.
		*/
		public boolean isActiveEdit()
		{
			return m_active;

		}	// isActiveEdit

		/**
		* Close the edit object - it cannot be used after this.
		*/
		protected void closeEdit()
		{
			m_active = false;

		}	// closeEdit

		/**
		* @return a description of the item this alias's target applies to.
		*/
		public String getDescription()
		{
			try
			{
				// the rest are references to some resource
				Reference ref = new Reference(getTarget());
				return ref.getDescription();
			}
			catch (Throwable any)
			{
				return "unknown";
			}

		}	// getDescription

		/*******************************************************************************
		* SessionStateBindingListener implementation
		*******************************************************************************/
	
		/**
		* Accept notification that this object has been bound as a SessionState attribute.
		* @param sessionStateKey The id of the session state which holds the attribute.
		* @param attributeName The id of the attribute to which this object is now the value.
		*/
		public void valueBound(String sessionStateKey, String attributeName) {}
	
		/**
		* Accept notification that this object has been removed from a SessionState attribute.
		* @param sessionStateKey The id of the session state which held the attribute.
		* @param attributeName The id of the attribute to which this object was the value.
		*/
		public void valueUnbound(String sessionStateKey, String attributeName)
		{
			if (m_logger.isDebugEnabled())
				m_logger.debug(this + ".valueUnbound()");
	
			// catch the case where an edit was made but never resolved
			if (m_active)
			{
				cancel(this);
			}
	
		}	// valueUnbound

	}   // BaseAliasEdit

	/*******************************************************************************
	* Storage
	*******************************************************************************/

	protected interface Storage
	{
		/**
		* Open.
		*/
		public void open();

		/**
		* Close.
		*/
		public void close();

		/**
		* Check if an alias with this id exists.
		* @param id The alias id (case insensitive).
		* @return true if an alias by this id exists, false if not.
		*/
		public boolean check(String id);

		/**
		* Get the alias with this id, or null if not found.
		* @param id The alias id (case insensitive).
		* @return The alias with this id, or null if not found.
		*/
		public AliasEdit get(String id);

		/**
		* Get all the alias.
		* @return The List (BaseAliasEdit) of all alias.
		*/
		public List getAll();

		/**
		* Add a new alias with this id.
		* @param id The alias id.
		* @return The locked Alias object with this id, or null if the id is in use.
		*/
		public AliasEdit put(String id);

		/**
		* Get a lock on the alias with this id, or null if a lock cannot be gotten.
		* @param id The alias id (case insensitive).
		* @return The locked Alias with this id, or null if this records cannot be locked.
		*/
		public AliasEdit edit(String id);

		/**
		* Commit the changes and release the lock.
		* @param user The user to commit.
		*/
		public void commit(AliasEdit user);

		/**
		* Cancel the changes and release the lock.
		* @param user The user to commit.
		*/
		public void cancel(AliasEdit user);

		/**
		* Remove this alias.
		* @param user The alias to remove.
		*/
		public void remove(AliasEdit user);

	}   // Storage

	/*******************************************************************************
	* StorageUser implementation (no container)
	*******************************************************************************/

	/**
	* Construct a new continer given just an id.
	* @param id The id for the new object.
	* @return The new containe Resource.
	*/
	public Resource newContainer(String ref) { return null; }

	/**
	* Construct a new container resource, from an XML element.
	* @param element The XML.
	* @return The new container resource.
	*/
	public Resource newContainer(Element element) { return null; }

	/**
	* Construct a new container resource, as a copy of another
	* @param other The other contianer to copy.
	* @return The new container resource.
	*/
	public Resource newContainer(Resource other) { return null; }

	/**
	* Construct a new resource given just an id.
	* @param container The Resource that is the container for the new resource (may be null).
	* @param id The id for the new object.
	* @param others (options) array of objects to load into the Resource's fields.
	* @return The new resource.
	*/
	public Resource newResource(Resource container, String id, Object[] others)
	{ return new BaseAliasEdit(id); }

	/**
	* Construct a new resource, from an XML element.
	* @param container The Resource that is the container for the new resource (may be null).
	* @param element The XML.
	* @return The new resource from the XML.
	*/
	public Resource newResource(Resource container, Element element)
	{ return new BaseAliasEdit(element); }

	/**
	* Construct a new resource from another resource of the same type.
	* @param container The Resource that is the container for the new resource (may be null).
	* @param other The other resource.
	* @return The new resource as a copy of the other.
	*/
	public Resource newResource(Resource container, Resource other)
	{ return new BaseAliasEdit((BaseAliasEdit) other); }

	/**
	* Construct a new continer given just an id.
	* @param id The id for the new object.
	* @return The new containe Resource.
	*/
	public Edit newContainerEdit(String ref) { return null; }

	/**
	* Construct a new container resource, from an XML element.
	* @param element The XML.
	* @return The new container resource.
	*/
	public Edit newContainerEdit(Element element) { return null; }

	/**
	* Construct a new container resource, as a copy of another
	* @param other The other contianer to copy.
	* @return The new container resource.
	*/
	public Edit newContainerEdit(Resource other) { return null; }

	/**
	* Construct a new resource given just an id.
	* @param container The Resource that is the container for the new resource (may be null).
	* @param id The id for the new object.
	* @param others (options) array of objects to load into the Resource's fields.
	* @return The new resource.
	*/
	public Edit newResourceEdit(Resource container, String id, Object[] others)
	{
		BaseAliasEdit e = new BaseAliasEdit(id);
		e.activate();
		return e;
	}

	/**
	* Construct a new resource, from an XML element.
	* @param container The Resource that is the container for the new resource (may be null).
	* @param element The XML.
	* @return The new resource from the XML.
	*/
	public Edit newResourceEdit(Resource container, Element element)
	{
		BaseAliasEdit e =  new BaseAliasEdit(element);
		e.activate();
		return e;
	}

	/**
	* Construct a new resource from another resource of the same type.
	* @param container The Resource that is the container for the new resource (may be null).
	* @param other The other resource.
	* @return The new resource as a copy of the other.
	*/
	public Edit newResourceEdit(Resource container, Resource other)
	{
		BaseAliasEdit e = new BaseAliasEdit((BaseAliasEdit) other);
		e.activate();
		return e;
	}

	/**
	* Collect the fields that need to be stored outside the XML (for the resource).
	* @return An array of field values to store in the record outside the XML (for the resource).
	*/
	public Object[] storageFields(Resource r) { return null; }

	/*******************************************************************************
	* CacheRefresher implementation (no container)
	*******************************************************************************/

	/**
	* Get a new value for this key whose value has already expired in the cache.
	* @param key The key whose value has expired and needs to be refreshed.
	* @param oldValue The old exipred value of the key.
	* @param event The event which triggered this refresh.
	* @return a new value for use in the cache for this key; if null, the entry will be removed.
	*/
	public Object refresh(Object key, Object oldValue, Event event)
	{
		// key is a reference, but our storage wants an id
		String id = aliasId((String) key);

		// get whatever we have from storage for the cache for this vale
		AliasEdit alias = m_storage.get(id);

		if (m_logger.isDebugEnabled())
			m_logger.debug(this + ".refresh(): " + key + " : " + id);

		return alias;

	}	// refresh
	
}   // BaseAliasService

/**********************************************************************************
*
* $Header: /cvs/sakai/legacy-component/src/java/org/sakaiproject/component/legacy/alias/BaseAliasService.java,v 1.11 2004/06/22 03:13:56 ggolden Exp $
*
**********************************************************************************/
