/**********************************************************************************
*
* $Header: /cvs/sakai/legacy-component/src/java/org/sakaiproject/component/legacy/presence/BasePresenceService.java,v 1.14 2004/06/22 03:14:01 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

// package
package org.sakaiproject.component.legacy.presence;

// imports
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

import org.sakaiproject.exception.IdUnusedException;
import org.sakaiproject.service.legacy.event.Event;
import org.sakaiproject.service.legacy.event.cover.EventTrackingService;
import org.sakaiproject.service.framework.log.Logger;
import org.sakaiproject.service.framework.memory.Cache;
import org.sakaiproject.service.framework.memory.CacheRefresher;
import org.sakaiproject.service.framework.memory.MemoryService;
import org.sakaiproject.service.legacy.presence.PresenceService;
import org.sakaiproject.service.legacy.resource.Resource;
import org.sakaiproject.service.framework.session.UsageSession;
import org.sakaiproject.service.framework.session.cover.UsageSessionService;
import org.sakaiproject.service.legacy.site.Site;
import org.sakaiproject.service.legacy.site.SitePage;
import org.sakaiproject.service.legacy.site.ToolConfiguration;
import org.sakaiproject.service.legacy.site.cover.SiteService;
import org.sakaiproject.service.framework.state.SessionState;
import org.sakaiproject.service.framework.state.StateService;
import org.sakaiproject.util.PresenceUtil;
import org.sakaiproject.util.SessionUtil;

/**
* <p>A basic TurbineService implementation of the PresenceService</p>
* <p>Note: when the timeout checker retires a user, any session state keys are retired with the
* StateServiceService.  This does *not* happen when a user's presence is removed in any other way.</p>
* 
* @author University of Michigan, CHEF Software Development Team
* @version $Revision: 1.14 $
*/
public abstract class BasePresenceService implements PresenceService, CacheRefresher
{
	/** Storage. */
	protected Storage m_storage = null;

	/** Maintenance. */
	protected Maintenance m_maintenance = null;

	/** How long to let an un-refreshed User remain present at a location. */
	protected int m_timeoutSeconds = 30;

	/** How long an inactive session can last before it's timed out. */
	protected int m_sessionTimeoutSeconds = 1800;

	/** How much time between automatic timeout checks. */
	protected int m_checkSeconds = 20;

	/** A Cache for this service - PresenceResource keyed by reference. */
	protected Cache m_cache = null;

	/**
	* Allocate a new storage object.
	* @return A new storage object.
	*/
	protected abstract Storage newStorage();

	/*******************************************************************************
	* Constructors, Dependencies and their setter methods
	*******************************************************************************/

	/** Dependency: logging service */
	protected Logger m_logger = null;

	/**
	 * Dependency: logging service.
	 * @param service The logging service.
	 */
	public void setLogger(Logger service)
	{
		m_logger = service;
	}

	/** Dependency: memory service */
	protected MemoryService m_memoryService = null;

	/**
	 * Dependency: MemoryService.
	 * @param service The MemoryService.
	 */
	public void setMemoryService(MemoryService service)
	{
		m_memoryService = service;
	}

	/** Dependency: StateService. */
	protected StateService m_stateService = null;

	/**
	 * Dependency: StateService.
	 * @param service The StateService.
	 */
	public void setStateService(StateService service)
	{
		m_stateService = service;
	}

	/**
	 * Configuration: set how long to let an un-refreshed User remain present at a location.
	 */
	public void setTimeoutSeconds(String value)
	{
		try
		{
			m_timeoutSeconds = Integer.parseInt(value);
		}
		catch (Exception ignore)
		{
		}
	}

	/**
	 * Configuration: set how much time between automatic timeout checks.
	 */
	public void setCheckSeconds(String value)
	{
		try
		{
			m_checkSeconds = Integer.parseInt(value);
		}
		catch (Exception ignore)
		{
		}
	}

	/**
	 * Configuration: set how long an inactive session can last before it's timed out.
	 */
	public void setSessionTimeoutSeconds(String value)
	{
		try
		{
			m_sessionTimeoutSeconds = Integer.parseInt(value);
		}
		catch (Exception ignore)
		{
		}
	}

	/*******************************************************************************
	* Init and Destroy
	*******************************************************************************/

	/**
	 * Final initialization, once all dependencies are set.
	 */
	public void init()
	{
		try
		{
			// storage
			m_storage = newStorage();

			// maintenance, and start the thread
			m_maintenance = new Maintenance();
			m_maintenance.start();

			// make the cache
			m_cache = m_memoryService.newCache(this, presenceReference(""));

			// read all current presence and make the cache complete to it keeps up with changes
			getLocations();

			m_logger.info(
				this
					+ ".init(): timeout: "
					+ m_timeoutSeconds
					+ "  check: "
					+ m_checkSeconds
					+ " session timeout: "
					+ m_sessionTimeoutSeconds);

		}
		catch (Throwable t)
		{
			m_logger.warn(this +".init(): ", t);
		}
	}

	/**
	* Returns to uninitialized state.
	*/
	public void destroy()
	{
		m_maintenance.stop();

		m_maintenance = null;
		m_storage = null;

		m_cache.destroy();
		m_cache = null;

		m_logger.info(this +".destroy()");

	} // destroy

	/*******************************************************************************
	* PresenceService implementation
	*******************************************************************************/

	/**
	* Access the time (seconds) we will wait for a request from a session before we consider the session lost.
	* @return the time (seconds) we will wait for a request from a session before we consider the session lost.
	*/
	public int getTimeout()
	{
		return m_timeoutSeconds;

	} // getTimeout

	/**
	* Form a presence reference from a location id
	* @param id the location id.
	* @return A presence reference based on a location id.
	*/
	public String presenceReference(String id)
	{
		return REFERENCE_ROOT + Resource.SEPARATOR + id;

	} // presenceReference

	/**
	* Access the presence (location) id extracted from a presence reference.
	* @param ref The presence reference string.
	* @return The the presence (location) id extracted from a presence reference.
	*/
	protected String presenceId(String ref)
	{
		String start = presenceReference("");
		int i = ref.indexOf(start);
		if (i == -1)
			return ref;
		String id = ref.substring(i + start.length());
		return id;

	} // presenceId

	/**
	* Refresh the activity of a session.
	* @param session The session object.
	*/
	public void setActiveSession(UsageSession session)
	{
		if (session == null)
			return;

		// %%% This might happen on server startup for some unknown (yet) reason - the m_maintenance is null -ggolden
		if (m_maintenance == null)
		{
			m_logger.warn(this +".setActiveSession() : called with null m_maintenance!");
			return;
		}

		if (m_logger.isDebugEnabled())
			m_logger.debug(this +".setActiveSession: " + session.getId());

		// tell the maintenance
		m_maintenance.setActiveSession(session.getId());

	} // setActiveSession

	/**
	* Establish or refresh the presence of a user in a location.
	* @param session The usage session.
	* @param locationId A presence location id.
	*/
	public void setPresence(UsageSession session, String locationId)
	{
		if ((session == null) || (locationId == null))
			return;

		// TODO: %%% This happens on server startup for some unknown (yet) reason - the m_maintenance is null -ggolden
		if (m_maintenance == null)
		{
			m_logger.warn(this +".setPresence() : called with null m_maintenance!");
			return;
		}

		if (m_logger.isDebugEnabled())
			m_logger.debug(this +".setPresence(" + session.getId() + ", " + locationId + ")");

		// first, tell the maintenance to set/update the presence (possibly adding this new stateKey, too)
		boolean added = m_maintenance.setPresence(session.getId(), locationId);

		// if this session is new to the location
		if (added)
		{
			// update the storage
			m_storage.setPresence(session.getId(), locationId);

			// generate the event
			Event event = EventTrackingService.newEvent(EVENT_PRESENCE, presenceReference(locationId), true);
			EventTrackingService.post(event, session);
		}

	} // setPresence

	/**
	* Remove the presence of a user from all locations.
	* @param session The usage session.
	*/
	public void removePresence(UsageSession session)
	{
		if (session == null)
			return;

		if (m_logger.isDebugEnabled())
			m_logger.debug(this +".removePresence(" + session.getId() + ")");

		// get all locations for this session
		List locations = m_maintenance.getLocations(session.getId());

		// remove each one
		for (int i = 0; i < locations.size(); i++)
		{
			removePresence(session, (String) locations.get(i));
		}

		// close the session
		m_maintenance.closeSession(session.getId());

	} // removePresence

	/**
	 * {@inheritDoc}
	 */
	public void doInContextMaintenance(boolean auto)
	{
		// process the current session, if any
		UsageSession session = SessionUtil.getSession(m_stateService);
		if (session == null) return;

		// catch closed sessions - get rid of the session as current and remove presence
		if (session.isClosed())
		{
			// removal of the session from state triggers the sessions's valueUnbound(), which generates the EVENT.LOGOUT event
			SessionUtil.setSession(m_stateService, null);
			removePresence(session);
		}

		// sponser active sessions unless auto
		else if (!auto)
		{
			setActiveSession(session);
		}
		
		// clean up closed presence from this session, if any
		m_maintenance.cleanupAbsenses(session.getId());
		
		// if the session has been closed, get rid of it
		m_maintenance.removeClosedSession(session.getId());
	}

	/**
	* Remove the presence of a user from a location.
	* @param session The usage session.
	* @param locationId A presence location id.
	*/
	public void removePresence(UsageSession session, String locationId)
	{
		if ((session == null) || (locationId == null))
			return;

		if (m_logger.isDebugEnabled())
			m_logger.debug(this +".removePresence(" + session.getId() + ", " + locationId + ")");

		// tell maintenance
		boolean removed = m_maintenance.removePresenceM(session.getId(), locationId);

		// if the session was found in the location and removed
		if (removed)
		{
			// remove it from storage
			m_storage.removePresence(session.getId(), locationId);

			// generate the event
			Event event = EventTrackingService.newEvent(EVENT_ABSENCE, presenceReference(locationId), true);
			EventTrackingService.post(event, session);
		}

	} // removePresence

	/**
	* Access a List of sessions (UsageSession) now present in a location.
	* @param locationId A presence location id.
	* @return The List of sessions (UsageSession) now present in the location (may be empty).
	*/
	public List getPresence(String locationId)
	{
		// %%% This happens on server startup for some unknown (yet) reason - the m_cache is null -ggolden
		if (m_cache == null)
		{
			m_logger.warn(this +".getPresence() : called with null m_cache!");
			return new Vector();
		}

		// check cache
		String key = presenceReference(locationId);
		List ids = (List) m_cache.get(key);

		// otherwise get from storage and cache
		if (ids == null)
		{
			// check storage
			ids = m_storage.getSessions(locationId);

			// cache it
			m_cache.put(key, ids);
		}

		// make a list of actual session objects
		List sessions = new Vector();
		for (int i = 0; i < ids.size(); i++)
		{
			UsageSession session = UsageSessionService.getSession((String) ids.get(i));
			if (session != null)
			{
				sessions.add(session);
			}
		}

		// sort
		Collections.sort(sessions);
		return sessions;

	} // getPresence

	/**
	* Access a List of all location ids (String).
	* @return The List of all location ids (String) (may be empty).
	*/
	public List getLocations()
	{
		List locations = new Vector();

		// if we have disabled the cache, don't use if
		if (m_cache.disabled())
		{
			locations = m_storage.getLocations();
		}

		else
		{
			// if the cache is complete, use it
			if (m_cache.isComplete())
			{
				locations = m_cache.getIds();
			}

			// otherwise get all the locations from storage and cache them and their presence
			else
			{
				// cache the presence for all these locations...
				synchronized (m_cache)
				{
					if (m_cache.isComplete())
					{
						locations = m_cache.getIds();
					}

					else
					{
						// save up any events to the cache until we get past this load
						m_cache.holdEvents();

						// get from storage
						locations = m_storage.getLocations();

						Iterator it = locations.iterator();
						while (it.hasNext())
						{
							// get and cache the ids for this session
							String locationId = (String) it.next();
							List ids = m_storage.getSessions(locationId);
							m_cache.put(presenceReference(locationId), ids);
						}

						m_cache.setComplete();

						// now we are complete, process any cached events
						m_cache.processEvents();
					}
				}
			}
		}

		Collections.sort(locations);
		return locations;

	} // getLocations

	/**
	* Get the sessions with current presence, as a List of UsageSession objects.
	* @return a Set (UsageSession) containing all sessions with current presence.
	*/
	public Set getPresentSessions()
	{
		Set rv = new HashSet();

		List locations = getLocations();
		for (Iterator iLocations = locations.iterator(); iLocations.hasNext();)
		{
			String location = (String) iLocations.next();
			List sessions = getPresence(location);
			rv.addAll(sessions);
		}

		return rv;

	} // getPresentSessions

	/**
	* Get the sessions with current presence by server, as a Map keyed by server id of UsageSession objects.
	* @return a Map (server id -> Set of UsageSession) containing all sessions with current presence by server.
	*/
	public Map getPresentServers()
	{
		Map rv = new Hashtable();

		List locations = getLocations();
		for (Iterator iLocations = locations.iterator(); iLocations.hasNext();)
		{
			String location = (String) iLocations.next();
			List sessions = getPresence(location);

			for (Iterator iSessions = sessions.iterator(); iSessions.hasNext();)
			{
				UsageSession session = (UsageSession) iSessions.next();

				String serverId = session.getServer();
				Set serverSessions = (Set) rv.get(serverId);
				if (serverSessions == null)
				{
					serverSessions = new HashSet();
					rv.put(serverId, serverSessions);
				}
				serverSessions.add(session);
			}
		}

		return rv;

	} // getPresentServers

	/**
	 * {@inheritDoc}
	 */
	public String locationId(String site, String page, String tool)
	{
		return PresenceUtil.locationId(site, page, tool);
	}

	/**
	 * {@inheritDoc}
	 */
	public String[] parseLocation(String location)
	{
		return PresenceUtil.parseLocation(location);
	}

	/**
	 * {@inheritDoc}
	 */
	public String getLocationDescription(String location)
	{
		String rv = "";
		String[] loc = parseLocation(location);

		// is it a site?
		Site site = null;
		try
		{
			site = SiteService.getSite(loc[0]);
		}
		catch (IdUnusedException ignore)
		{
			// if a user site, try the default user site
			if (SiteService.isUserSite(loc[0]))
			{
				try
				{
					site = SiteService.getSite(SiteService.getUserSiteId(""));
				}
				catch (IdUnusedException ignoreAgain)
				{
				}
			}
		}

		// if we found the site
		if (site != null)
		{
			rv = site.getTitle() + " (" + site.getId() + ")";

			// for page locations, add the page description
			if (loc[1] != null)
			{
				SitePage p = site.getPage(loc[1]);
				if (p != null)
				{
					rv += " - " + p.getTitle() /* + " (" + p.getId() + ")" */;
				}
				
			}
			// for tool locations, get a tool description, too
			if (loc[2] != null)
			{
				ToolConfiguration t = site.getTool(loc[2]);
				if (t != null)
				{
					rv += " - " + t.getTitle() /* + " (" + t.getId() + ")" */;
				}
			}
		}

		return rv;
	}

	/*******************************************************************************
	* Storage
	*******************************************************************************/

	protected interface Storage
	{
		/**
		* Add this session id's presence at this location.
		* @param sessionId The session id.
		* @param locationId The location id.
		*/
		public void setPresence(String sessionId, String locationId);

		/**
		* Remove this sessions id's presence at this location.
		* @param sessionId The session id.
		* @param locationId The location id.
		*/
		public void removePresence(String sessionId, String locationId);

		/**
		* Access the List of sessions (sessionId String) present at this location.
		* @param locationId The location id.
		* @return The List of sessions (sessionId String) present at this location.
		*/
		public List getSessions(String locationId);

		/**
		* Access the List of all known locations (locationId String).
		* @return The List of all known locations (locationId String).
		*/
		public List getLocations();

	} // Storage

	/*******************************************************************************
	* Maintenance
	*******************************************************************************/

	protected class Maintenance implements Runnable
	{
		/** My thread running my timeout checker. */
		protected Thread m_timeoutChecker = null;

		/** Signal to the timeout checker to stop. */
		protected boolean m_timeoutCheckerStop = false;

		/** Map of session presences - keyed by SessionId
			- values a map of presence, keyed by locationId, value a Presence. */
		protected Map m_sessions = null;

		public Maintenance()
		{
			m_sessions = new HashMap();

		} // Maintenance

		public void start()
		{
			if (m_timeoutChecker != null)
				return;

			m_timeoutChecker = new Thread(this, "BasePresenceService.TimeoutChecker");
			m_timeoutCheckerStop = false;
			m_timeoutChecker.start();

		} // start

		public void stop()
		{
			if (m_timeoutChecker != null)
			{
				m_timeoutCheckerStop = true;
				m_timeoutChecker.interrupt();
				try
				{
					// wait for it to die
					m_timeoutChecker.join();
				}
				catch (InterruptedException ignore)
				{
				}
				m_timeoutChecker = null;
			}

			// remove any presence remaining
			synchronized (m_sessions)
			{
				Iterator i = m_sessions.entrySet().iterator();
				while (i.hasNext())
				{
					Map.Entry locEntry = (Map.Entry) i.next();
					String sessionId = (String) locEntry.getKey();
					TrackedSession ts = (TrackedSession) locEntry.getValue();
					if (ts == null)
						continue;

					Map locations = ts.locations;
					if (locations.isEmpty())
						continue;

					Iterator p = locations.entrySet().iterator();
					while (p.hasNext())
					{
						Map.Entry presEntry = (Map.Entry) p.next();
						Presence presence = (Presence) presEntry.getValue();
						String locationId = (String) presEntry.getKey();

						// Note: no  locations.remove(locationId); or ts.absentedLocations.add(locationId); -ggolden

						// remove it from storage
						m_storage.removePresence(sessionId, locationId);

						// generate the event
						Event event = EventTrackingService.newEvent(EVENT_ABSENCE, presenceReference(locationId), true);
						EventTrackingService.post(event, UsageSessionService.getSession(sessionId));
					}
				}
			}

		} // stop

		/**
		* In each iteration, check all sessions, each location, and retire any
		* presence that has timed out.  If there are keys associated with this
		* presence, retire them.
		*/
		public void run()
		{
			if (m_logger.isDebugEnabled())
				m_logger.debug(this +".run()");

			while (!m_timeoutCheckerStop)
			{
				try
				{
					if (m_logger.isDebugEnabled())
						m_logger.debug(this +".checking...");

					long now = System.currentTimeMillis();

					// grab the current set of sessions
					List sessionIds = new Vector();
					synchronized (m_sessions)
					{
						sessionIds.addAll(m_sessions.keySet());
					}

					// check each one
					for (int i = 0; i < sessionIds.size(); i++)
					{
						String sessionId = (String) sessionIds.get(i);
						TrackedSession ts = null;
						synchronized (m_sessions)
						{
							ts = (TrackedSession) m_sessions.get(sessionId);
						}
						if (ts == null)
							continue;

						UsageSession session = UsageSessionService.getSession(sessionId);

						// if this session has had no recent activity, log it out
						if (ts.isExpired(now))
						{
							// close the entire session's presence
							removePresence(session);

							// close the session
							session.forceClosed();

							if (m_logger.isDebugEnabled())
								m_logger.debug(this +".run(): session inactivity timeout: " + sessionId);
						}

						else
						{
							// grab the sessions's locations
							Map locations = ts.locations;
							if (locations.isEmpty())
								continue;

							List presences = new Vector();
							synchronized (m_sessions)
							{
								presences.addAll(locations.entrySet());
							}

							for (int p = 0; p < presences.size(); p++)
							{
								Map.Entry entry = (Map.Entry) presences.get(p);
								Presence presence = (Presence) entry.getValue();
								String locationId = (String) entry.getKey();
								if (presence.isExpired(now))
								{
									removePresence(session, locationId);
								}
							}
						}
					}
				}
				catch (Throwable e)
				{
					m_logger.warn(this +": exception: ", e);
				}

				try
				{
					Thread.sleep(m_checkSeconds * 1000);
				}
				catch (Exception ignore)
				{
				}

			} // while

			if (m_logger.isDebugEnabled())
				m_logger.debug(this +": done");

		} // run

		/**
		* Update this session's active time to show it's active.
		* @param sessionId The session's id
		*/
		public void setActiveSession(String sessionId)
		{
			synchronized (m_sessions)
			{
				TrackedSession ts = (TrackedSession) m_sessions.get(sessionId);
				if (ts != null)
				{
					ts.refresh();
				}
			}

		} // setActiveSession

		/**
		* Establish or refresh the presence of this location in this session.
		* @return true if the sessionId is newly added to the location, false if already there.
		*/
		public boolean setPresence(String sessionId, String locationId)
		{
			boolean newlyAdded = false;

			// find, and if needed, create the locations - presence map
			synchronized (m_sessions)
			{
				TrackedSession ts = (TrackedSession) m_sessions.get(sessionId);
				if (ts == null)
				{
					ts = new TrackedSession();
					m_sessions.put(sessionId, ts);
				}

				Map locations = ts.locations;

				// find, and if needed, create the presence for this session - location
				Presence presence = (Presence) locations.get(locationId);
				if (presence == null)
				{
					presence = new Presence();
					locations.put(locationId, presence);
					newlyAdded = true;
				}

				// refresh the presence's time stamp
				presence.refresh();
			}

			return newlyAdded;

		} // setPresence

		/**
		* If the sessionId is already present in this location, remove it
		* and process removal (i.e. retire the keys if any known).
		* @return true of the sessionId's presence was retired by this call,
		* false if it was not there to begin with.
		*/
		public boolean removePresenceM(String sessionId, String locationId)
		{
			Presence presence = null;

			// find the locations - presence map
			synchronized (m_sessions)
			{
				TrackedSession ts = (TrackedSession) m_sessions.get(sessionId);
				if (ts == null)
					return false;

				Map locations = ts.locations;

				// remove this location from the sessions's presence
				// find the presence for this session - location
				presence = (Presence) locations.get(locationId);
				if (presence == null)
					return false;

				locations.remove(locationId);
				ts.absentedLocations.add(locationId);

				// Note: even if the locations for this sesion becomes empty,
				// leave it in the m_sessions for now.
			}

			return true;

		} // removePresence

		/**
		* Close a session - keep it around before doing removeSession so we can clean it up in context.
		* @param id The session id to close.
		*/
		public void closeSession(String sessionId)
		{
			Map locations = null;
			synchronized (m_sessions)
			{
				// find the locations - presence map
				TrackedSession ts = (TrackedSession) m_sessions.get(sessionId);
				if (ts == null)
					return;

				locations = ts.locations;
				
				ts.closed = true;
			}

			if (!locations.isEmpty())
			{
				m_logger.warn(this +".closeSession(): locations not empty: session: " + sessionId);
			}
		}

		/**
		* Remove a session from the sessions we are maintaining if it is closed.
		* @param id The session id to remove.
		*/
		public void removeClosedSession(String sessionId)
		{
			synchronized (m_sessions)
			{
				// find the locations - presence map
				TrackedSession ts = (TrackedSession) m_sessions.get(sessionId);
				if (ts == null)
					return;

				if (!ts.closed)
					return;

				m_sessions.remove(sessionId);
			}
		}

		/**
		 * Cleanup any absenced locations from this session.
		* @param id The session id to cleanup.
		 */
		public void cleanupAbsenses(String sessionId)
		{
			TrackedSession ts = (TrackedSession) m_sessions.get(sessionId);
			if (ts == null)
				return;

			synchronized (ts)
			{
				for (Iterator i = ts.absentedLocations.iterator(); i.hasNext();)
				{
					String location = (String) i.next();
//					m_logger.warn(this + ".cleanupAbsenses: " + location);

					String[] loc = parseLocation(location);

					// site presence?
					if ((loc[0] != null) &&(loc[1] == null) && (loc[2] == null))
					{
						// if there's a current session in this site's name, clear it
						SessionState state = m_stateService.getCurrentSessionState(loc[0]);
						state.clear();
//						m_logger.warn(this + ".cleanupAbsenses: clearing site state " + state.getKey());
					}

					// page presence?
					if ((loc[0] != null) && (loc[1] != null) && (loc[2] == null))
					{
						// if there's a current session in this page's name, clear it
						SessionState state = m_stateService.getCurrentSessionState(loc[1]);
						state.clear();
//						m_logger.warn(this + ".cleanupAbsenses: clearing page state " + state.getKey());
					}

					// tool presence?
					if ((loc[0] != null) && (loc[1] != null) && (loc[2] != null))
					{
						// if there's a current session in this page's name, clear it
						SessionState state = m_stateService.getCurrentSessionState(loc[2]);
						state.clear();
//						m_logger.warn(this + ".cleanupAbsenses: clearing tool state " + state.getKey());
					}
				}
				
				ts.absentedLocations.clear();
			}
		}

		/**
		* For this one of our sessions, access the location ids present.
		* @return a List of location ids (String) in which this session is present.
		*/
		public List getLocations(String sessionId)
		{
			// find the locations - presence map
			synchronized (m_sessions)
			{
				List rv = new Vector();

				TrackedSession ts = (TrackedSession) m_sessions.get(sessionId);
				if (ts == null)
					return rv;

				Map locations = ts.locations;
				if (locations.isEmpty())
					return rv;

				rv.addAll(locations.keySet());

				return rv;
			}

		} // getLocations

	} // class Maintenance

	/*******************************************************************************
	* TrackedSession
	*******************************************************************************/

	protected class TrackedSession
	{
		/** The location map. */
		public Map locations = new HashMap();

		/** The last activity time stamp. */
		protected long m_timestamp = System.currentTimeMillis();

		/** Set if this session has been closed, and is waiting for cleanup. */
		public boolean closed = false;

		/** A set of location strings for presence that has been absenced, awating cleanup. */
		public Set absentedLocations = new HashSet();

		/**
		* Refresh the time-out time stamp.
		*/
		public void refresh()
		{
			m_timestamp = System.currentTimeMillis();

		} // refresh

		/**
		* Check to see if this has expired via time out.
		* @param now The current time.
		* @return true if this has expired via time out, false if not.
		*/
		public boolean isExpired(long now)
		{
			return (m_timestamp + (m_sessionTimeoutSeconds * 1000) < now);

		} // isExpired

	} // class TrackedSession

	/*******************************************************************************
	* Presence
	*******************************************************************************/

	protected class Presence
	{
		/** The time last refreshed for this presence. */
		protected long m_timestamp = 0;

		/**
		* Construct a Presence.
		*/
		public Presence()
		{
			m_timestamp = System.currentTimeMillis();

		} // Presence

		/**
		* Refresh the time-out time stamp.
		*/
		public void refresh()
		{
			m_timestamp = System.currentTimeMillis();

		} // refresh

		/**
		* Check to see if this has expired via time out.
		* @param now The current time.
		* @return true if this has expired via time out, false if not.
		*/
		public boolean isExpired(long now)
		{
			return (m_timestamp + (m_timeoutSeconds * 1000) < now);

		} // isExpired

	} // class Presence

	/*******************************************************************************
	* CacheRefresher implementation (no container)
	*******************************************************************************/

	/**
	* Get a new value for this key whose value has already expired in the cache.
	* @param key The key whose value has expired and needs to be refreshed.
	* @param oldValue The old exipred value of the key.
	* @param event The event which triggered this refresh.
	* @return a new value for use in the cache for this key; if null, the entry will be removed.
	*/
	public Object refresh(Object key, Object oldValue, Event event)
	{
		if (m_logger.isDebugEnabled())
			m_logger.debug(this +".refresh(): " + key + " event: " + event);

		if (event == null)
			return oldValue;

		String sessionId = event.getSessionId();
		if (sessionId == null)
			return oldValue;

		// if we don't have a value yet
		if (oldValue == null)
		{
			// get the ids for the event's reference's location id from storage
			String locationId = presenceId(event.getResource());
			List rv = m_storage.getSessions(locationId);

			// if empty of presence, return a null
			if (rv.isEmpty())
				rv = null;
			return rv;
		}

		// else update the old value List and return it
		List ids = (List) oldValue;
		if (event.getEvent().equals(EVENT_PRESENCE))
		{
			if (!ids.contains(sessionId))
				ids.add(sessionId);
		}
		else if (event.getEvent().equals(EVENT_ABSENCE))
		{
			if (ids.contains(sessionId))
				ids.remove(sessionId);
		}

		// if empty of presence, return a null
		if (ids.isEmpty())
			oldValue = null;

		return oldValue;

	} // refresh

} // PresenceService

/**********************************************************************************
*
* $Header: /cvs/sakai/legacy-component/src/java/org/sakaiproject/component/legacy/presence/BasePresenceService.java,v 1.14 2004/06/22 03:14:01 ggolden Exp $
*
**********************************************************************************/
