/**********************************************************************************
*
* $Header: /cvs/sakai/legacy-component/src/java/org/sakaiproject/component/legacy/realm/Abilities.java,v 1.5 2004/06/22 03:13:55 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

// package
package org.sakaiproject.component.legacy.realm;

// imports
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.sakaiproject.service.legacy.realm.Realm;
import org.sakaiproject.service.legacy.realm.Role;

/**
* <p>A set of abilities - locks that may be unlocked, roles that may be used for their abilities.</p>
* 
* @author University of Michigan, CHEF Software Development Team
* @version $Revision: 1.5 $
*/
public class Abilities
	extends HashSet
{
	/**
	* Construct.
	*/
	public Abilities()
	{
		super();

	}	// Abilities

	/**
	* Construct as a copy
	*/
	public Abilities(Abilities other)
	{
		super(other);

	}	// Abilities

	/**
	* Check this ability set for a lock.
	* @param lock The lock.
	* @param helperRealms Other realms that might refine role abilities.
	* @return true if this lock is within this abilities, false if not.
	*/
	public boolean unlock(String lock, List helperRealms)
	{
		// direct?
		if (contains(lock)) return true;

		// check each role
		for (Iterator it = iterator(); it.hasNext();)
		{
			Object next = it.next();
			if (next instanceof Role)
			{
				if (((Role) next).unlock(lock)) return true;
				
				// see if this role is refined in the helper realms
				if ((helperRealms != null) && (helperRealms.size() > 0))
				{
					for (Iterator iRealms = helperRealms.iterator(); iRealms.hasNext(); )
					{
						Realm realm = (Realm) iRealms.next();

						// find this role by id
						Role role = realm.getRole(((Role) next).getId());
						if (role != null)
						{
							if (role.unlock(lock)) return true;
						}
					}
				}
			}
		}

		return false;

	}	// unlock

	/**
	* Check for this role in this abilities.
	* @param role The Role.
	*/
	public boolean hasRole(Role role)
	{
		// direct?
		if (contains(role)) return true;

		// check each role
		for (Iterator it = iterator(); it.hasNext();)
		{
			Object next = it.next();
			if (next instanceof Role)
			{
				if (((Role) next).hasRole(role)) return true;
			}
		}

		return false;

	}	// hasRole

	/**
	* Access all the locks this ability set can open.
	* @param helperRealms Other realms that might refine role abilities.
	* @return The Set of locks (String) this ability set can open.
	*/
	public Set collectLocks(List helperRealms)
	{
		Set rv = new HashSet();
		
		// check all abilities
		for (Iterator it = iterator(); it.hasNext();)
		{
			Object next = it.next();

			// String - a lock
			if (next instanceof String)
			{
				rv.add(next);
			}

			// else, a Role
			else
			{
				rv.addAll(((Role) next).collectLocks());

				// see if this role is refined in the helper realms
				if ((helperRealms != null) && (helperRealms.size() > 0))
				{
					for (Iterator iRealms = helperRealms.iterator(); iRealms.hasNext(); )
					{
						Realm realm = (Realm) iRealms.next();

						// find this role by id
						Role role = realm.getRole(((Role) next).getId());
						if (role != null)
						{
							rv.addAll(role.collectLocks());
						}
					}
				}
			}
		}

		return rv;

	}	// collectLocks

	/**
	* Access all the Roles this ability set can open.
	* @return The Set of roles (Role) this ability set can open.
	*/
	public Set collectRoles()
	{
		Set rv = new HashSet();

		// check all abilities
		for (Iterator it = iterator(); it.hasNext();)
		{
			Object next = it.next();

			// for the Roles
			if (next instanceof Role)
			{
				// add this Role
				rv.add(next);
				
				// and any Roles contained herein
				rv.addAll(((Role) next).collectRoles());
			}
		}

		return rv;

	}	// collectRoles

	/**
	* Access all the locks directly added to this ability set.
	* @return The Set of locks (String) directly added to this ability set.
	*/
	public Set getLocks()
	{
		Set rv = new HashSet();
		
		// check all abilities
		for (Iterator it = iterator(); it.hasNext();)
		{
			Object next = it.next();

			// String - a lock
			if (next instanceof String)
			{
				rv.add(next);
			}
		}

		return rv;

	}	// getLocks

	/**
	* Access all roles directly added to this ability set.
	* @return The Set of roles (Role) directly added to this ability set.
	*/
	public Set getRoles()
	{
		Set rv = new HashSet();
		
		// check all abilities
		for (Iterator it = iterator(); it.hasNext();)
		{
			Object next = it.next();

			// for the Roles
			if (next instanceof Role)
			{
				rv.add(next);
			}
		}

		return rv;

	}	// getRoles

}	// Abilities

/**********************************************************************************
*
* $Header: /cvs/sakai/legacy-component/src/java/org/sakaiproject/component/legacy/realm/Abilities.java,v 1.5 2004/06/22 03:13:55 ggolden Exp $
*
**********************************************************************************/
