/**********************************************************************************
*
* $Header: /cvs/sakai/legacy-component/src/java/org/sakaiproject/component/legacy/user/BaseUserDirectoryService.java,v 1.15 2004/06/22 03:13:55 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

// package
package org.sakaiproject.component.legacy.user;

// imports
import java.util.Iterator;
import java.util.List;
import java.util.Stack;
import java.util.Vector;

import org.sakaiproject.exception.IdInvalidException;
import org.sakaiproject.exception.IdUnusedException;
import org.sakaiproject.exception.IdUsedException;
import org.sakaiproject.exception.InUseException;
import org.sakaiproject.exception.PermissionException;
import org.sakaiproject.service.framework.config.cover.ServerConfigurationService;
import org.sakaiproject.service.legacy.event.Event;
import org.sakaiproject.service.legacy.event.cover.EventTrackingService;
import org.sakaiproject.service.framework.log.Logger;
import org.sakaiproject.service.framework.memory.Cache;
import org.sakaiproject.service.framework.memory.CacheRefresher;
import org.sakaiproject.service.framework.memory.MemoryService;
import org.sakaiproject.service.legacy.realm.cover.RealmService;
import org.sakaiproject.service.legacy.resource.Edit;
import org.sakaiproject.service.legacy.resource.Resource;
import org.sakaiproject.service.legacy.resource.ResourceProperties;
import org.sakaiproject.service.legacy.resource.ResourcePropertiesEdit;
import org.sakaiproject.service.legacy.security.cover.SecurityService;
import org.sakaiproject.service.framework.session.cover.UsageSessionService;
import org.sakaiproject.service.framework.state.SessionStateBindingListener;
import org.sakaiproject.service.legacy.time.cover.TimeService;
import org.sakaiproject.service.legacy.user.User;
import org.sakaiproject.service.legacy.user.UserDirectoryService;
import org.sakaiproject.service.legacy.user.UserEdit;
import org.sakaiproject.util.Validator;
import org.sakaiproject.util.resource.BaseResourcePropertiesEdit;
import org.sakaiproject.util.storage.StorageUser;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
* <p>BaseUserDirectoryService is a CHEF user directory services implemented as a Turbine Service.</p>
* 
* @author University of Michigan, CHEF Software Development Team
* @version $Revision: 1.15 $
* @see org.chefproject.core.User
*/
public abstract class BaseUserDirectoryService implements UserDirectoryService, StorageUser, CacheRefresher
{
	/** Storage manager for this service. */
	protected Storage m_storage = null;

	/** A Cache for this service - Users keyed by reference. */
	protected Cache m_cache = null;

	/** The initial portion of a relative access point URL. */
	protected String m_relativeAccessPoint = null;

	/** An anon. user. */
	protected User m_anon = null;

	/** A user directory provider class name, as configured. */
	protected String m_providerClassName = null;

	/** A user directory provider. */
	protected UserDirectoryProvider m_provider = null;

	/*******************************************************************************
	* Abstractions, etc.
	*******************************************************************************/

	/**
	* Construct storage for this service.
	*/
	protected abstract Storage newStorage();

	/**
	* Access the partial URL that forms the root of resource URLs.
	* @param relative if true, form within the access path only (i.e. starting with /content)
	* @return the partial URL that forms the root of resource URLs.
	*/
	protected String getAccessPoint(boolean relative)
	{
		return (relative ? "" : ServerConfigurationService.getAccessUrl()) + m_relativeAccessPoint;

	} // getAccessPoint

	/**
	* Access the internal reference which can be used to access the resource from within the system.
	* @param id The user id string.
	* @return The the internal reference which can be used to access the resource from within the system.
	*/
	public String userReference(String id)
	{
		return getAccessPoint(true) + Resource.SEPARATOR + id;

	} // userReference

	/**
	* Access the user id extracted from a user reference.
	* @param ref The user reference string.
	* @return The the user id extracted from a user reference.
	*/
	protected String userId(String ref)
	{
		String start = getAccessPoint(true) + Resource.SEPARATOR;
		int i = ref.indexOf(start);
		if (i == -1)
			return ref;
		String id = ref.substring(i + start.length());
		return id;

	} // userId

	/**
	* Check security permission.
	* @param lock The lock id string.
	* @param resource The resource reference string, or null if no resource is involved.
	* @return true if allowd, false if not
	*/
	protected boolean unlockCheck(String lock, String resource)
	{
		if (!SecurityService.unlock(lock, resource))
		{
			return false;
		}

		return true;

	} // unlockCheck

	/**
	* Check security permission.
	* @param lock1 The lock id string.
	* @param lock2 The lock id string.
	* @param resource The resource reference string, or null if no resource is involved.
	* @return true if either allowed, false if not
	*/
	protected boolean unlockCheck2(String lock1, String lock2, String resource)
	{
		if (!SecurityService.unlock(lock1, resource))
		{
			if (!SecurityService.unlock(lock2, resource))
			{
				return false;
			}
		}

		return true;

	} // unlockCheck2

	/**
	* Check security permission.
	* @param lock The lock id string.
	* @param resource The resource reference string, or null if no resource is involved.
	* @exception PermissionException Thrown if the user does not have access
	*/
	protected void unlock(String lock, String resource) throws PermissionException
	{
		if (!unlockCheck(lock, resource))
		{
			throw new PermissionException(UsageSessionService.getSessionUser().getId(), lock, resource);
		}

	} // unlock

	/**
	* Check security permission.
	* @param lock1 The lock id string.
	* @param lock2 The lock id string.
	* @param resource The resource reference string, or null if no resource is involved.
	* @exception PermissionException Thrown if the user does not have access to either.
	*/
	protected void unlock2(String lock1, String lock2, String resource) throws PermissionException
	{
		if (!unlockCheck2(lock1, lock2, resource))
		{
			throw new PermissionException(UsageSessionService.getSessionUser().getId(), lock1 + "/" + lock2, resource);
		}

	} // unlock2

	/*******************************************************************************
	* Dependencies and their setter methods
	*******************************************************************************/

	/** Dependency: logging service */
	protected Logger m_logger = null;

	/**
	 * Dependency: logging service.
	 * @param service The logging service.
	 */
	public void setLogger(Logger service)
	{
		m_logger = service;
	}

	/** Dependency: memory service */
	protected MemoryService m_memoryService = null;

	/**
	 * Dependency: MemoryService.
	 * @param service The MemoryService
	 */
	public void setMemoryService(MemoryService service)
	{
		m_memoryService = service;
	}

	/**
	 * Configuration: set the realm provider class name.
	 * @param name the realm provider class name.
	 */
	public void setProvider(String name)
	{
		m_providerClassName = name;
	}

	/*******************************************************************************
	* Init and Destroy
	*******************************************************************************/

	/**
	 * Final initialization, once all dependencies are set.
	 */
	public void init()
	{
		try
		{
			m_relativeAccessPoint = REFERENCE_ROOT;

			// construct storage and read
			m_storage = newStorage();
			m_storage.open();

			// make the cache
			m_cache = m_memoryService.newCache(this, userReference(""));

			// make an anon. user
			m_anon = new BaseUser("");

			// create provider
			if (m_providerClassName != null)
			{
				try
				{
					m_provider = (UserDirectoryProvider) Class.forName(m_providerClassName).newInstance();
					m_logger.info(this +".init(): provider: " + m_providerClassName);
				}
				catch (Exception e)
				{
					m_logger.warn(this +".init(): trouble loading provider: " + e);
				}
			}

			m_logger.info(this +".init()");
		}
		catch (Throwable t)
		{
			m_logger.warn(this +".init(): ", t);
		}

	} // init

	/**
	* Returns to uninitialized state.
	*
	* You can use this method to release resources thet your Service
	* allocated when Turbine shuts down.
	*/
	public void destroy()
	{
		m_cache.destroy();
		m_cache = null;
		m_storage.close();
		m_storage = null;
		m_provider = null;
		m_anon = null;

		m_logger.info(this +".destroy()");

	} // destroy

	/*******************************************************************************
	* UserDirectoryService implementation
	*******************************************************************************/

	/**
	* Access a user object.
	* @param id The user id string.
	* @return A user object containing the user information
	* @exception IdUnusedException if not found
	*/
	public User getUser(String id) throws IdUnusedException
	{
		if (id == null)
			throw new IdUnusedException("null");

		UserEdit user = findUser(id);

		// if the provider is a CompleteUdp, it will provide the full information
		if ((user == null) && (m_provider != null) && (m_provider instanceof CompleteUdp))
		{
			// make a new edit to hold the provider's info
			user = new BaseUserEdit(id);
			if (!((CompleteUdp) m_provider).getUser(user))
			{
				user = null;
			}
		}

		// if not found at all
		if (user == null)
		{
			throw new IdUnusedException(id);
		}

		// track it - Note: we are not tracking user access -ggolden
		// EventTrackingService.post(EventTrackingService.newEvent(SECURE_ACCESS_USER, user.getReference()));

		return user;

	} // getUser

	/**
	* check permissions for updating a user
	* (i.s. using getUserEditClone() and calling commitEditClone())
	* @param id The user id.
	* @return true if the user is allowed to update the user, false if not.
	*/
	public boolean allowUpdateUser(String id)
	{
		// is this the user's own?
		if (id.equals(UsageSessionService.getSessionUser().getId()))
		{
			// own or any
			return unlockCheck2(SECURE_UPDATE_USER_OWN, SECURE_UPDATE_USER_ANY, userReference(id));
		}

		else
		{
			// just any
			return unlockCheck(SECURE_UPDATE_USER_ANY, userReference(id));
		}

	} // allowUpdateUser

	/**
	* Get a locked user object for editing.  Must commitEdit() to make official, or cancelEdit() when done!
	* @param id The user id string.
	* @return A UserEdit object for editing.
	* @exception IdUnusedException if not found, or if not an UserEdit object
	* @exception PermissionException if the current user does not have permission to mess with this user.
	* @exception InUseException if the current user does not have permission to mess with this user.
	*/
	public UserEdit editUser(String id) throws IdUnusedException, PermissionException, InUseException
	{
		if (id == null)
			throw new IdUnusedException("null");

		// is this the user's own?
		String function = null;
		if (id.equals(UsageSessionService.getSessionUser().getId()))
		{
			// own or any
			unlock2(SECURE_UPDATE_USER_OWN, SECURE_UPDATE_USER_ANY, userReference(id));
			function = SECURE_UPDATE_USER_OWN;
		}
		else
		{
			// just any
			unlock(SECURE_UPDATE_USER_ANY, userReference(id));
			function = SECURE_UPDATE_USER_ANY;
		}

		// check for existance
		if ((m_cache.get(userReference(id)) == null) && (!m_storage.check(id)))
		{
			throw new IdUnusedException(id);
		}

		// ignore the cache - get the user with a lock from the info store
		UserEdit user = m_storage.edit(id);
		if (user == null)
			throw new InUseException(id);

		((BaseUserEdit) user).setEvent(function);

		return user;

	} // editUser

	/**
	* Commit the changes made to a UserEdit object, and release the lock.
	* @param user The UserEdit object to commit.
	*/
	public void commitEdit(UserEdit user)
	{
		// check for closed edit
		if (!user.isActiveEdit())
		{
			try
			{
				throw new Exception();
			}
			catch (Exception e)
			{
				m_logger.warn(this +".commitEdit(): closed UserEdit", e);
			}
			return;
		}

		// update the properties
		addLiveUpdateProperties(user.getPropertiesEdit());

		// complete the edit
		m_storage.commit(user);

		// track it
		EventTrackingService.post(
			EventTrackingService.newEvent(((BaseUserEdit) user).getEvent(), user.getReference(), true));

		// close the edit object
		 ((BaseUserEdit) user).closeEdit();

		// %%% sync with portal service, i.e. user's portal page, any others? -ggolden

	} // commitEdit

	/**
	* Cancel the changes made to a UserEdit object, and release the lock.
	* @param user The UserEdit object to commit.
	*/
	public void cancelEdit(UserEdit user)
	{
		// check for closed edit
		if (!user.isActiveEdit())
		{
			try
			{
				throw new Exception();
			}
			catch (Exception e)
			{
				m_logger.warn(this +".cancelEdit(): closed UserEdit", e);
			}
			return;
		}

		// release the edit lock
		m_storage.cancel(user);

		// close the edit object
		 ((BaseUserEdit) user).closeEdit();

	} // cancelEdit

	/**
	* Access all user objects - known to us (not from external providers).
	* @return A list of user objects containing each user's information.
	*/
	public List getUsers()
	{
		List users = new Vector();

		// if we have disabled the cache, don't use if
		if (m_cache.disabled())
		{
			users = m_storage.getAll();
		}

		else
		{
			// if the cache is complete, use it
			if (m_cache.isComplete())
			{
				users = m_cache.getAll();
			}

			// otherwise get all the users from storage
			else
			{
				// Note: while we are getting from storage, storage might change.  These can be processed
				// after we get the storage entries, and put them in the cache, and mark the cache complete.
				// -ggolden
				synchronized (m_cache)
				{
					// if we were waiting and it's now complete...
					if (m_cache.isComplete())
					{
						users = m_cache.getAll();
						return users;
					}

					// save up any events to the cache until we get past this load
					m_cache.holdEvents();

					users = m_storage.getAll();

					// update the cache, and mark it complete
					for (int i = 0; i < users.size(); i++)
					{
						User user = (User) users.get(i);
						m_cache.put(user.getReference(), user);
					}

					m_cache.setComplete();

					// now we are complete, process any cached events
					m_cache.processEvents();
				}
			}
		}

		return users;

	} // getUsers

	/**
	* Find a user object who has this email address.
	* @param email The email address string.
	* @return A user object containing the user information
	* @exception IdUnusedException if not found
	*/
	public User findUserByEmail(String email) throws IdUnusedException
	{
		// if the provider is a CompleteUdp, it will provide the full information
		if ((m_provider != null) && (m_provider instanceof CompleteUdp))
		{
			// make a new edit to hold the provider's info
			UserEdit user = new BaseUserEdit((String) null);
			if (((CompleteUdp) m_provider).findUserByEmail(user, email))
			{
				return user;
			}
		}

		// check internal users
		List users = getUsers();
		for (Iterator iUsers = users.iterator(); iUsers.hasNext();)
		{
			User user = (User) iUsers.next();
			if (user.getEmail().equalsIgnoreCase(email))
			{
				return user;
			}
		}

		throw new IdUnusedException(email);

	} // findUserByEmail

	/**
	* Access the anonymous user object.
	* @return the anonymous user object.
	*/
	public User getAnonymousUser()
	{
		return m_anon;

	} // getAnonymousUser

	/**
	* check permissions for addUser().
	* @param id The user id.
	* @return true if the user is allowed to addUser(id), false if not.
	*/
	public boolean allowAddUser(String id)
	{
		return unlockCheck(SECURE_ADD_USER, userReference(id));

	} // allowAddUser

	/**
	* Add a new user to the directory.  Must commit() to make official, or cancel() when done!
	* @param id The user id.
	* @return An edit clone to the new user object.
	* @exception IdInvalidException if the user id is invalid.
	* @exception IdUsedException if the user id is already used.
	* @exception PermissionException if the current user does not have permission to add a user.
	*/
	public UserEdit addUser(String id) throws IdInvalidException, IdUsedException, PermissionException
	{
		// check for a valid user name
		Validator.checkResourceId(id);

		// check security (throws if not permitted)
		unlock(SECURE_ADD_USER, userReference(id));

		// if we have a CompleteUdp provider, we cannot use an id locally that exists there
		if ((m_provider != null) && (m_provider instanceof CompleteUdp) && ((CompleteUdp) m_provider).userExists(id))
		{
			throw new IdUsedException(id);
		}

		// reserve a user with this id from the info store - if it's in use, this will return null
		UserEdit user = m_storage.put(id);
		if (user == null)
		{
			throw new IdUsedException(id);
		}

		((BaseUserEdit) user).setEvent(SECURE_ADD_USER);

		return user;

	} // addUser

	/**
	* Add a new user to the directory, from a definition in XML
	* @param el The XML DOM Element defining the user.
	* @return An edit clone to the new user object.
	* @exception IdInvalidException if the user id is invalid.
	* @exception IdUsedException if the user id is already used.
	* @exception PermissionException if the current user does not have permission to add a user.
	*/
	public UserEdit mergeUser(Element el) throws IdInvalidException, IdUsedException, PermissionException
	{
		// construct from the XML
		User userFromXml = new BaseUser(el);

		// check for a valid user name
		Validator.checkResourceId(userFromXml.getId());

		// check security (throws if not permitted)
		unlock(SECURE_ADD_USER, userFromXml.getReference());

		// if we have a CompleteUdp provider, we cannot use an id locally that exists there
		if ((m_provider != null)
			&& (m_provider instanceof CompleteUdp)
			&& ((CompleteUdp) m_provider).userExists(userFromXml.getId()))
		{
			throw new IdUsedException(userFromXml.getId());
		}

		// reserve a user with this id from the info store - if it's in use, this will return null
		UserEdit user = m_storage.put(userFromXml.getId());
		if (user == null)
		{
			throw new IdUsedException(userFromXml.getId());
		}

		// transfer from the XML read user object to the UserEdit
		 ((BaseUserEdit) user).set(userFromXml);

		((BaseUserEdit) user).setEvent(SECURE_ADD_USER);

		return user;

	} // mergeUser

	/**
	* check permissions for removeUser().
	* @param id The user id.
	* @return true if the user is allowed to removeUser(id), false if not.
	*/
	public boolean allowRemoveUser(String id)
	{
		return unlockCheck(SECURE_REMOVE_USER, userReference(id));

	} // allowRemoveUser

	/**
	* Remove this user's information from the directory - it must be a user with a lock from editUser()
	* @param user The locked user object to remove.
	* @exception PermissionException if the current user does not have permission to remove this user.
	*/
	public void removeUser(UserEdit user) throws PermissionException
	{
		// check for closed edit
		if (!user.isActiveEdit())
		{
			try
			{
				throw new Exception();
			}
			catch (Exception e)
			{
				m_logger.warn(this +".removeUser(): closed UserEdit", e);
			}
			return;
		}

		// check security (throws if not permitted)
		unlock(SECURE_REMOVE_USER, user.getReference());

		// complete the edit
		m_storage.remove(user);

		// track it
		EventTrackingService.post(EventTrackingService.newEvent(SECURE_REMOVE_USER, user.getReference(), true));

		// %%% sync with portal service, i.e. user's portal page, any others? -ggolden

		// close the edit object
		 ((BaseUserEdit) user).closeEdit();

		// remove any realm defined for this resource
		try
		{
			RealmService.removeRealm(RealmService.editRealm(user.getReference()));
		}
		catch (InUseException e)
		{
			m_logger.warn(this +".removeUser: removing realm for : " + user.getReference() + " : " + e);
		}
		catch (PermissionException e)
		{
			m_logger.warn(this +".removeUser: removing realm for : " + user.getReference() + " : " + e);
		}
		catch (IdUnusedException ignore)
		{
		}

	} // removeUser

	/**
	 * {@inheritDoc}
	 */
	public User authenticate(String id, String password)
	{
		boolean authenticated = false;

		// dig up our in-service user record
		UserEdit user = findUser(id);
		// m_logger.info(" *** UserDirectory.authenticate: id: " + id + " found: " + (user != null));

		// if we have a CompleteUdp provider, let it authenticate if we didn't find the user
		if ((m_provider != null) && (m_provider instanceof CompleteUdp) && (user == null))
		{
			authenticated = ((CompleteUdp) m_provider).authenticateUser(id, password);
			// m_logger.info(" *** UserDirectory.authenticate: id: " + id + " result of CUDP.authenticateUser: " + authenticated);
		}

		// if we are dealing with a piggyback udp, and we found the user record, let it authenticate
		if ((!authenticated) && (m_provider != null) && (m_provider instanceof PiggybackUdp) && (user != null))
		{
			authenticated = ((PiggybackUdp) m_provider).authenticateUser(user, password);
			// m_logger.info(" *** UserDirectory.authenticate: id: " + id + " result of PUDP.authenticateUser: " + authenticated);
			if (authenticated)
			{
				// save user
				BaseUserEdit edit = (BaseUserEdit) m_storage.edit(id);
				if (edit != null)
				{
					edit.setAll(user);
					edit.setEvent(SECURE_UPDATE_USER_ANY);

					m_storage.commit(edit);

					EventTrackingService.post(EventTrackingService.newEvent(edit.getEvent(), edit.getReference(), true));
				}
				else
				{
					m_logger.warn(this +".authenticate(): could not save user after auth: " + id);
				}
			}
		}

		// if not yet authenticated, and we have this user, authenticate here
		if ((!authenticated) && (user != null))
		{ // TODO: container auth check!
			//			// if we are accepting container auth, and this user matches the session user
			//			// and the password is null, let the user in.
			//			boolean containerAuthentication = "true".equals(ServerConfigurationService.getString("container.auth","false"));
			//			if ((containerAuthentication) && (password == null))
			//			{
			//				JetspeedRunDataService runDataService = (JetspeedRunDataService)TurbineServices.getInstance()
			//										.getService(RunDataService.SERVICE_NAME);
			//				JetspeedRunData data = runDataService.getCurrentRunData();
			//				if (user.getId().equals(data.getRequest().getRemoteUser()))
			//				{
			//					authenticated = true;
			//				}
			//// m_logger.info(" *** UserDirectory.authenticate: id: " + id + " allowing container authorized: " + authenticated);
			//			}

			// if the container does not match, do a pw check
			if (!authenticated)
			{
				authenticated = user.checkPassword(password);
				// m_logger.info(" *** UserDirectory.authenticate: id: " + id + " result of checkPassword: " + authenticated);
			}
		}

		// if authenticated, return the user
		User rv = null;
		if (authenticated)
		{
			try
			{
				rv = getUser(id);
			}
			catch (Exception ignore)
			{
			}
		}

		return rv;

	} // authenticate

	/**
	 * {@inheritDoc}
	 */
	public void destroyAuthentication()
	{
	}

	/**
	* Find the user object, in cache or storage (only - no provider check).
	* @param id The user id.
	* @return The user object found in cache or storage, or null if not found.
	*/
	protected BaseUserEdit findUser(String id)
	{
		BaseUserEdit user = null;

		// check the cache for presence (or knowledge of absence w/ null)
		String key = userReference(id);
		if (m_cache.containsKey(key))
		{
			user = (BaseUserEdit) m_cache.get(key);
		}

		// get the user from storage
		else
		{
			user = (BaseUserEdit) m_storage.get(id);

			// cache the result, hit or miss
			m_cache.put(key, user);
		}

		return user;

	} // findUser

	/**
	* Create the live properties for the user.
	*/
	protected void addLiveProperties(ResourcePropertiesEdit props)
	{
		String current = UsageSessionService.getSessionUser().getId();

		props.addProperty(ResourceProperties.PROP_CREATOR, current);
		props.addProperty(ResourceProperties.PROP_MODIFIED_BY, current);

		String now = TimeService.newTime().toString();
		props.addProperty(ResourceProperties.PROP_CREATION_DATE, now);
		props.addProperty(ResourceProperties.PROP_MODIFIED_DATE, now);

	} //  addLiveProperties

	/**
	* Update the live properties for a user for when modified.
	*/
	protected void addLiveUpdateProperties(ResourcePropertiesEdit props)
	{
		String current = UsageSessionService.getSessionUser().getId();

		props.addProperty(ResourceProperties.PROP_MODIFIED_BY, current);
		props.addProperty(ResourceProperties.PROP_MODIFIED_DATE, TimeService.newTime().toString());

	} //  addLiveUpdateProperties

	/*******************************************************************************
	* User implementation
	*******************************************************************************/

	/**
	* <p>BaseUser is an implementation of the CHEF User object.</p>
	* 
	* @author University of Michigan, CHEF Software Development Team
	*/
	public class BaseUser implements User
	{
		/** The user id. */
		protected String m_id = null;

		/** The user first name. */
		protected String m_firstName = null;

		/** The user last name. */
		protected String m_lastName = null;

		/** The user email address. */
		protected String m_email = null;

		/** The user password. */
		protected String m_pw = null;

		/** The properties. */
		protected ResourcePropertiesEdit m_properties = null;

		/**
		* Construct.
		* @param id The user id.
		*/
		public BaseUser(String id)
		{
			m_id = id;

			// setup for properties
			ResourcePropertiesEdit props = new BaseResourcePropertiesEdit();
			m_properties = props;

			// if the id is not null (a new user, rather than a reconstruction)
			// and not the anon (id == "") user,
			// add the automatic (live) properties
			if ((m_id != null) && (m_id.length() > 0))
				addLiveProperties(props);

		} // BaseUser

		/**
		* Construct from another User object.
		* @param user The user object to use for values.
		*/
		public BaseUser(User user)
		{
			setAll(user);

		} // BaseUser

		/**
		* Construct from information in XML.
		* @param el The XML DOM Element definining the user.
		*/
		public BaseUser(Element el)
		{
			// setup for properties
			m_properties = new BaseResourcePropertiesEdit();

			m_id = el.getAttribute("id");
			m_firstName = el.getAttribute("first-name");
			if (m_firstName.trim().length() == 0)
				m_firstName = null;
			m_lastName = el.getAttribute("last-name");
			if (m_lastName.trim().length() == 0)
				m_lastName = null;
			m_email = el.getAttribute("email");
			m_pw = el.getAttribute("pw");

			// the children (roles, properties)
			NodeList children = el.getChildNodes();
			final int length = children.getLength();
			for (int i = 0; i < length; i++)
			{
				Node child = children.item(i);
				if (child.getNodeType() != Node.ELEMENT_NODE)
					continue;
				Element element = (Element) child;

				// look for properties
				if (element.getTagName().equals("properties"))
				{
					// re-create properties
					m_properties = new BaseResourcePropertiesEdit(element);
				}
			}

		} // BaseUser

		/**
		* Take all values from this object.
		* @param user The user object to take values from.
		*/
		protected void setAll(User user)
		{
			m_id = user.getId();
			m_firstName = user.getFirstName();
			m_lastName = user.getLastName();
			m_email = user.getEmail();
			m_pw = ((BaseUser) user).m_pw;

			m_properties = new BaseResourcePropertiesEdit();
			m_properties.addAll(user.getProperties());

		} // setAll

		/**
		* Serialize the resource into XML, adding an element to the doc under the top of the stack element.
		* @param doc The DOM doc to contain the XML (or null for a string return).
		* @param stack The DOM elements, the top of which is the containing element of the new "resource" element.
		* @return The newly added element.
		*/
		public Element toXml(Document doc, Stack stack)
		{
			Element user = doc.createElement("user");

			if (stack.isEmpty())
			{
				doc.appendChild(user);
			}
			else
			{
				((Element) stack.peek()).appendChild(user);
			}

			stack.push(user);

			user.setAttribute("id", getId());
			if (m_firstName != null)
				user.setAttribute("first-name", m_firstName);
			if (m_lastName != null)
				user.setAttribute("last-name", m_lastName);
			user.setAttribute("email", getEmail());
			user.setAttribute("pw", m_pw);

			// properties
			m_properties.toXml(doc, stack);

			stack.pop();

			return user;

		} // toXml

		/**
		* Access the user id.
		* @return The user id string.
		*/
		public String getId()
		{
			if (m_id == null)
				return "";
			return m_id;

		} // getId

		/**
		* Access the URL which can be used to access the resource.
		* @return The URL which can be used to access the resource.
		*/
		public String getUrl()
		{
			return getAccessPoint(false) + m_id;

		} // getUrl

		/**
		* Access the internal reference which can be used to access the resource from within the system.
		* @return The the internal reference which can be used to access the resource from within the system.
		*/
		public String getReference()
		{
			return userReference(m_id);

		} // getReference

		/**
		* Access the resources's properties.
		* @return The resources's properties.
		*/
		public ResourceProperties getProperties()
		{
			return m_properties;

		} // getProperties

		/**
		* Access the user's name for display purposes.
		* @return The user's name for display purposes.
		*/
		public String getDisplayName()
		{
			StringBuffer buf = new StringBuffer(128);
			if (m_firstName != null)
				buf.append(m_firstName);
			if (m_lastName != null)
			{
				buf.append(" ");
				buf.append(m_lastName);
			}

			if (buf.length() == 0)
				return getId();

			return buf.toString();

		} // getDisplayName

		/**
		* Access the user's first name.
		* @return The user's first name.
		*/
		public String getFirstName()
		{
			if (m_firstName == null)
				return "";
			return m_firstName;

		} // getFirstName

		/**
		* Access the user's last name.
		* @return The user's last name.
		*/
		public String getLastName()
		{
			if (m_lastName == null)
				return "";
			return m_lastName;

		} // getLastName

		/**
		* Access the user's name for sorting purposes.
		* @return The user's name for sorting purposes.
		*/
		public String getSortName()
		{
			StringBuffer buf = new StringBuffer(128);
			if (m_lastName != null)
				buf.append(m_lastName);
			if (m_firstName != null)
			{
				buf.append(", ");
				buf.append(m_firstName);
			}

			if (buf.length() == 0)
				return getId();

			return buf.toString();

		} // getSortName

		/**
		* Access the email address.
		* @return The email address string.
		*/
		public String getEmail()
		{
			if (m_email == null)
				return "";
			return m_email;

		} // getEmail

		/**
		* Check if this is the user's password.
		* @param pw The clear text password to check.
		* @return true if the password matches, false if not.
		*/
		public boolean checkPassword(String pw)
		{
			// encode this password
			String encoded = OneWayHash.encode(pw);

			if (m_pw.equals(encoded))
				return true;

			return false;

		} // checkPassword

		/**
		* Are these objects equal?  If they are both User objects, and they have
		* matching id's, they are.
		* @return true if they are equal, false if not.
		*/
		public boolean equals(Object obj)
		{
			if (!(obj instanceof User))
				return false;
			return ((User) obj).getId().equals(getId());

		} // equals

		/**
		* Make a hash code that reflects the equals() logic as well.
		* We want two objects, even if different instances, if they have the same id to hash the same.
		*/
		public int hashCode()
		{
			return getId().hashCode();

		} // hashCode

		/**
		* Compare this object with the specified object for order.
		* @return A negative integer, zero, or a positive integer as this object is
		* less than, equal to, or greater than the specified object.
		*/
		public int compareTo(Object obj)
		{
			if (!(obj instanceof User))
				throw new ClassCastException();

			// if the object are the same, say so
			if (obj == this)
				return 0;

			// start the compare by comparing their sort names
			int compare = getSortName().compareTo(((User) obj).getSortName());

			// if these are the same
			if (compare == 0)
			{
				// sort based on (unique) id
				compare = getId().compareTo(((User) obj).getId());
			}

			return compare;

		} // compareTo

	} // BaseUser

	/*******************************************************************************
	* UserEdit implementation
	*******************************************************************************/

	/**
	* <p>BaseUserEdit is an implementation of the CHEF UserEdit object.</p>
	* 
	* @author University of Michigan, CHEF Software Development Team
	*/
	public class BaseUserEdit extends BaseUser implements UserEdit, SessionStateBindingListener
	{
		/** The event code for this edit. */
		protected String m_event = null;

		/** Active flag. */
		protected boolean m_active = false;

		/**
		* Construct.
		* @param id The user id.
		*/
		public BaseUserEdit(String id)
		{
			super(id);

		} // BaseUserEdit

		/**
		* Construct from information in XML.
		* @param el The XML DOM Element definining the user.
		*/
		public BaseUserEdit(Element el)
		{
			super(el);

		} // BaseUserEdit

		/**
		* Construct from another User object.
		* @param user The user object to use for values.
		*/
		public BaseUserEdit(User user)
		{
			super(user);

		} // BaseUserEdit

		/**
		* Clean up.
		*/
		protected void finalize()
		{
			// catch the case where an edit was made but never resolved
			if (m_active)
			{
				cancelEdit(this);
			}

		} // finalize

		/**
		* Set the user's id.
		* Note: this is a special purpose routine that is used only to establish the id field,
		* when the id is null, and cannot be used to change a user's id.
		* @param name The user id.
		*/
		public void setId(String id)
		{
			if (m_id == null)
			{
				m_id = id;
			}

		} // setId

		/**
		* Set the user's first name.
		* @param name The user's first name.
		*/
		public void setFirstName(String name)
		{
			m_firstName = name;

		} // setFirstName

		/**
		* Set the user's last name.
		* @param name The user's last name.
		*/
		public void setLastName(String name)
		{
			m_lastName = name;

		} // setLastName

		/**
		* Set the email address.
		* @param email The email address string.
		*/
		public void setEmail(String email)
		{
			m_email = email;

		} // setEmail

		/**
		* Set the user's password
		* @param pw The user's new password.
		*/
		public void setPassword(String pw)
		{
			// encode this password
			String encoded = OneWayHash.encode(pw);

			m_pw = encoded;

		} // setPassword

		/**
		* Take all values from this object.
		* @param user The user object to take values from.
		*/
		protected void set(User user)
		{
			setAll(user);

		} // set

		/**
		* Access the event code for this edit.
		* @return The event code for this edit.
		*/
		protected String getEvent()
		{
			return m_event;
		}

		/**
		* Set the event code for this edit.
		* @param event The event code for this edit.
		*/
		protected void setEvent(String event)
		{
			m_event = event;
		}

		/**
		* Access the resource's properties for modification
		* @return The resource's properties.
		*/
		public ResourcePropertiesEdit getPropertiesEdit()
		{
			return m_properties;

		} // getPropertiesEdit

		/**
		* Enable editing.
		*/
		protected void activate()
		{
			m_active = true;

		} // activate

		/**
		* Check to see if the edit is still active, or has already been closed.
		* @return true if the edit is active, false if it's been closed.
		*/
		public boolean isActiveEdit()
		{
			return m_active;

		} // isActiveEdit

		/**
		* Close the edit object - it cannot be used after this.
		*/
		protected void closeEdit()
		{
			m_active = false;

		} // closeEdit

		/*******************************************************************************
		* SessionStateBindingListener implementation
		*******************************************************************************/

		/**
		* Accept notification that this object has been bound as a SessionState attribute.
		* @param sessionStateKey The id of the session state which holds the attribute.
		* @param attributeName The id of the attribute to which this object is now the value.
		*/
		public void valueBound(String sessionStateKey, String attributeName)
		{
		}

		/**
		* Accept notification that this object has been removed from a SessionState attribute.
		* @param sessionStateKey The id of the session state which held the attribute.
		* @param attributeName The id of the attribute to which this object was the value.
		*/
		public void valueUnbound(String sessionStateKey, String attributeName)
		{
			if (m_logger.isDebugEnabled())
				m_logger.debug(this +".valueUnbound()");

			// catch the case where an edit was made but never resolved
			if (m_active)
			{
				cancelEdit(this);
			}

		} // valueUnbound

	} // BaseUserEdit

	/*******************************************************************************
	* Storage
	*******************************************************************************/

	protected interface Storage
	{
		/**
		* Open.
		*/
		public void open();

		/**
		* Close.
		*/
		public void close();

		/**
		* Check if a user by this id exists.
		* @param id The user id.
		* @return true if a user by this id exists, false if not.
		*/
		public boolean check(String id);

		/**
		* Get the user with this id, or null if not found.
		* @param id The user id.
		* @return The user with this id, or null if not found.
		*/
		public UserEdit get(String id);

		/**
		* Get all users.
		* @return The List (UserEdit) of all users.
		*/
		public List getAll();

		/**
		* Add a new user with this id.
		* @param id The user id.
		* @return The locked User object with this id, or null if the id is in use.
		*/
		public UserEdit put(String id);

		/**
		* Get a lock on the user with this id, or null if a lock cannot be gotten.
		* @param id The user id.
		* @return The locked User with this id, or null if this records cannot be locked.
		*/
		public UserEdit edit(String id);

		/**
		* Commit the changes and release the lock.
		* @param user The user to commit.
		*/
		public void commit(UserEdit user);

		/**
		* Cancel the changes and release the lock.
		* @param user The user to commit.
		*/
		public void cancel(UserEdit user);

		/**
		* Remove this user.
		* @param user The user to remove.
		*/
		public void remove(UserEdit user);

	} // Storage

	/*******************************************************************************
	* StorageUser implementation (no container)
	*******************************************************************************/

	/**
	* Construct a new continer given just an id.
	* @param id The id for the new object.
	* @return The new containe Resource.
	*/
	public Resource newContainer(String ref)
	{
		return null;
	}

	/**
	* Construct a new container resource, from an XML element.
	* @param element The XML.
	* @return The new container resource.
	*/
	public Resource newContainer(Element element)
	{
		return null;
	}

	/**
	* Construct a new container resource, as a copy of another
	* @param other The other contianer to copy.
	* @return The new container resource.
	*/
	public Resource newContainer(Resource other)
	{
		return null;
	}

	/**
	* Construct a new resource given just an id.
	* @param container The Resource that is the container for the new resource (may be null).
	* @param id The id for the new object.
	* @param others (options) array of objects to load into the Resource's fields.
	* @return The new resource.
	*/
	public Resource newResource(Resource container, String id, Object[] others)
	{
		return new BaseUserEdit(id);
	}

	/**
	* Construct a new resource, from an XML element.
	* @param container The Resource that is the container for the new resource (may be null).
	* @param element The XML.
	* @return The new resource from the XML.
	*/
	public Resource newResource(Resource container, Element element)
	{
		return new BaseUserEdit(element);
	}

	/**
	* Construct a new resource from another resource of the same type.
	* @param container The Resource that is the container for the new resource (may be null).
	* @param other The other resource.
	* @return The new resource as a copy of the other.
	*/
	public Resource newResource(Resource container, Resource other)
	{
		return new BaseUserEdit((User) other);
	}

	/**
	* Construct a new continer given just an id.
	* @param id The id for the new object.
	* @return The new containe Resource.
	*/
	public Edit newContainerEdit(String ref)
	{
		return null;
	}

	/**
	* Construct a new container resource, from an XML element.
	* @param element The XML.
	* @return The new container resource.
	*/
	public Edit newContainerEdit(Element element)
	{
		return null;
	}

	/**
	* Construct a new container resource, as a copy of another
	* @param other The other contianer to copy.
	* @return The new container resource.
	*/
	public Edit newContainerEdit(Resource other)
	{
		return null;
	}

	/**
	* Construct a new resource given just an id.
	* @param container The Resource that is the container for the new resource (may be null).
	* @param id The id for the new object.
	* @param others (options) array of objects to load into the Resource's fields.
	* @return The new resource.
	*/
	public Edit newResourceEdit(Resource container, String id, Object[] others)
	{
		BaseUserEdit e = new BaseUserEdit(id);
		e.activate();
		return e;
	}

	/**
	* Construct a new resource, from an XML element.
	* @param container The Resource that is the container for the new resource (may be null).
	* @param element The XML.
	* @return The new resource from the XML.
	*/
	public Edit newResourceEdit(Resource container, Element element)
	{
		BaseUserEdit e = new BaseUserEdit(element);
		e.activate();
		return e;
	}

	/**
	* Construct a new resource from another resource of the same type.
	* @param container The Resource that is the container for the new resource (may be null).
	* @param other The other resource.
	* @return The new resource as a copy of the other.
	*/
	public Edit newResourceEdit(Resource container, Resource other)
	{
		BaseUserEdit e = new BaseUserEdit((User) other);
		e.activate();
		return e;
	}

	/**
	* Collect the fields that need to be stored outside the XML (for the resource).
	* @return An array of field values to store in the record outside the XML (for the resource).
	*/
	public Object[] storageFields(Resource r)
	{
		return null;
	}

	/*******************************************************************************
	* CacheRefresher implementation (no container)
	*******************************************************************************/

	/**
	* Get a new value for this key whose value has already expired in the cache.
	* @param key The key whose value has expired and needs to be refreshed.
	* @param oldValue The old exipred value of the key.
	* @param event The event which triggered this refresh.
	* @return a new value for use in the cache for this key; if null, the entry will be removed.
	*/
	public Object refresh(Object key, Object oldValue, Event event)
	{
		// key is a reference, but our storage wants an id
		String id = userId((String) key);

		// get whatever we have from storage for the cache for this vale
		User user = m_storage.get(id);

		if (m_logger.isDebugEnabled())
			m_logger.debug(this +".refresh(): " + key + " : " + id);

		return user;

	} // refresh

} // BaseUserDirectoryService

/**********************************************************************************
*
* $Header: /cvs/sakai/legacy-component/src/java/org/sakaiproject/component/legacy/user/BaseUserDirectoryService.java,v 1.15 2004/06/22 03:13:55 ggolden Exp $
*
**********************************************************************************/
