/**********************************************************************************
*
* $Header: /cvs/sakai/service/src/java/org/sakaiproject/service/common/agent/AgentManager.java,v 1.5 2004/06/22 03:14:40 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

package org.sakaiproject.service.common.agent;

import java.util.Collection;

import org.sakaiproject.exception.IdUnusedException;
import org.sakaiproject.exception.PermissionException;
import org.sakaiproject.service.common.id.Id;
import org.sakaiproject.service.common.shared.Type;
import org.sakaiproject.service.common.shared.TypeIterator;

/**
* <p>AgentManager is responsible for access to all Agents.</p>
* <p>One type of Agent is defined: Users.</p>
* 
* @author University of Michigan, CHEF Software Development Team
* @version $Revision: 1.5 $
*/
public interface AgentManager
{
	/**
	 * Check permissions for createUser().
	 * @return true if the current User is allowed to perform this action, false if not.
	 */
	boolean allowCreateUser();

	/**
	 * Create a new User, auto-generating an user Id.
	 * @return A new User, ready to modify.
	 * @exception PermissionException if the current User does not have permission to perform this action.
	 */
	User createUser() throws PermissionException;

	/**
	 * Access a User.
	 * @param id The User Id.
	 * @return The User with this Id.
	 * @exception IdUnusedException if no User is defined with this Id.
	 */
	User getUser(Id id) throws IdUnusedException;

	/**
	 * Access the User that has this email address.
	 * @param email The email address string.
	 * @return The User with this email address.
	 * @exception IdUnusedException if no User is defined with this email address.
	 */
	User getUserByEmail(String email) throws IdUnusedException;

	/**
	 * Access all Users.
	 * @return An AgentIterator listing all the Users.
	 */
	AgentIterator getUsers();

	/**
	 * Search for Users using this search type and criteria.
	 * @param searchCriteria The search criteria, in a form suitable to the search type.
	 * @param UserSearchType The Type of search to perform
	 * @return An AgentIterator listing all the Users found.
	 */
	AgentIterator getUsersBySearch(Object searchCriteria, Type userSearchType);

	/**
	 * Access all the Users Search types supported by this service.
	 * @return A TypeIterator listing all the Users Search Types.
	 */
	TypeIterator getUserSearchTypes();

	/**
	 * Access the anonymous User.
	 * @return the anonymous User.
	 */
	User getAnonymousUser();

	/**
	 * Create and return a new AgentIterator based on a complete set of Agents in an array.
	 * @param agents The array of Agent objects.
	 * @return a new AgentIterator over the Agents in the collection.
	 */
	AgentIterator getIteratorOnArray(Agent[] agents);

	/**
	 * Create and return a new AgentIterator based on a complete set of Agents in a Collection.
	 * @param useagentsrs The collection of Agent objects.
	 * @return a new AgentIterator over the Agents in the collection.
	 */
	AgentIterator getIteratorOnCollection(Collection agents);
}

/**********************************************************************************
*
* $Header: /cvs/sakai/service/src/java/org/sakaiproject/service/common/agent/AgentManager.java,v 1.5 2004/06/22 03:14:40 ggolden Exp $
*
**********************************************************************************/
