/**********************************************************************************
*
* $Header: /cvs/sakai/service/src/java/org/sakaiproject/service/common/authentication/AuthenticationManager.java,v 1.2 2004/06/22 03:14:42 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

package org.sakaiproject.service.common.authentication;

import org.sakaiproject.exception.IdUnusedException;
import org.sakaiproject.exception.PermissionException;
import org.sakaiproject.service.common.agent.User;
import org.sakaiproject.service.common.id.Id;

/**
 * <p>AuthenticationManager Models the Authentication process for Sakai.</p>
 *
 * @author University of Michigan, CHEF Software Development Team
 * @version $Revision: 1.2 $
 */
public interface AuthenticationManager
{
	/**
	 * Attempt an authentication based on no additional information.
	 * The result will be either an authentication user associated with this processing cycle's session, or not.
	 * @return true if there's an authenticated user, false if not.
	 */
	boolean authenticate();

	/**
	 * Attempt an authentication based on provided username and password.
	 * The result will be either an authentication user associated with this processing cycle's session, or not.
	 * @return true if there's an authenticated user, false if not.
	 */
	boolean authenticate(String username, String password);

	/**
	 * Get the User authenticated with the current processing cycle.
	 * If the user is not authenticated, the anon. Use is returned.
	 * @return The User authenticated with the current processing cycle.
	 */
	User getAuthenticatedUser();

	/**
	 * Remove any authentication associated with this processing cycle's session.
	 */
	void clearAuthentication();

	/**
	 * Check permissions for createAuthentication().
	 * @param agentId The agent id.
	 * @return true if the current user is allowed to perform this action, false if not.
	 */
	boolean allowCreateAuthentication(Id agentId);

	/**
	 * Access the authentication record for the agent with this id.
	 * @param agentId The agent id.
	 * @return The authentication record.
	 * @throws UnusedException if this agent id has no authentication record.
	 */
	Authentication getAuthentication(Id agentId) throws IdUnusedException;

	/**
	 * Access the authentication record for the given user id string.
	 * @param uid The user id string.
	 * @return The authentication record.
	 * @throws UnusedException if there is no authentication record that has this uid.
	 */
	Authentication getAuthenticationByUid(String uid) throws IdUnusedException;

	/**
	 * Create a new authentication record for this agent id, or return the existing one if already defined.
	 * @param agentId The agent id.
	 * @return The authentication record.
	 * @throws PermissionException if the current user does not have permission to perform this action.
	 */
	Authentication createAuthentication(Id agentId) throws PermissionException;
}

/**********************************************************************************
*
* $Header: /cvs/sakai/service/src/java/org/sakaiproject/service/common/authentication/AuthenticationManager.java,v 1.2 2004/06/22 03:14:42 ggolden Exp $
*
**********************************************************************************/
