/**********************************************************************************
*
* $Header: /cvs/sakai/service/src/java/org/sakaiproject/service/common/shared/Resource.java,v 1.5 2004/06/22 03:14:50 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

package org.sakaiproject.service.common.shared;

import org.sakaiproject.exception.PermissionException;
import org.sakaiproject.exception.VersionException;
import org.sakaiproject.service.common.id.Id;

/**
* <p>Resource is the interface for user created artifacts stored in and accessable from a Sakai system.</p>
* <p>A resource has a unique id which never changes and is never reused.</p>
* <p>A resource has a simple displayName string, and a longer description string.</p>
* <p>A resource has a public URL, and an internal reference string.</p>
* <p>A resource has a Type.</p>
* <p>A resource has a set of name=value properties which can hold any values associated with the resource.
* These are grouped into sets, each with a Type.</p>
* <p>A resource has a version.  This is automatically updated with each committed changed.
* It can be checked, checked for current, or checked against another Version to see if that version is current.</p>
* <p>A resource may be updatable.  Changes made to a Resource object are private to the object until the Resource
* is updated.  Update may fail if the version of the Resource is not current.  Update may fail because the end-user
* does not have sufficient permissions to perform the action.</p>
* <p>A resource may be deleted, if the end-user has permission to do so.</p>
* 
* @author University of Michigan, Sakai Software Development Team
* @version $Revision: 1.5 $
*/
public interface Resource extends Comparable
{
	/**
	 * Add a new type of properties to the resource, if one does not yet exist.
	 * @param propertiesType The new property type.
	 * @return The properties of this type.
	 */
	Properties addPropertiesType(Type propertiesType);

	/**
	 * Check if the current user is allowed to delete this resource.
	 * @return true if the user is allowed to delete this resource, false if not.
	 */	
	boolean allowDelete();
	
	/**
	 * Check if the current user is allowed to update this resource.
	 * @return true if the user is allowed to update this resource, false if not.
	 */
	boolean allowUpdate();

	/**
	 * Delete this object.  Once deleted, it should be nulled out and not used.
	 * @throws PermissionsException if the current user does not have permission to perform this activity.
	 */
	void delete() throws PermissionException;

	/**
	 * Access the resource's descriptive text.
	 * @return The resource's descriptive text.
	 */
	String getDescription();

	/**
	 * Access the display name for the resource.
	 * @return A simple display name string that represents the resource.
	 */
	String getDisplayName();

	/**
	 * Access the unique permanent id of the resource.
	 * @return The resource's unique permanent id.
	 */
	Id getId();

	/**
	 * Access all the resource's properties.
	 * @return A properties iterator listing each typed property set.
	 */
	PropertiesIterator getProperties();

	/**
	 * Access the resource's properties of the specified type.
	 * @param propertiesType The type.
	 * @return The resource's properties of the specified type, if any,
	 * or an empty properties if there are none of this type
	 */
	Properties getPropertiesByType(Type propertiesType);

	/**
	 * Access the types of properties available for this resource.
	 * @return An iterator (TypeIterator) of types of properties defined for this resource.
	 */
	TypeIterator getPropertyTypes();

	/**
	 * Access the resource's internal reference.
	 * @return The resource's internal reference.
	 */
	String getReference();

	/**
	 * Access the resource's type.
	 * @return The resource's type.
	 */
	Type getType();

	/**
	 * Access the resource's URL.
	 * @return The resource's URL.
	 */
	String getUrl();

	/**
	 * Access the version of this resource.
	 * @return The version of this resource.
	 */
	Version getVersion();
	
	/**
	 * Check if this resource is the current version.
	 * @return true if it is the current version, false if not.
	 */
	boolean isCurrentVersion();

	/**
	 * Check if this version is the current version for this resource.
	 * @param version The version to check.
	 * @return true if it is the current version, false if not.
	 */
	boolean isCurrentVersion(Version version);

	/**
	 * Remove the properties of this type from the resource.
	 * @param propertiesType The property type.
	 */
	void removePropertiesType(Type propertiesType);

	/**
	 * Set the resource's descriptive text.
	 * @param description The resource's descriptive text.
	 */
	void setDescription(String description);

	/**
	 * Set the resource's display name.
	 * @param displayName The new display name for the resource.
	 */
	void setDisplayName(String displayName);

	/**
	 * Set the resource's type.
	 * @param type The new type for the resource.
	 */
	void setType(Type type);

	/**
	 * Cause changes made to this resource to be committed to the system, if this resource's version is current.
	 * @throws VersionException if the update failed due to this resource's version not being current.
	 * @throws PermissionsException if the current user does not have permission to perform this activity.
	 */
	void update() throws VersionException, PermissionException;

	/**
	 * Cause changes made to this resource to be committed to the system.
	 * If the version of this object is not current, this object's values will replace the values in the system anyway.
	 * @throws PermissionsException if the current user does not have permission to perform this activity.
	 */
	void updateIgnoreVersion() throws PermissionException;
}

/**********************************************************************************
*
* $Header: /cvs/sakai/service/src/java/org/sakaiproject/service/common/shared/Resource.java,v 1.5 2004/06/22 03:14:50 ggolden Exp $
*
**********************************************************************************/
