/**********************************************************************************
*
* $Header: /cvs/sakai/service/src/java/org/sakaiproject/service/framework/state/StateService.java,v 1.6 2004/06/22 03:14:51 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

package org.sakaiproject.service.framework.state;

import java.util.List;

/**
* <p>The StateManagerService is a service that manages SessionState information.
* Each SessionState is identified by a unique key in this service.  The SessionState
* is composed of name - value sets of attributes, stored under the key by the service.</p>
* <p>Attribute values placed into SessionStates may be automatically removed, for example when
* a SessionState expires in some way, or by means of some other automatic service.  These value
* objects can be notified of their placement into and out of the SessionState.  Objects that wish
* to receive this notification implement the SessionStateBindingListener interface.  This is based
* on and similar to the HttpSessionBindingListener mechanism.</p>
* <p>To support the "current" routines; the ability to get a session state based
* on the current http session, this service must be installed properly into the
* procssing of each http request.  At the start of the request, the HttpSession
* of the request must be passed into the setCurrentContext() method.  At the end
* of the request, clearCurrentContext() is called.</p>
* @version $Revision: 1.6 $
* @author <a href="mailto:ggolden@apache.org">Glenn R. Golden</a>
*/
public interface StateService
{
	/**
	 * Access the named attribute of the keyed state.
	 * @param key The state key.
	 * @param name The attribute name.
	 * @return The named attribute value of the keyed state.
	 */
	Object getAttribute(String key, String name);

	/**
	 * Set the named state attribute of the keyed state with the provided object.
	 * @param key The state key.
	 * @param name The attribute name.
	 * @param value The new value of the attribute (any object type).
	 */
	void setAttribute(String key, String name, Object value);

	/**
	 * Remove the named state attribute of the keyed state, if it exists.
	 * @param key The state key.
	 * @param name The attribute name.
	 */
	void removeAttribute(String key, String name);

	/**
	 * Remove all state attribute of the keyed state.
	 * @param key The state key.
	 */
	void clear(String key);

	/**
	 * Access an array of all names of attributes stored in the keyed state.
	 * @param key The state key.
	 * @return An array of all names of attributes stored in the keyed state.
	 */
	List getAttributeNames(String key);

	/**
	 * Access an SessionState object with the given key.
	 * @param key The SessionState key.
	 * @return an SessionState object with the given key.
	 */
	SessionState getSessionState(String key);

	/**
	 * Access the SessionState object associated with the current request's http session.
	 * The session id is used as the key.
	 * @return an SessionState object associated with the current request's http session.
	 */
	SessionState getCurrentSessionState();

	/**
	 * Access the SessionState object associated with the current request's http session with the given key.
	 * @param key The string to add to the session id to form the SessionState key.
	 * @return an SessionState object associated with the current request's http session with the given key.
	 */
	SessionState getCurrentSessionState(String key);

	/**
	* Retire, forget about and clean up all states that start with the given key.
	 * @param keyStart The beginning of the key of the states to clean up.
	 */
	void retireState(String keyStart);
}

/**********************************************************************************
*
* $Header: /cvs/sakai/service/src/java/org/sakaiproject/service/framework/state/StateService.java,v 1.6 2004/06/22 03:14:51 ggolden Exp $
*
**********************************************************************************/
