/**********************************************************************************
*
* $Header: /cvs/sakai/service/src/java/org/sakaiproject/service/legacy/presence/PresenceService.java,v 1.6 2004/06/22 03:14:49 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

// package
package org.sakaiproject.service.legacy.presence;

// imports
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.sakaiproject.service.legacy.resource.Resource;
import org.sakaiproject.service.framework.session.UsageSession;

/**
* <p>A PresenceService keeps track of a session's presence at various location 
* in the system.  Presence is refreshed and lasts a short period of time between
* refreshed.  When not refreshed in time, presence times out and becomes absence.
* Absence can also be called for explicitely.</p>
* <p>Presence Locations may be any strings.</p>
* 
* @author University of Michigan, CHEF Software Development Team
* @version $Revision: 1.6 $
* @see org.chefproject.core.UsageSession
*/
public interface PresenceService
{
	/** This string can be used to find the service in the service manager. */
	public static final String SERVICE_NAME = PresenceService.class.getName();

	/** This string starts the references to resources in this service. */
	public static final String REFERENCE_ROOT = Resource.SEPARATOR + "presence";

	/** Name for the event of establishing presence at a location. */
	public static final String EVENT_PRESENCE = "pres.begin";

	/** Name for the event of ending presence at a location. */
	public static final String EVENT_ABSENCE = "pres.end";

	/**
	* Form a presence reference from a location id
	* @param id the location id.
	* @return A presence reference based on a location id.
	*/
	public String presenceReference(String id);

	/**
	 * Construct a location id from site, page, and tool.
	 * @param site The site id.
	 * @param page The page id (optional).
	 * @param tool The tool id (optional).
	 * @return a Location Id.
	 */
	String locationId(String site, String page, String tool);

	/**
	* Form a description for the location.
	* @param location The presence location.
	* @return A description for the location.
	*/
	String getLocationDescription(String location);

	/**
	 * Parse the location into three strings: site, page, tool- possibly null.
	 * @return the parsed site, page and tool strings from the location id.
	 */
	String[] parseLocation(String location);

	/**
	* Refresh the activity of a session.
	* @param session The session object.
	*/
	public void setActiveSession(UsageSession session);

	/**
	* Establish or refresh the presence of a session in a location.
	* @param session The session object.
	* @param locationId A presence location id.
	*/
	public void setPresence(UsageSession session, String locationId);

	/**
	* Remove the presence of a session from all locations.
	* @param session The session object.
	*/
	public void removePresence(UsageSession session);

	/**
	* Remove the presence of a session from a location.
	* @param session The session object.
	* @param locationId A presence location id.
	*/
	public void removePresence(UsageSession session, String locationId);

	/**
	 * Perform any presence maintainance that required being in context for the current session.
	 * This may involve final closure of "closed" sessions.
	 * It also includes sponsering the session as active, unless the auto parameter is set.
	 * @param auto true if this context is in response to automatic (courier delivered) activity,
	 * false if it's considered a user initiated event.
	 */
	public void doInContextMaintenance(boolean auto);

	/**
	* Access a List of sessions (UsageSession) now present in a location.
	* @param locationId A presence location id.
	* @return The a List of sessions (UsageSession) now present in the location (may be empty).
	*/
	public List getPresence(String locationId);

	/**
	* Access a List of all location ids (String).
	* @return The List of all location ids (Strings) (may be empty).
	*/
	public List getLocations();

	/**
	* Get the sessions with current presence, as a List of UsageSession objects.
	* @return a Set (UsageSession) containing all sessions with current presence.
	*/
	public Set getPresentSessions();

	/**
	* Get the sessions with current presence by server, as a Map keyed by server id of UsageSession objects.
	* @return a Map (server id -> Set of UsageSession) containing all sessions with current presence by server.
	*/
	public Map getPresentServers();

	/**
	* Access the time (seconds) we will wait for a request from a session before we consider the session lost.
	* @return the time (seconds) we will wait for a request from a session before we consider the session lost.
	*/
	public int getTimeout();

}	// PresenceService

/**********************************************************************************
*
* $Header: /cvs/sakai/service/src/java/org/sakaiproject/service/legacy/presence/PresenceService.java,v 1.6 2004/06/22 03:14:49 ggolden Exp $
*
**********************************************************************************/
