/**********************************************************************************
*
* $Header: /cvs/sakai/util/src/java/org/sakaiproject/util/SessionUtil.java,v 1.5 2004/06/22 03:15:26 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

package org.sakaiproject.util;

import org.sakaiproject.service.framework.current.CurrentService;
import org.sakaiproject.service.framework.log.cover.Logger;
import org.sakaiproject.service.framework.session.UsageSession;
import org.sakaiproject.service.framework.state.SessionState;
import org.sakaiproject.service.framework.state.StateService;

/**
 * <p>SessionUtil extracts some session keeping logic for use in places where we don't want
 * startup dependencies on the UsageSessionService.  The service impls. and these impls. can share
 * this logic.</p>
 * 
 * @author University of Michigan, Sakai Software Development Team
 * @version $Revision: 1.5 $
 */
public class SessionUtil
{
	protected static final String KEY = SessionUtil.class.getName();

	/**
	 * Establish a UsageSession for the current user (session) (or by passing in null, clear it).
	 * @param stateService The StateService component.
	 * @param session The UsageSession for the current user (session), or null to clear it.
	 */
	static public void setSession(StateService stateService, UsageSession session)
	{
		// get session state used to store the current session
		SessionState state = stateService.getCurrentSessionState(KEY);
		if (state != null)
		{
			if (session == null)
			{
				state.removeAttribute(KEY);
			}
			else
			{
				state.setAttribute(KEY, session);
			}
		}
		else
		{
			Logger.warn("SessionUtil.setSession: missing SessionState for session");
		}
	}

	/**
	 * Access the current session, if any.
	 * @param stateService The StateService component.
	 * @return The current UsageSession.
	 */
	static public UsageSession getSession(StateService stateService)
	{
		// get session state used to store the current session
		SessionState state = stateService.getCurrentSessionState(KEY);
		if (state != null)
		{
			return (UsageSession) state.getAttribute(KEY);
		}
		else
		{
			Logger.warn("SessionUtil.getSession: missing SessionState for session");
		}
		
		return null;
	}

	/**
	 * Access the current session's id, if any.
	 * @param stateService The StateService component.
	 * @return The current UsageSession.
	 */
	static public String getSessionId(StateService stateService)
	{
		// get session state used to store the current session
		SessionState state = stateService.getCurrentSessionState(KEY);
		if (state != null)
		{
			UsageSession us = (UsageSession) state.getAttribute(KEY);
			if (us != null)
			{
				return us.getId();
			}
		}
		
		return null;
	}

	/**
	 * Establish a "user id" for this thread.
	 * @param currentService The CurrentService component.
	 * @param id The "user" id for this thread.
	 */
	static public void setThreadSessionUserId(CurrentService currentService, String id)
	{
		currentService.setInThread(KEY, id);
	}

	/**
	 * Access the thread "user" id, if any.
	 * @param currentService The CurrentService component.
	 * @return The "user" id for this thread, if any.
	 */
	static public String getThreadSessionUserId(CurrentService currentService)
	{
		String userId = (String) currentService.getInThread(KEY);
		return userId;
	}
}

/**********************************************************************************
*
* $Header: /cvs/sakai/util/src/java/org/sakaiproject/util/SessionUtil.java,v 1.5 2004/06/22 03:15:26 ggolden Exp $
*
**********************************************************************************/
