/**********************************************************************************
*
* $Header: /cvs/sakai/util/src/java/org/sakaiproject/util/storage/BaseDbSingleStorage.java,v 1.6 2004/06/22 03:15:25 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

// package
package org.sakaiproject.util.storage;

// import
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Hashtable;
import java.util.List;
import java.util.Stack;
import java.util.Vector;

import org.sakaiproject.service.framework.log.cover.Log;
import org.sakaiproject.service.legacy.resource.Edit;
import org.sakaiproject.service.legacy.resource.Resource;
import org.sakaiproject.service.framework.sql.SqlReader;
import org.sakaiproject.service.framework.sql.SqlService;
import org.sakaiproject.util.Filter;
import org.sakaiproject.util.Validator;
import org.sakaiproject.util.Xml;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
* <p>BaseDbSingleStorage is a class that stores Resources (of some type) in a database,
* provides locked access, and generally implements a services "storage" class.  The
* service's storage class can extend this to provide covers to turn Resource and
* Edit into something more type specific to the service.</p>
*
* Note: the methods here are all "id" based, with the following assumptions:
* - just the Resource Id field is enough to distinguish one Resource from another
* - a resource's reference is based on no more than the resource id
* - a resource's id cannot change.
*
* @author University of Michigan, CHEF Software Development Team
* @version $Revision: 1.6 $
*/
public class BaseDbSingleStorage
{
	/** Table name for resource records. */
	protected String m_resourceTableName = null;

	/** The field in the resource table that holds the resource id. */
	protected String m_resourceTableIdField = null;

	/** The additional field names in the resource table that go between the two ids and the xml */
	protected String[] m_resourceTableOtherFields = null;

	/** The xml tag name for the element holding each actual resource entry. */
	protected String m_resourceEntryTagName = null;

	/** If true, we do our locks in the remote database, otherwise we do them here. */
	protected boolean m_locksAreInDb = true;

	/** The StorageUser to callback for new Resource and Edit objects. */
	protected StorageUser m_user = null;

	/** Locks, keyed by reference, holding Connections
	(or, if locks are done locally, holding an Edit). */
	protected Hashtable m_locks = null;

	/** If set, we treat reasource ids as case insensitive. */
	protected boolean m_caseInsensitive = false;

	/** Injected (by constructor) SqlService. */
	protected SqlService m_sql = null;

	/**
	* Construct.
	* @param resourceTableName Table name for resources.
	* @param resourceTableIdField The field in the resource table that holds the id.
	* @param resourceTableOtherFields The other fields in the resource table
	* (between the two id and the xml fields).
	* @param locksInDb If true, we do our locks in the remote database, otherwise we do them here.
	* @param resourceEntryName The xml tag name for the element holding each actual resource entry.
	* @param user The StorageUser class to call back for creation of Resource and Edit objects.
	* @param sqlService The SqlService.
	*/
	public BaseDbSingleStorage(String resourceTableName, String resourceTableIdField,
		String[] resourceTableOtherFields, boolean locksInDb, String resourceEntryName, StorageUser user,
		SqlService sqlService)
	{
		m_resourceTableName = resourceTableName;
		m_resourceTableIdField = resourceTableIdField;
		m_resourceTableOtherFields = resourceTableOtherFields;
		m_locksAreInDb = locksInDb;
		m_resourceEntryTagName = resourceEntryName;
		m_user = user;
		m_sql = sqlService;

	}	// BaseDbSingleStorage
		
	/**
	* Open and be ready to read / write.
	*/
	public void open()
	{
		// setup for locks
		m_locks = new Hashtable();

	}   // open

	/**
	* Close.
	*/
	public void close()
	{
		if (!m_locks.isEmpty())
		{
			Log.warn("chef", this + ".close(): locks remain!");
			// %%%
		}
		m_locks.clear();
		m_locks = null;

	}   // close

	/**
	* Read one Resource from xml
	* @param xml An string containing the xml which describes the resource.
	* @return The Resource object created from the xml.
	*/
	protected Resource readResource(String xml)
	{
		try
		{
			// read the xml
			Document doc =  Xml.readDocumentFromString(xml);

			// verify the root element
			Element root = doc.getDocumentElement();
			if (!root.getTagName().equals(m_resourceEntryTagName))
			{
				Log.warn("chef", this + ".readResource(): not = "
							+ m_resourceEntryTagName + " : " + root.getTagName());
				return null;
			}

			// re-create a resource
			Resource entry = m_user.newResource(null, root);

			return entry;
		}
		catch (Exception e)
		{
			Log.debug("chef", this + ".readResource(): ", e);
			return null;
		}

	}   // readResource

	/**
	* Check if a Resource by this id exists.
	* @param id The id.
	* @return true if a Resource by this id exists, false if not.
	*/
	public boolean checkResource(String id)
	{
		// just see if the record exists
		String sql =
				"select " + m_resourceTableIdField + " from " + m_resourceTableName
			+   " where ( " + m_resourceTableIdField + " = '"
			+	Validator.escapeSql(caseId(id)) + "' )";

		List ids = m_sql.dbRead(sql);

		return (!ids.isEmpty());

	}	// check

	/**
	* Get the Resource with this id, or null if not found.
	* @param id The id.
	* @return The Resource with this id, or null if not found.
	*/
	public Resource getResource(String id)
	{
		Resource entry = null;

		// get the user from the db
		String sql =
				"select XML from " + m_resourceTableName
			+   " where ( " + m_resourceTableIdField + " = '"
			+	Validator.escapeSql(caseId(id)) + "' )";
		List xml = m_sql.dbRead(sql);
		if (!xml.isEmpty())
		{
			// create the Resource from the db xml
			entry = readResource((String) xml.get(0));
		}

		return entry;

	}   // getResource

	/**
	* Get all Resources.
	* @return The list of all Resources.
	*/
	public List getAllResources()
	{
		List all = new Vector();

		// read all users from the db
		String sql = "select XML from " + m_resourceTableName;
		// %%%	+	"order by " + m_resourceTableOrderField + " asc";

		List xml = m_sql.dbRead(sql);

		// process all result xml into user objects
		if (!xml.isEmpty())
		{
			for (int i = 0; i < xml.size(); i++)
			{
				Resource entry = readResource((String) xml.get(i));
				if (entry != null) all.add(entry);
			}
		}

		return all;

	}   // getAllResources

	/**
	* Get selected Resources, filtered by a test on the id field
	* @param filter A filter to select what gets returned.
	* @return The list of selected Resources.
	*/
	public List getSelectedResources(final Filter filter)
	{
		List all = new Vector();

		// read all users from the db
		String sql = "select " + m_resourceTableIdField + ", XML from " + m_resourceTableName;
		// %%%	+	"order by " + m_resourceTableOrderField + " asc";

		List xml = m_sql.dbRead(sql, null,
				new SqlReader()
				{
					public Object readSqlResultRecord(ResultSet result)
					{
						try
						{
							// read the id m_resourceTableIdField
							String id =  result.getString(1);
							
							// read the xml
							String xml = result.getString(2);
	
							if (!filter.accept(caseId(id))) return null;
							
							return xml;
						}
						catch (SQLException ignore) { return null;}
					}
				} );

		// process all result xml into user objects
		if (!xml.isEmpty())
		{
			for (int i = 0; i < xml.size(); i++)
			{
				Resource entry = readResource((String) xml.get(i));
				if (entry != null) all.add(entry);
			}
		}

		return all;

	}   // getSelectedResources

	/**
	* Get selected Resources, using a supplied where clause
	* @param sqlWhere The SQL where clause.
	* @return The list of selected Resources.
	*/
	public List getSelectedResourcesWhere(String sqlWhere)
	{
		List all = new Vector();

		// read all users from the db
		String sql = "select XML from " + m_resourceTableName + " " + sqlWhere;
		// %%%	+	"order by " + m_resourceTableOrderField + " asc";

		List xml = m_sql.dbRead(sql);

		// process all result xml into user objects
		if (!xml.isEmpty())
		{
			for (int i = 0; i < xml.size(); i++)
			{
				Resource entry = readResource((String) xml.get(i));
				if (entry != null) all.add(entry);
			}
		}

		return all;

	}   // getSelectedResourcesWhere

	/**
	* Add a new Resource with this id.
	* @param id The id.
	* @param others Other fields for the newResource call
	* @return The locked Resource object with this id, or null if the id is in use.
	*/
	public Edit putResource(String id, Object[] others)
	{
		// create one with just the id, and perhaps some other fields as well
		Resource entry = m_user.newResource(null, id, others);

		Object[] fields = m_user.storageFields(entry);

		// form the XML and SQL for the insert
		Document doc = Xml.createDocument();
		entry.toXml(doc, new Stack());
		String xml = Xml.writeDocumentToString(doc);
		String statement =
				"insert into " + m_resourceTableName
			+	insertFields(m_resourceTableIdField, m_resourceTableOtherFields, "XML")
			+   " values ( "
			+   "'" + Validator.escapeSql(caseId(entry.getId())) + "',"
			+	valuesParams(m_resourceTableOtherFields)
			+   " ? )";

		// process the insert
		boolean ok = m_sql.dbWrite(statement, fields, xml);

		// if this failed, assume a key conflict (i.e. id in use)
		if (!ok) return null;

		// now get a lock on the record for edit
		Edit edit = editResource(id);
		if (edit == null)
		{
			Log.warn("chef", this + ".putResource(): didn't get a lock!");
			return null;
		}
		
		return edit;

	}   // putResource

	/**
	* Get a lock on the Resource with this id, or null if a lock cannot be gotten.
	* @param id The user id.
	* @return The locked Resource with this id, or null if this records cannot be locked.
	*/
	public Edit editResource(String id)
	{
		Edit edit = null;

		if (m_locksAreInDb)
		{
			// read the record and get a lock on it (non blocking)
			String statement =
				"select XML from " + m_resourceTableName
				+   " where ( " + m_resourceTableIdField + " = '"
				+	Validator.escapeSql(caseId(id)) + "' )"
				+	" for update nowait";
			StringBuffer result = new StringBuffer();
			Connection lock = m_sql.dbReadLock(statement, result);

			// for missing or already locked...
			if ((lock == null) || (result.length() == 0)) return null;

			// make first a Resource, then an Edit
			Resource entry = readResource(result.toString());
			edit = m_user.newResourceEdit(null, entry);

			// store the lock for this object
			m_locks.put(entry.getReference(), lock);
		}

		// otherwise, get the lock locally
		else
		{
			// get the entry, and check for existence
			Resource entry = getResource(id);
			if (entry == null) return null;

			// we only sync this getting - someone may release a lock out of sync
			synchronized (m_locks)
			{
				// if already locked
				if (m_locks.containsKey(entry.getReference())) return null;
				
				// make the edit from the Resource
				edit = m_user.newResourceEdit(null, entry);

				// store the edit in the locks by reference
				m_locks.put(entry.getReference(), edit);
			}
		}

		return edit;

	}	// editResource

	/**
	* Commit the changes and release the lock.
	* @param user The Edit to commit.
	*/
	public void commitResource(Edit edit)
	{
		Object[] fields = m_user.storageFields(edit);

		// form the SQL statement and the var w/ the XML
		Document doc = Xml.createDocument();
		edit.toXml(doc, new Stack());
		String xml = Xml.writeDocumentToString(doc);
		String statement =
				"update " + m_resourceTableName
			+	" set "
			+ 	updateSet(m_resourceTableOtherFields)
			+   " XML = ?"
			+   " where ( " + m_resourceTableIdField + " = '"
			+	Validator.escapeSql(caseId(edit.getId())) + "' )";

		if (m_locksAreInDb)
		{
			// use this connection that is stored with the lock
			Connection lock = (Connection) m_locks.get(edit.getReference());
			if (lock == null)
			{
				Log.warn("chef", this + ".commitResource(): edit not in locks");
				return;
			}

			// update, commit, release the lock's connection
			m_sql.dbUpdateCommit(statement, fields, xml, lock);
		}

		else
		{
			// just process the update
			m_sql.dbWrite(statement, fields, xml);
		}

		// remove the lock
		m_locks.remove(edit.getReference());

	}	// commitResource

	/**
	* Cancel the changes and release the lock.
	* @param user The Edit to cancel.
	*/
	public void cancelResource(Edit edit)
	{
		if (m_locksAreInDb)
		{
			// use this connection that is stored with the lock
			Connection lock = (Connection) m_locks.get(edit.getReference());
			if (lock == null)
			{
				Log.warn("chef", this + ".cancelResource(): edit not in locks");
				return;
			}

			// rollback and release the lock's connection
			m_sql.dbCancel(lock);
		}

		// release the lock
		m_locks.remove(edit.getReference());

	}	// cancelResource

	/**
	* Remove this (locked) Resource.
	* @param user The Edit to remove.
	*/
	public void removeResource(Edit edit)
	{
		// form the SQL delete statement
		String statement =
				"delete from " + m_resourceTableName
				+   " where ( " + m_resourceTableIdField + " = '"
				+	Validator.escapeSql(caseId(edit.getId())) + "' )";

		if (m_locksAreInDb)
		{
			// use this connection that is stored with the lock
			Connection lock = (Connection) m_locks.get(edit.getReference());
			if (lock == null)
			{
				Log.warn("chef", this + ".removeResource(): edit not in locks");
				return;
			}

			// process the delete statement, commit, and release the lock's connection
			m_sql.dbUpdateCommit(statement, null, null, lock);
		}
		
		else
		{
			// process the delete statement
			m_sql.dbWrite(statement);
		}

		// release the lock
		m_locks.remove(edit.getReference());

	}   // removeResource

	/**
	* Form a string of n question marks with commas, for sql value statements, one for each
	* item in the values array, or an empty string if null.
	* @param values The values to be inserted into the sql statement.
	* @return A sql statement fragment for the values part of an insert, one for each value in the array.
	*/
	protected String valuesParams(String[] fields)
	{
		if ((fields == null) || (fields.length == 0)) return "";
		StringBuffer buf = new StringBuffer();
		for (int i = 0; i < fields.length; i++)
		{
			buf.append(" ?,");
		}
		return buf.toString();

	}	// valuesParams

	/**
	* Form a string of n name=?, for sql update set statements, one for each
	* item in the values array, or an empty string if null.
	* @param values The values to be inserted into the sql statement.
	* @return A sql statement fragment for the values part of an insert, one for each value in the array.
	*/
	protected String updateSet(String[] fields)
	{
		if ((fields == null) || (fields.length == 0)) return "";
		StringBuffer buf = new StringBuffer();
		for (int i = 0; i < fields.length; i++)
		{
			buf.append(fields[i] + " = ?,");
		}
		return buf.toString();

	}	// updateSet

	/**
	* Form a string of (field, field, field), for sql insert statements, one for each
	* item in the fields array, plus one before, and one after.
	* @param before The first field name.
	* @param values The extra field names, in the middle.
	* @param after The last field name.
	* @return A sql statement fragment for the insert fields.
	*/
	protected String insertFields(String before, String[] fields, String after)
	{
		StringBuffer buf = new StringBuffer();
		buf.append(" (");

		buf.append(before);
		buf.append(",");

		if (fields != null)
		{
			for (int i = 0; i < fields.length; i++)
			{
				buf.append(fields[i] + ",");
			}
		}

		buf.append(after);

		buf.append(")");

		return buf.toString();

	}	// insertFields

	/**
	* Fix the case of resource ids to support case insensitive ids if enabled
	* @param The id to fix.
	* @return The id, case modified as needed.
	*/
	protected String caseId(String id)
	{
		if (m_caseInsensitive)
		{
			return id.toLowerCase();
		}

		return id;

	}	// caseId

	/**
	* Enable / disable case insensitive ids.
	* @param setting true to set case insensitivity, false to set case sensitivity.
	*/
	protected void setCaseInsensitivity(boolean setting)
	{
		m_caseInsensitive = setting;

	}	// setCaseInsensitivity

}   // BaseDbSingleStorage

/**********************************************************************************
*
* $Header: /cvs/sakai/util/src/java/org/sakaiproject/util/storage/BaseDbSingleStorage.java,v 1.6 2004/06/22 03:15:25 ggolden Exp $
*
**********************************************************************************/
