/**********************************************************************************
*
* $Header: /cvs/sakai/velocity/src/java/org/sakaiproject/vm/ComponentServlet.java,v 1.10 2004/06/22 03:15:41 ggolden Exp $
*
***********************************************************************************
*
* Copyright (c) 2003, 2004 The Regents of the University of Michigan, Trustees of Indiana University,
*                  Board of Trustees of the Leland Stanford, Jr., University, and The MIT Corporation
* 
* Licensed under the Educational Community License Version 1.0 (the "License");
* By obtaining, using and/or copying this Original Work, you agree that you have read,
* understand, and will comply with the terms and conditions of the Educational Community License.
* You may obtain a copy of the License at:
* 
*      http://cvs.sakaiproject.org/licenses/license_1_0.html
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

package org.sakaiproject.vm;

import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.sakaiproject.service.framework.component.ComponentManager;
import org.sakaiproject.service.framework.config.cover.ServerConfigurationService;
import org.sakaiproject.service.framework.current.cover.CurrentService;
import org.sakaiproject.service.legacy.presence.cover.PresenceService;
import org.sakaiproject.service.framework.session.UsageSession;
import org.sakaiproject.service.framework.session.cover.UsageSessionService;

/**
* <p>ComponentServlet is an HttpServlet that can use Sakai components (by covered locator).<p>
* <p>Extension classes can use the <code>getComponentManager</code> call to access the component manager.</p>
* <p>Registration for "current" (thread based) access to session (...) is handled here, too.</p>
* 
* @author University of Michigan, Sakai Software Development Team
* @version $Revision: 1.10 $
*/
public abstract class ComponentServlet extends HttpServlet
{
	/** Session attribute holding the cross-webapp session. */
	public static final String GLOBAL_SESSION_ATTR = "org.sakaiproject.global.session";

	/**
	 * Access the CHEF Component Manager.
	 * @return the ComponentManager.
	 */
	public ComponentManager getComponentManager()
	{
		return org.sakaiproject.service.framework.component.cover.ComponentManager.getInstance();
	}

	/**
	 * Override service, adding CurrentService binding of the HttpSession into the current (request) thread.
	 */
	protected void service(HttpServletRequest req, HttpServletResponse resp) throws ServletException, java.io.IOException
	{
		try
		{
			if (setup(req, resp))
				return;

			super.service(req, resp);
		}
		finally
		{
			// clear out any current access bindings
			CurrentService.clearInThread();
		}
	}

	/**
	 * Send a redirect so our "top" ends up at the url, via javascript.
	 * @param url The redirect url
	 */
	protected void sendTopRedirect(HttpServletResponse resp, String url)
	{
		try
		{
			resp.setContentType("text/html; charset=UTF-8");
			PrintWriter out = resp.getWriter();
			out.println(
				"<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">");
			out.println("<html><head></head><body>");
			out.println("<script type=\"text/javascript\" language=\"JavaScript\">");
			out.println("if (parent)\n\tif (parent.courier)\n" + "\t{\n\t\tparent.location.replace('" + url + "');\n\t}\n");
			out.println(
				"if (parent)\n\tif(parent.parent)\n\t\tif (parent.parent.courier)\n"
					+ "\t\t{\n\t\t\tparent.parent.location.replace('"
					+ url
					+ "');\n\t\t}\n");
			out.println("</script>");
			out.println("</body></html>");
		}
		catch (IOException e)
		{
		}
		//		resp.sendRedirect(ServerConfigurationService.getLoggedOutUrl());
	}

	/**
	 * Send a redirect so our parent ends up at the url, via javascript.
	 * @param url The redirect url
	 */
	protected void sendParentRedirect(HttpServletResponse resp, String url)
	{
		try
		{
			resp.setContentType("text/html; charset=UTF-8");
			PrintWriter out = resp.getWriter();
			out.println(
				"<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">");
			out.println("<html><head></head><body>");
			out.println("<script type=\"text/javascript\" language=\"JavaScript\">");
			out.println("if (parent)\n" + "{\n\tparent.location.replace('" + url + "');\n}\n");
			out.println("</script>");
			out.println("</body></html>");
		}
		catch (IOException e)
		{
		}
		//		resp.sendRedirect(ServerConfigurationService.getLoggedOutUrl());
	}

	// set standard no-cache headers
	protected void setNoCacheHeaders(HttpServletResponse resp)
	{
		resp.setContentType("text/html; charset=UTF-8");
		resp.addDateHeader("Expires", System.currentTimeMillis() - (1000L * 60L * 60L * 24L * 365L));
		resp.addDateHeader("Last-Modified", System.currentTimeMillis());
		resp.addHeader("Cache-Control", "no-store, no-cache, must-revalidate, max-age=0, post-check=0, pre-check=0");
		resp.addHeader("Pragma", "no-cache");
	}

	/**
	 * Setup for a service request.
	 * @param req The servlet request.
	 * @param resp The servlet response.
	 * @return true if we have handled things, false if more processing is required.
	 */
	protected boolean setup(HttpServletRequest req, HttpServletResponse resp)
	{
		// set the session up for current thread access
		// Note: component:org.sakaiproject.component.framework.componentsServlet has matching code.
		HttpSession hsession = (HttpSession) req.getAttribute(GLOBAL_SESSION_ATTR);
		if (hsession == null)
		{
			hsession = req.getSession(true);
		}

		CurrentService.setInThread(HttpSession.class.getName(), hsession);
		CurrentService.setInThread(HttpServletRequest.class.getName(), req);

		// debug(req, hsession);

		// grab the session so we can check for a closed one
		UsageSession session = UsageSessionService.getSession();

		// let the presence service catch up with this session -
		// deciding if this event is not user initiated based on the presence of "auto=courier" in the req.
		PresenceService.doInContextMaintenance("courier".equals(req.getParameter("auto")));

		// if the session was just closed, redirect
		if ((session != null) && session.isClosed())
		{
			sendTopRedirect(resp, ServerConfigurationService.getLoggedOutUrl());
			return true;
		}

		return false;
	}

	//	private void debug(HttpServletRequest req, HttpSession hsession)
	//	{
	//		log(
	//			":Request: "
	//				+ req.getContextPath()
	//				+ " "
	//				+ req.getPathInfo()
	//				+ " "
	//				+ req.getQueryString()
	//				+ " globalSession: "
	//				+ hsession.getId());
	//
	//		Cookie[] c = req.getCookies();
	//		if (c != null)
	//		{
	//			for (int i = 0; i < c.length; i++)
	//			{
	//				log(
	//					":Request: "
	//						+ req.getContextPath()
	//						+ " "
	//						+ req.getPathInfo()
	//						+ " cookie: "
	//						+ c[i].getName()
	//						+ " = "
	//						+ c[i].getValue());
	//			}
	//		}
	//	}
}

/**********************************************************************************
*
* $Header: /cvs/sakai/velocity/src/java/org/sakaiproject/vm/ComponentServlet.java,v 1.10 2004/06/22 03:15:41 ggolden Exp $
*
**********************************************************************************/
